@@ -1,26 +1,22 @@
 #![feature(test)]
 #![allow(unused_imports)]
+#![allow(
+    clippy::many_single_char_names,
+    clippy::deref_addrof,
+    clippy::unreadable_literal,
+    clippy::many_single_char_names
+)]
 
 extern crate test;
-#[macro_use(s, azip)]
-extern crate ndarray;
 
-use ndarray::{
-    Array,
-    Axis,
-    Ix,
-    Array1,
-    Array2,
-    Zip,
-};
-use ndarray::{arr0, arr1, arr2};
 use ndarray::ShapeBuilder;
+use ndarray::{arr0, arr1, arr2, azip, s};
+use ndarray::{Array, Array1, Array2, Axis, Ix, Zip};
 
 use test::black_box;
 
 #[bench]
-fn iter_sum_1d_regular(bench: &mut test::Bencher)
-{
+fn iter_sum_1d_regular(bench: &mut test::Bencher) {
     let a = Array::<i32, _>::zeros(64 * 64);
     let a = black_box(a);
     bench.iter(|| {
@@ -33,8 +29,7 @@ fn iter_sum_1d_regular(bench: &mut test::Bencher)
 }
 
 #[bench]
-fn iter_sum_1d_raw(bench: &mut test::Bencher)
-{
+fn iter_sum_1d_raw(bench: &mut test::Bencher) {
     // this is autovectorized to death (= great performance)
     let a = Array::<i32, _>::zeros(64 * 64);
     let a = black_box(a);
@@ -48,8 +43,7 @@ fn iter_sum_1d_raw(bench: &mut test::Bencher)
 }
 
 #[bench]
-fn iter_sum_2d_regular(bench: &mut test::Bencher)
-{
+fn iter_sum_2d_regular(bench: &mut test::Bencher) {
     let a = Array::<i32, _>::zeros((64, 64));
     let a = black_box(a);
     bench.iter(|| {
@@ -62,8 +56,7 @@ fn iter_sum_2d_regular(bench: &mut test::Bencher)
 }
 
 #[bench]
-fn iter_sum_2d_by_row(bench: &mut test::Bencher)
-{
+fn iter_sum_2d_by_row(bench: &mut test::Bencher) {
     let a = Array::<i32, _>::zeros((64, 64));
     let a = black_box(a);
     bench.iter(|| {
@@ -78,8 +71,7 @@ fn iter_sum_2d_by_row(bench: &mut test::Bencher)
 }
 
 #[bench]
-fn iter_sum_2d_raw(bench: &mut test::Bencher)
-{
+fn iter_sum_2d_raw(bench: &mut test::Bencher) {
     // this is autovectorized to death (= great performance)
     let a = Array::<i32, _>::zeros((64, 64));
     let a = black_box(a);
@@ -93,8 +85,7 @@ fn iter_sum_2d_raw(bench: &mut test::Bencher)
 }
 
 #[bench]
-fn iter_sum_2d_cutout(bench: &mut test::Bencher)
-{
+fn iter_sum_2d_cutout(bench: &mut test::Bencher) {
     let a = Array::<i32, _>::zeros((66, 66));
     let av = a.slice(s![1..-1, 1..-1]);
     let a = black_box(av);
@@ -108,8 +99,7 @@ fn iter_sum_2d_cutout(bench: &mut test::Bencher)
 }
 
 #[bench]
-fn iter_sum_2d_cutout_by_row(bench: &mut test::Bencher)
-{
+fn iter_sum_2d_cutout_by_row(bench: &mut test::Bencher) {
     let a = Array::<i32, _>::zeros((66, 66));
     let av = a.slice(s![1..-1, 1..-1]);
     let a = black_box(av);
@@ -125,8 +115,7 @@ fn iter_sum_2d_cutout_by_row(bench: &mut test::Bencher)
 }
 
 #[bench]
-fn iter_sum_2d_cutout_outer_iter(bench: &mut test::Bencher)
-{
+fn iter_sum_2d_cutout_outer_iter(bench: &mut test::Bencher) {
     let a = Array::<i32, _>::zeros((66, 66));
     let av = a.slice(s![1..-1, 1..-1]);
     let a = black_box(av);
@@ -142,8 +131,7 @@ fn iter_sum_2d_cutout_outer_iter(bench: &mut test::Bencher)
 }
 
 #[bench]
-fn iter_sum_2d_transpose_regular(bench: &mut test::Bencher)
-{
+fn iter_sum_2d_transpose_regular(bench: &mut test::Bencher) {
     let mut a = Array::<i32, _>::zeros((64, 64));
     a.swap_axes(0, 1);
     let a = black_box(a);
@@ -157,8 +145,7 @@ fn iter_sum_2d_transpose_regular(bench: &mut test::Bencher)
 }
 
 #[bench]
-fn iter_sum_2d_transpose_by_row(bench: &mut test::Bencher)
-{
+fn iter_sum_2d_transpose_by_row(bench: &mut test::Bencher) {
     let mut a = Array::<i32, _>::zeros((64, 64));
     a.swap_axes(0, 1);
     let a = black_box(a);
@@ -174,116 +161,85 @@ fn iter_sum_2d_transpose_by_row(bench: &mut test::Bencher)
 }
 
 #[bench]
-fn sum_2d_regular(bench: &mut test::Bencher)
-{
+fn sum_2d_regular(bench: &mut test::Bencher) {
     let a = Array::<i32, _>::zeros((64, 64));
     let a = black_box(a);
-    bench.iter(|| {
-        a.sum()
-    });
+    bench.iter(|| a.sum());
 }
 
 #[bench]
-fn sum_2d_cutout(bench: &mut test::Bencher)
-{
+fn sum_2d_cutout(bench: &mut test::Bencher) {
     let a = Array::<i32, _>::zeros((66, 66));
     let av = a.slice(s![1..-1, 1..-1]);
     let a = black_box(av);
-    bench.iter(|| {
-        a.sum()
-    });
+    bench.iter(|| a.sum());
 }
 
 #[bench]
-fn sum_2d_float(bench: &mut test::Bencher)
-{
+fn sum_2d_float(bench: &mut test::Bencher) {
     let a = Array::<f32, _>::zeros((64, 64));
     let a = black_box(a.view());
-    bench.iter(|| {
-        a.sum()
-    });
+    bench.iter(|| a.sum());
 }
 
 #[bench]
-fn sum_2d_float_cutout(bench: &mut test::Bencher)
-{
+fn sum_2d_float_cutout(bench: &mut test::Bencher) {
     let a = Array::<f32, _>::zeros((66, 66));
     let av = a.slice(s![1..-1, 1..-1]);
     let a = black_box(av);
-    bench.iter(|| {
-        a.sum()
-    });
+    bench.iter(|| a.sum());
 }
 
 #[bench]
-fn sum_2d_float_t_cutout(bench: &mut test::Bencher)
-{
+fn sum_2d_float_t_cutout(bench: &mut test::Bencher) {
     let a = Array::<f32, _>::zeros((66, 66));
     let av = a.slice(s![1..-1, 1..-1]).reversed_axes();
     let a = black_box(av);
-    bench.iter(|| {
-        a.sum()
-    });
+    bench.iter(|| a.sum());
 }
 
 #[bench]
-fn fold_sum_i32_2d_regular(bench: &mut test::Bencher)
-{
+fn fold_sum_i32_2d_regular(bench: &mut test::Bencher) {
     let a = Array::<i32, _>::zeros((64, 64));
-    bench.iter(|| {
-        a.fold(0, |acc, &x| acc + x)
-    });
+    bench.iter(|| a.fold(0, |acc, &x| acc + x));
 }
 
 #[bench]
-fn fold_sum_i32_2d_cutout(bench: &mut test::Bencher)
-{
+fn fold_sum_i32_2d_cutout(bench: &mut test::Bencher) {
     let a = Array::<i32, _>::zeros((66, 66));
     let av = a.slice(s![1..-1, 1..-1]);
     let a = black_box(av);
-    bench.iter(|| {
-        a.fold(0, |acc, &x| acc + x)
-    });
+    bench.iter(|| a.fold(0, |acc, &x| acc + x));
 }
 
 #[bench]
-fn fold_sum_i32_2d_stride(bench: &mut test::Bencher)
-{
+fn fold_sum_i32_2d_stride(bench: &mut test::Bencher) {
     let a = Array::<i32, _>::zeros((64, 128));
     let av = a.slice(s![.., ..;2]);
     let a = black_box(av);
-    bench.iter(|| {
-        a.fold(0, |acc, &x| acc + x)
-    });
+    bench.iter(|| a.fold(0, |acc, &x| acc + x));
 }
 
 #[bench]
-fn fold_sum_i32_2d_transpose(bench: &mut test::Bencher)
-{
+fn fold_sum_i32_2d_transpose(bench: &mut test::Bencher) {
     let a = Array::<i32, _>::zeros((64, 64));
     let a = a.t();
-    bench.iter(|| {
-        a.fold(0, |acc, &x| acc + x)
-    });
+    bench.iter(|| a.fold(0, |acc, &x| acc + x));
 }
 
 #[bench]
-fn fold_sum_i32_2d_cutout_transpose(bench: &mut test::Bencher)
-{
+fn fold_sum_i32_2d_cutout_transpose(bench: &mut test::Bencher) {
     let a = Array::<i32, _>::zeros((66, 66));
     let mut av = a.slice(s![1..-1, 1..-1]);
     av.swap_axes(0, 1);
     let a = black_box(av);
-    bench.iter(|| {
-        a.fold(0, |acc, &x| acc + x)
-    });
+    bench.iter(|| a.fold(0, |acc, &x| acc + x));
 }
 
 const ADD2DSZ: usize = 64;
 
 #[bench]
-fn add_2d_regular(bench: &mut test::Bencher)
-{
+fn add_2d_regular(bench: &mut test::Bencher) {
     let mut a = Array::<i32, _>::zeros((ADD2DSZ, ADD2DSZ));
     let b = Array::<i32, _>::zeros((ADD2DSZ, ADD2DSZ));
     let bv = b.view();
@@ -293,8 +249,7 @@ fn add_2d_regular(bench: &mut test::Bencher)
 }
 
 #[bench]
-fn add_2d_zip(bench: &mut test::Bencher)
-{
+fn add_2d_zip(bench: &mut test::Bencher) {
     let mut a = Array::<i32, _>::zeros((ADD2DSZ, ADD2DSZ));
     let b = Array::<i32, _>::zeros((ADD2DSZ, ADD2DSZ));
     bench.iter(|| {
@@ -303,32 +258,25 @@ fn add_2d_zip(bench: &mut test::Bencher)
 }
 
 #[bench]
-fn add_2d_alloc(bench: &mut test::Bencher)
-{
+fn add_2d_alloc(bench: &mut test::Bencher) {
     let a = Array::<i32, _>::zeros((ADD2DSZ, ADD2DSZ));
     let b = Array::<i32, _>::zeros((ADD2DSZ, ADD2DSZ));
-    bench.iter(|| {
-        &a + &b
+    bench.iter(|| &a + &b);
+}
+
+#[bench]
+fn add_2d_zip_alloc(bench: &mut test::Bencher) {
+    let a = Array::<i32, _>::zeros((ADD2DSZ, ADD2DSZ));
+    let b = Array::<i32, _>::zeros((ADD2DSZ, ADD2DSZ));
+    bench.iter(|| unsafe {
+        let mut c = Array::uninitialized(a.dim());
+        azip!((&a in &a, &b in &b, c in &mut c) *c = a + b);
+        c
     });
 }
 
 #[bench]
-fn add_2d_zip_alloc(bench: &mut test::Bencher)
-{
-    let a = Array::<i32, _>::zeros((ADD2DSZ, ADD2DSZ));
-    let b = Array::<i32, _>::zeros((ADD2DSZ, ADD2DSZ));
-    bench.iter(|| {
-        unsafe {
-            let mut c = Array::uninitialized(a.dim());
-            azip!(a, b, mut c in { *c = a + b });
-            c
-        }
-    });
-}
-
-#[bench]
-fn add_2d_assign_ops(bench: &mut test::Bencher)
-{
+fn add_2d_assign_ops(bench: &mut test::Bencher) {
     let mut a = Array::<i32, _>::zeros((ADD2DSZ, ADD2DSZ));
     let b = Array::<i32, _>::zeros((ADD2DSZ, ADD2DSZ));
     let bv = b.view();
@@ -340,8 +288,7 @@ fn add_2d_assign_ops(bench: &mut test::Bencher)
 }
 
 #[bench]
-fn add_2d_cutout(bench: &mut test::Bencher)
-{
+fn add_2d_cutout(bench: &mut test::Bencher) {
     let mut a = Array::<i32, _>::zeros((ADD2DSZ + 2, ADD2DSZ + 2));
     let mut acut = a.slice_mut(s![1..-1, 1..-1]);
     let b = Array::<i32, _>::zeros((ADD2DSZ, ADD2DSZ));
@@ -352,8 +299,7 @@ fn add_2d_cutout(bench: &mut test::Bencher)
 }
 
 #[bench]
-fn add_2d_zip_cutout(bench: &mut test::Bencher)
-{
+fn add_2d_zip_cutout(bench: &mut test::Bencher) {
     let mut a = Array::<i32, _>::zeros((ADD2DSZ + 2, ADD2DSZ + 2));
     let mut acut = a.slice_mut(s![1..-1, 1..-1]);
     let b = Array::<i32, _>::zeros((ADD2DSZ, ADD2DSZ));
@@ -363,8 +309,8 @@ fn add_2d_zip_cutout(bench: &mut test::Bencher)
 }
 
 #[bench]
-fn add_2d_cutouts_by_4(bench: &mut test::Bencher)
-{
+#[allow(clippy::identity_op)]
+fn add_2d_cutouts_by_4(bench: &mut test::Bencher) {
     let mut a = Array::<i32, _>::zeros((64 * 1, 64 * 1));
     let b = Array::<i32, _>::zeros((64 * 1, 64 * 1));
     let chunksz = (4, 4);
@@ -376,8 +322,8 @@ fn add_2d_cutouts_by_4(bench: &mut test::Bencher)
 }
 
 #[bench]
-fn add_2d_cutouts_by_16(bench: &mut test::Bencher)
-{
+#[allow(clippy::identity_op)]
+fn add_2d_cutouts_by_16(bench: &mut test::Bencher) {
     let mut a = Array::<i32, _>::zeros((64 * 1, 64 * 1));
     let b = Array::<i32, _>::zeros((64 * 1, 64 * 1));
     let chunksz = (16, 16);
@@ -389,8 +335,8 @@ fn add_2d_cutouts_by_16(bench: &mut test::Bencher)
 }
 
 #[bench]
-fn add_2d_cutouts_by_32(bench: &mut test::Bencher)
-{
+#[allow(clippy::identity_op)]
+fn add_2d_cutouts_by_32(bench: &mut test::Bencher) {
     let mut a = Array::<i32, _>::zeros((64 * 1, 64 * 1));
     let b = Array::<i32, _>::zeros((64 * 1, 64 * 1));
     let chunksz = (32, 32);
@@ -402,8 +348,7 @@ fn add_2d_cutouts_by_32(bench: &mut test::Bencher)
 }
 
 #[bench]
-fn add_2d_broadcast_1_to_2(bench: &mut test::Bencher)
-{
+fn add_2d_broadcast_1_to_2(bench: &mut test::Bencher) {
     let mut a = Array2::<i32>::zeros((ADD2DSZ, ADD2DSZ));
     let b = Array1::<i32>::zeros(ADD2DSZ);
     let bv = b.view();
@@ -413,8 +358,7 @@ fn add_2d_broadcast_1_to_2(bench: &mut test::Bencher)
 }
 
 #[bench]
-fn add_2d_broadcast_0_to_2(bench: &mut test::Bencher)
-{
+fn add_2d_broadcast_0_to_2(bench: &mut test::Bencher) {
     let mut a = Array::<i32, _>::zeros((ADD2DSZ, ADD2DSZ));
     let b = Array::<i32, _>::zeros(());
     let bv = b.view();
@@ -426,52 +370,40 @@ fn add_2d_broadcast_0_to_2(bench: &mut test::Bencher)
 #[bench]
 fn scalar_toowned(bench: &mut test::Bencher) {
     let a = Array::<f32, _>::zeros((64, 64));
-    bench.iter(|| {
-        a.to_owned()
-    });
+    bench.iter(|| a.to_owned());
 }
 
 #[bench]
 fn scalar_add_1(bench: &mut test::Bencher) {
     let a = Array::<f32, _>::zeros((64, 64));
     let n = 1.;
-    bench.iter(|| {
-        &a + n
-    });
+    bench.iter(|| &a + n);
 }
 
 #[bench]
 fn scalar_add_2(bench: &mut test::Bencher) {
     let a = Array::<f32, _>::zeros((64, 64));
     let n = 1.;
-    bench.iter(|| {
-        n + &a
-    });
+    bench.iter(|| n + &a);
 }
 
 #[bench]
 fn scalar_sub_1(bench: &mut test::Bencher) {
     let a = Array::<f32, _>::zeros((64, 64));
     let n = 1.;
-    bench.iter(|| {
-        &a - n
-    });
+    bench.iter(|| &a - n);
 }
 
 #[bench]
 fn scalar_sub_2(bench: &mut test::Bencher) {
     let a = Array::<f32, _>::zeros((64, 64));
     let n = 1.;
-    bench.iter(|| {
-        n - &a
-    });
+    bench.iter(|| n - &a);
 }
 
-
 // This is for comparison with add_2d_broadcast_0_to_2
 #[bench]
-fn add_2d_0_to_2_iadd_scalar(bench: &mut test::Bencher)
-{
+fn add_2d_0_to_2_iadd_scalar(bench: &mut test::Bencher) {
     let mut a = Array::<i32, _>::zeros((ADD2DSZ, ADD2DSZ));
     let n = black_box(0);
     bench.iter(|| {
@@ -480,8 +412,7 @@ fn add_2d_0_to_2_iadd_scalar(bench: &mut test::Bencher)
 }
 
 #[bench]
-fn add_2d_strided(bench: &mut test::Bencher)
-{
+fn add_2d_strided(bench: &mut test::Bencher) {
     let mut a = Array::<i32, _>::zeros((ADD2DSZ, ADD2DSZ * 2));
     let mut a = a.slice_mut(s![.., ..;2]);
     let b = Array::<i32, _>::zeros((ADD2DSZ, ADD2DSZ));
@@ -492,8 +423,7 @@ fn add_2d_strided(bench: &mut test::Bencher)
 }
 
 #[bench]
-fn add_2d_regular_dyn(bench: &mut test::Bencher)
-{
+fn add_2d_regular_dyn(bench: &mut test::Bencher) {
     let mut a = Array::<i32, _>::zeros(&[ADD2DSZ, ADD2DSZ][..]);
     let b = Array::<i32, _>::zeros(&[ADD2DSZ, ADD2DSZ][..]);
     let bv = b.view();
@@ -503,8 +433,7 @@ fn add_2d_regular_dyn(bench: &mut test::Bencher)
 }
 
 #[bench]
-fn add_2d_strided_dyn(bench: &mut test::Bencher)
-{
+fn add_2d_strided_dyn(bench: &mut test::Bencher) {
     let mut a = Array::<i32, _>::zeros(&[ADD2DSZ, ADD2DSZ * 2][..]);
     let mut a = a.slice_mut(s![.., ..;2]);
     let b = Array::<i32, _>::zeros(&[ADD2DSZ, ADD2DSZ][..]);
@@ -514,10 +443,8 @@ fn add_2d_strided_dyn(bench: &mut test::Bencher)
     });
 }
 
-
 #[bench]
-fn add_2d_zip_strided(bench: &mut test::Bencher)
-{
+fn add_2d_zip_strided(bench: &mut test::Bencher) {
     let mut a = Array::<i32, _>::zeros((ADD2DSZ, ADD2DSZ * 2));
     let mut a = a.slice_mut(s![.., ..;2]);
     let b = Array::<i32, _>::zeros((ADD2DSZ, ADD2DSZ));
@@ -527,8 +454,7 @@ fn add_2d_zip_strided(bench: &mut test::Bencher)
 }
 
 #[bench]
-fn add_2d_one_transposed(bench: &mut test::Bencher)
-{
+fn add_2d_one_transposed(bench: &mut test::Bencher) {
     let mut a = Array::<i32, _>::zeros((ADD2DSZ, ADD2DSZ));
     a.swap_axes(0, 1);
     let b = Array::<i32, _>::zeros((ADD2DSZ, ADD2DSZ));
@@ -538,8 +464,7 @@ fn add_2d_one_transposed(bench: &mut test::Bencher)
 }
 
 #[bench]
-fn add_2d_zip_one_transposed(bench: &mut test::Bencher)
-{
+fn add_2d_zip_one_transposed(bench: &mut test::Bencher) {
     let mut a = Array::<i32, _>::zeros((ADD2DSZ, ADD2DSZ));
     a.swap_axes(0, 1);
     let b = Array::<i32, _>::zeros((ADD2DSZ, ADD2DSZ));
@@ -549,8 +474,7 @@ fn add_2d_zip_one_transposed(bench: &mut test::Bencher)
 }
 
 #[bench]
-fn add_2d_both_transposed(bench: &mut test::Bencher)
-{
+fn add_2d_both_transposed(bench: &mut test::Bencher) {
     let mut a = Array::<i32, _>::zeros((ADD2DSZ, ADD2DSZ));
     a.swap_axes(0, 1);
     let mut b = Array::<i32, _>::zeros((ADD2DSZ, ADD2DSZ));
@@ -561,8 +485,7 @@ fn add_2d_both_transposed(bench: &mut test::Bencher)
 }
 
 #[bench]
-fn add_2d_zip_both_transposed(bench: &mut test::Bencher)
-{
+fn add_2d_zip_both_transposed(bench: &mut test::Bencher) {
     let mut a = Array::<i32, _>::zeros((ADD2DSZ, ADD2DSZ));
     a.swap_axes(0, 1);
     let mut b = Array::<i32, _>::zeros((ADD2DSZ, ADD2DSZ));
@@ -573,8 +496,7 @@ fn add_2d_zip_both_transposed(bench: &mut test::Bencher)
 }
 
 #[bench]
-fn add_2d_f32_regular(bench: &mut test::Bencher)
-{
+fn add_2d_f32_regular(bench: &mut test::Bencher) {
     let mut a = Array::<f32, _>::zeros((ADD2DSZ, ADD2DSZ));
     let b = Array::<f32, _>::zeros((ADD2DSZ, ADD2DSZ));
     let bv = b.view();
@@ -586,8 +508,7 @@ fn add_2d_f32_regular(bench: &mut test::Bencher)
 const ADD3DSZ: usize = 16;
 
 #[bench]
-fn add_3d_strided(bench: &mut test::Bencher)
-{
+fn add_3d_strided(bench: &mut test::Bencher) {
     let mut a = Array::<i32, _>::zeros((ADD3DSZ, ADD3DSZ, ADD3DSZ * 2));
     let mut a = a.slice_mut(s![.., .., ..;2]);
     let b = Array::<i32, _>::zeros(a.dim());
@@ -598,8 +519,7 @@ fn add_3d_strided(bench: &mut test::Bencher)
 }
 
 #[bench]
-fn add_3d_strided_dyn(bench: &mut test::Bencher)
-{
+fn add_3d_strided_dyn(bench: &mut test::Bencher) {
     let mut a = Array::<i32, _>::zeros(&[ADD3DSZ, ADD3DSZ, ADD3DSZ * 2][..]);
     let mut a = a.slice_mut(s![.., .., ..;2]);
     let b = Array::<i32, _>::zeros(a.dim());
@@ -609,12 +529,10 @@ fn add_3d_strided_dyn(bench: &mut test::Bencher)
     });
 }
 
-
 const ADD1D_SIZE: usize = 64 * 64;
 
 #[bench]
-fn add_1d_regular(bench: &mut test::Bencher)
-{
+fn add_1d_regular(bench: &mut test::Bencher) {
     let mut a = Array::<f32, _>::zeros(ADD1D_SIZE);
     let b = Array::<f32, _>::zeros(a.dim());
     bench.iter(|| {
@@ -623,8 +541,7 @@ fn add_1d_regular(bench: &mut test::Bencher)
 }
 
 #[bench]
-fn add_1d_strided(bench: &mut test::Bencher)
-{
+fn add_1d_strided(bench: &mut test::Bencher) {
     let mut a = Array::<f32, _>::zeros(ADD1D_SIZE * 2);
     let mut av = a.slice_mut(s![..;2]);
     let b = Array::<f32, _>::zeros(av.dim());
@@ -634,8 +551,7 @@ fn add_1d_strided(bench: &mut test::Bencher)
 }
 
 #[bench]
-fn iadd_scalar_2d_regular(bench: &mut test::Bencher)
-{
+fn iadd_scalar_2d_regular(bench: &mut test::Bencher) {
     let mut a = Array::<f32, _>::zeros((ADD2DSZ, ADD2DSZ));
     bench.iter(|| {
         a += 1.;
@@ -643,8 +559,7 @@ fn iadd_scalar_2d_regular(bench: &mut test::Bencher)
 }
 
 #[bench]
-fn iadd_scalar_2d_strided(bench: &mut test::Bencher)
-{
+fn iadd_scalar_2d_strided(bench: &mut test::Bencher) {
     let mut a = Array::<f32, _>::zeros((ADD2DSZ, ADD2DSZ * 2));
     let mut a = a.slice_mut(s![.., ..;2]);
     bench.iter(|| {
@@ -653,8 +568,7 @@ fn iadd_scalar_2d_strided(bench: &mut test::Bencher)
 }
 
 #[bench]
-fn iadd_scalar_2d_regular_dyn(bench: &mut test::Bencher)
-{
+fn iadd_scalar_2d_regular_dyn(bench: &mut test::Bencher) {
     let mut a = Array::<f32, _>::zeros(vec![ADD2DSZ, ADD2DSZ]);
     bench.iter(|| {
         a += 1.;
@@ -662,8 +576,7 @@ fn iadd_scalar_2d_regular_dyn(bench: &mut test::Bencher)
 }
 
 #[bench]
-fn iadd_scalar_2d_strided_dyn(bench: &mut test::Bencher)
-{
+fn iadd_scalar_2d_strided_dyn(bench: &mut test::Bencher) {
     let mut a = Array::<f32, _>::zeros(vec![ADD2DSZ, ADD2DSZ * 2]);
     let mut a = a.slice_mut(s![.., ..;2]);
     bench.iter(|| {
@@ -672,19 +585,17 @@ fn iadd_scalar_2d_strided_dyn(bench: &mut test::Bencher)
 }
 
 #[bench]
-fn scaled_add_2d_f32_regular(bench: &mut test::Bencher)
-{
+fn scaled_add_2d_f32_regular(bench: &mut test::Bencher) {
     let mut av = Array::<f32, _>::zeros((ADD2DSZ, ADD2DSZ));
     let bv = Array::<f32, _>::zeros((ADD2DSZ, ADD2DSZ));
-    let scalar = 3.1415926535;
+    let scalar = std::f32::consts::PI;
     bench.iter(|| {
         av.scaled_add(scalar, &bv);
     });
 }
 
 #[bench]
-fn assign_scalar_2d_corder(bench: &mut test::Bencher)
-{
+fn assign_scalar_2d_corder(bench: &mut test::Bencher) {
     let a = Array::zeros((ADD2DSZ, ADD2DSZ));
     let mut a = black_box(a);
     let s = 3.;
@@ -692,8 +603,7 @@ fn assign_scalar_2d_corder(bench: &mut test::Bencher)
 }
 
 #[bench]
-fn assign_scalar_2d_cutout(bench: &mut test::Bencher)
-{
+fn assign_scalar_2d_cutout(bench: &mut test::Bencher) {
     let mut a = Array::zeros((66, 66));
     let a = a.slice_mut(s![1..-1, 1..-1]);
     let mut a = black_box(a);
@@ -702,8 +612,7 @@ fn assign_scalar_2d_cutout(bench: &mut test::Bencher)
 }
 
 #[bench]
-fn assign_scalar_2d_forder(bench: &mut test::Bencher)
-{
+fn assign_scalar_2d_forder(bench: &mut test::Bencher) {
     let mut a = Array::zeros((ADD2DSZ, ADD2DSZ));
     a.swap_axes(0, 1);
     let mut a = black_box(a);
@@ -712,16 +621,14 @@ fn assign_scalar_2d_forder(bench: &mut test::Bencher)
 }
 
 #[bench]
-fn assign_zero_2d_corder(bench: &mut test::Bencher)
-{
+fn assign_zero_2d_corder(bench: &mut test::Bencher) {
     let a = Array::zeros((ADD2DSZ, ADD2DSZ));
     let mut a = black_box(a);
     bench.iter(|| a.fill(0.))
 }
 
 #[bench]
-fn assign_zero_2d_cutout(bench: &mut test::Bencher)
-{
+fn assign_zero_2d_cutout(bench: &mut test::Bencher) {
     let mut a = Array::zeros((66, 66));
     let a = a.slice_mut(s![1..-1, 1..-1]);
     let mut a = black_box(a);
@@ -729,8 +636,7 @@ fn assign_zero_2d_cutout(bench: &mut test::Bencher)
 }
 
 #[bench]
-fn assign_zero_2d_forder(bench: &mut test::Bencher)
-{
+fn assign_zero_2d_forder(bench: &mut test::Bencher) {
     let mut a = Array::zeros((ADD2DSZ, ADD2DSZ));
     a.swap_axes(0, 1);
     let mut a = black_box(a);
@@ -738,26 +644,35 @@ fn assign_zero_2d_forder(bench: &mut test::Bencher)
 }
 
 #[bench]
-fn bench_iter_diag(bench: &mut test::Bencher)
-{
+fn bench_iter_diag(bench: &mut test::Bencher) {
     let a = Array::<f32, _>::zeros((1024, 1024));
-    bench.iter(|| for elt in a.diag() { black_box(elt); })
+    bench.iter(|| {
+        for elt in a.diag() {
+            black_box(elt);
+        }
+    })
 }
 
 #[bench]
-fn bench_row_iter(bench: &mut test::Bencher)
-{
+fn bench_row_iter(bench: &mut test::Bencher) {
     let a = Array::<f32, _>::zeros((1024, 1024));
     let it = a.row(17);
-    bench.iter(|| for elt in it.clone() { black_box(elt); })
+    bench.iter(|| {
+        for elt in it {
+            black_box(elt);
+        }
+    })
 }
 
 #[bench]
-fn bench_col_iter(bench: &mut test::Bencher)
-{
+fn bench_col_iter(bench: &mut test::Bencher) {
     let a = Array::<f32, _>::zeros((1024, 1024));
     let it = a.column(17);
-    bench.iter(|| for elt in it.clone() { black_box(elt); })
+    bench.iter(|| {
+        for elt in it {
+            black_box(elt);
+        }
+    })
 }
 
 macro_rules! mat_mul {
@@ -780,7 +695,7 @@ macro_rules! mat_mul {
     }
 }
 
-mat_mul!{mat_mul_f32, f32,
+mat_mul! {mat_mul_f32, f32,
     (m004, 4, 4, 4)
     (m007, 7, 7, 7)
     (m008, 8, 8, 8)
@@ -794,7 +709,7 @@ mat_mul!{mat_mul_f32, f32,
     (mix10000, 128, 10000, 128)
 }
 
-mat_mul!{mat_mul_f64, f64,
+mat_mul! {mat_mul_f64, f64,
     (m004, 4, 4, 4)
     (m007, 7, 7, 7)
     (m008, 8, 8, 8)
@@ -808,7 +723,7 @@ mat_mul!{mat_mul_f64, f64,
     (mix10000, 128, 10000, 128)
 }
 
-mat_mul!{mat_mul_i32, i32,
+mat_mul! {mat_mul_i32, i32,
     (m004, 4, 4, 4)
     (m007, 7, 7, 7)
     (m008, 8, 8, 8)
@@ -820,115 +735,96 @@ mat_mul!{mat_mul_i32, i32,
 }
 
 #[bench]
-fn create_iter_4d(bench: &mut test::Bencher)
-{
+fn create_iter_4d(bench: &mut test::Bencher) {
     let mut a = Array::from_elem((4, 5, 3, 2), 1.0);
     a.swap_axes(0, 1);
     a.swap_axes(2, 1);
     let v = black_box(a.view());
 
-    bench.iter(|| {
-        v.into_iter()
-    });
+    bench.iter(|| v.into_iter());
 }
 
 #[bench]
-fn bench_to_owned_n(bench: &mut test::Bencher)
-{
+fn bench_to_owned_n(bench: &mut test::Bencher) {
     let a = Array::<f32, _>::zeros((32, 32));
     bench.iter(|| a.to_owned());
 }
 
 #[bench]
-fn bench_to_owned_t(bench: &mut test::Bencher)
-{
+fn bench_to_owned_t(bench: &mut test::Bencher) {
     let mut a = Array::<f32, _>::zeros((32, 32));
     a.swap_axes(0, 1);
     bench.iter(|| a.to_owned());
 }
 
 #[bench]
-fn bench_to_owned_strided(bench: &mut test::Bencher)
-{
+fn bench_to_owned_strided(bench: &mut test::Bencher) {
     let a = Array::<f32, _>::zeros((32, 64));
     let a = a.slice(s![.., ..;2]);
     bench.iter(|| a.to_owned());
 }
 #[bench]
-fn equality_i32(bench: &mut test::Bencher)
-{
+fn equality_i32(bench: &mut test::Bencher) {
     let a = Array::<i32, _>::zeros((64, 64));
     let b = Array::<i32, _>::zeros((64, 64));
     bench.iter(|| a == b);
 }
 
 #[bench]
-fn equality_f32(bench: &mut test::Bencher)
-{
+fn equality_f32(bench: &mut test::Bencher) {
     let a = Array::<f32, _>::zeros((64, 64));
     let b = Array::<f32, _>::zeros((64, 64));
     bench.iter(|| a == b);
 }
 
 #[bench]
-fn equality_f32_mixorder(bench: &mut test::Bencher)
-{
+fn equality_f32_mixorder(bench: &mut test::Bencher) {
     let a = Array::<f32, _>::zeros((64, 64));
     let b = Array::<f32, _>::zeros((64, 64).f());
     bench.iter(|| a == b);
 }
 
 #[bench]
-fn dot_f32_16(bench: &mut test::Bencher)
-{
+fn dot_f32_16(bench: &mut test::Bencher) {
     let a = Array::<f32, _>::zeros(16);
     let b = Array::<f32, _>::zeros(16);
     bench.iter(|| a.dot(&b));
 }
 
 #[bench]
-fn dot_f32_20(bench: &mut test::Bencher)
-{
+fn dot_f32_20(bench: &mut test::Bencher) {
     let a = Array::<f32, _>::zeros(20);
     let b = Array::<f32, _>::zeros(20);
     bench.iter(|| a.dot(&b));
 }
 
 #[bench]
-fn dot_f32_32(bench: &mut test::Bencher)
-{
+fn dot_f32_32(bench: &mut test::Bencher) {
     let a = Array::<f32, _>::zeros(32);
     let b = Array::<f32, _>::zeros(32);
     bench.iter(|| a.dot(&b));
 }
 
 #[bench]
-fn dot_f32_256(bench: &mut test::Bencher)
-{
+fn dot_f32_256(bench: &mut test::Bencher) {
     let a = Array::<f32, _>::zeros(256);
     let b = Array::<f32, _>::zeros(256);
     bench.iter(|| a.dot(&b));
 }
 
 #[bench]
-fn dot_f32_1024(bench: &mut test::Bencher)
-{
+fn dot_f32_1024(bench: &mut test::Bencher) {
     let av = Array::<f32, _>::zeros(1024);
     let bv = Array::<f32, _>::zeros(1024);
-    bench.iter(|| {
-        av.dot(&bv)
-    });
+    bench.iter(|| av.dot(&bv));
 }
 
 #[bench]
-fn dot_f32_10e6(bench: &mut test::Bencher)
-{
+fn dot_f32_10e6(bench: &mut test::Bencher) {
     let n = 1_000_000;
     let av = Array::<f32, _>::zeros(n);
     let bv = Array::<f32, _>::zeros(n);
-    bench.iter(|| {
-        av.dot(&bv)
-    });
+    bench.iter(|| av.dot(&bv));
 }
 
 #[bench]
@@ -955,7 +851,9 @@ const MEAN_SUM_N: usize = 127;
 
 fn range_mat(m: Ix, n: Ix) -> Array2<f32> {
     assert!(m * n != 0);
-    Array::linspace(0., (m * n - 1) as f32, m * n).into_shape((m, n)).unwrap()
+    Array::linspace(0., (m * n - 1) as f32, m * n)
+        .into_shape((m, n))
+        .unwrap()
 }
 
 #[bench]
