@@ -1,12 +1,5 @@
-
-extern crate ndarray;
-
 use ndarray::prelude::*;
-use ndarray::{
-    Data,
-    RemoveAxis,
-    Zip,
-};
+use ndarray::{Data, RemoveAxis, Zip};
 
 use std::cmp::Ordering;
 use std::ptr::copy_nonoverlapping;
@@ -34,10 +27,12 @@ impl Permutation {
         for &i in &self.indices {
             match seen.get_mut(i) {
                 None => return false,
-                Some(s) => if *s {
-                    return false;
-                } else {
-                    *s = true;
+                Some(s) => {
+                    if *s {
+                        return false;
+                    } else {
+                        *s = true;
+                    }
                 }
             }
         }
@@ -49,20 +44,23 @@ pub trait SortArray {
     /// ***Panics*** if `axis` is out of bounds.
     fn identity(&self, axis: Axis) -> Permutation;
     fn sort_axis_by<F>(&self, axis: Axis, less_than: F) -> Permutation
-        where F: FnMut(usize, usize) -> bool;
+    where
+        F: FnMut(usize, usize) -> bool;
 }
 
 pub trait PermuteArray {
     type Elem;
     type Dim;
-    fn permute_axis(self, axis: Axis, perm: &Permutation)
-        -> Array<Self::Elem, Self::Dim>
-        where Self::Elem: Clone, Self::Dim: RemoveAxis;
+    fn permute_axis(self, axis: Axis, perm: &Permutation) -> Array<Self::Elem, Self::Dim>
+    where
+        Self::Elem: Clone,
+        Self::Dim: RemoveAxis;
 }
 
 impl<A, S, D> SortArray for ArrayBase<S, D>
-    where S: Data<Elem=A>,
-          D: Dimension,
+where
+    S: Data<Elem = A>,
+    D: Dimension,
 {
     fn identity(&self, axis: Axis) -> Permutation {
         Permutation {
@@ -71,29 +69,33 @@ impl<A, S, D> SortArray for ArrayBase<S, D>
     }
 
     fn sort_axis_by<F>(&self, axis: Axis, mut less_than: F) -> Permutation
-        where F: FnMut(usize, usize) -> bool
+    where
+        F: FnMut(usize, usize) -> bool,
     {
         let mut perm = self.identity(axis);
-        perm.indices.sort_by(move |&a, &b|
+        perm.indices.sort_by(move |&a, &b| {
             if less_than(a, b) {
                 Ordering::Less
             } else if less_than(b, a) {
                 Ordering::Greater
             } else {
                 Ordering::Equal
-            });
+            }
+        });
         perm
     }
 }
 
 impl<A, D> PermuteArray for Array<A, D>
-    where D: Dimension,
+where
+    D: Dimension,
 {
     type Elem = A;
     type Dim = D;
 
     fn permute_axis(self, axis: Axis, perm: &Permutation) -> Array<A, D>
-        where D: RemoveAxis,
+    where
+        D: RemoveAxis,
     {
         let axis = axis;
         let axis_len = self.len_of(axis);
@@ -111,9 +113,7 @@ impl<A, D> PermuteArray for Array<A, D>
                 let perm_i = perm.indices[i];
                 Zip::from(result.index_axis_mut(axis, perm_i))
                     .and(self.index_axis(axis, i))
-                    .apply(|to, from| {
-                        copy_nonoverlapping(from, to, 1)
-                    });
+                    .apply(|to, from| copy_nonoverlapping(from, to, 1));
             }
             // forget moved array elements but not its vec
             let mut old_storage = self.into_raw_vec();
@@ -125,14 +125,11 @@ impl<A, D> PermuteArray for Array<A, D>
     }
 }
 
-
 fn main() {
     let a = Array::linspace(0., 63., 64).into_shape((8, 8)).unwrap();
     let strings = a.map(|x| x.to_string());
 
-    let perm = a.sort_axis_by(Axis(1), |i, j| {
-        a[[i, 0]] > a[[j, 0]]
-    });
+    let perm = a.sort_axis_by(Axis(1), |i, j| a[[i, 0]] > a[[j, 0]]);
     println!("{:?}", perm);
     let b = a.permute_axis(Axis(0), &perm);
     println!("{:?}", b);
