@@ -1,6 +1,9 @@
-
-#[macro_use]
-extern crate ndarray;
+#![allow(
+    clippy::many_single_char_names,
+    clippy::deref_addrof,
+    clippy::unreadable_literal,
+    clippy::many_single_char_names
+)]
 
 use ndarray::prelude::*;
 use ndarray::Zip;
@@ -17,25 +20,23 @@ fn main() {
 
     {
         let a = a.view_mut().reversed_axes();
-        azip!(mut a (a), b (b.t()) in { *a = b });
-
+        azip!((a in a, &b in b.t()) *a = b);
     }
     assert_eq!(a, b);
 
-    azip!(mut a, b, c in { *a = b + c; });
+    azip!((a in &mut a, &b in &b, &c in c) *a = b + c);
     assert_eq!(a, &b + &c);
 
     // sum of each row
     let ax = Axis(0);
     let mut sums = Array::zeros(a.len_of(ax));
-    azip!(mut sums, ref a (a.axis_iter(ax)) in { *sums = a.sum() });
+    azip!((s in &mut sums, a in a.axis_iter(ax)) *s = a.sum());
 
     // sum of each chunk
     let chunk_sz = (2, 2);
     let nchunks = (n / chunk_sz.0, n / chunk_sz.1);
     let mut sums = Array::zeros(nchunks);
-    azip!(mut sums, ref a (a.exact_chunks(chunk_sz)) in { *sums = a.sum() });
-
+    azip!((s in &mut sums, a in a.exact_chunks(chunk_sz)) *s = a.sum());
 
     // Let's imagine we split to parallelize
     {
