@@ -5,107 +5,115 @@
 // <LICENSE-MIT or http://opensource.org/licenses/MIT>, at your
 // option. This file may not be copied, modified, or distributed
 // except according to those terms.
-use serde::{Serialize, Serializer, Deserialize, Deserializer};
-use serde::de::{self, Visitor, SeqAccess, MapAccess};
+use serde::de::{self, MapAccess, SeqAccess, Visitor};
 use serde::ser::{SerializeSeq, SerializeStruct};
+use serde::{Deserialize, Deserializer, Serialize, Serializer};
 
 use std::fmt;
 use std::marker::PhantomData;
 
-use imp_prelude::*;
+use crate::imp_prelude::*;
 
 use super::arraytraits::ARRAY_FORMAT_VERSION;
 use super::Iter;
-use IntoDimension;
+use crate::IntoDimension;
 
 /// Verifies that the version of the deserialized array matches the current
 /// `ARRAY_FORMAT_VERSION`.
 pub fn verify_version<E>(v: u8) -> Result<(), E>
-        where E: de::Error
+where
+    E: de::Error,
 {
     if v != ARRAY_FORMAT_VERSION {
         let err_msg = format!("unknown array version: {}", v);
-        try!(Err(de::Error::custom(err_msg)));
+        Err(de::Error::custom(err_msg))
+    } else {
+        Ok(())
     }
-
-    Ok(())
 }
 
-/// **Requires crate feature `"serde-1"`**
+/// **Requires crate feature `"serde"`**
 impl<I> Serialize for Dim<I>
-    where I: Serialize,
+where
+    I: Serialize,
 {
     fn serialize<Se>(&self, serializer: Se) -> Result<Se::Ok, Se::Error>
-        where Se: Serializer
+    where
+        Se: Serializer,
     {
         self.ix().serialize(serializer)
     }
 }
 
-/// **Requires crate feature `"serde-1"`**
+/// **Requires crate feature `"serde"`**
 impl<'de, I> Deserialize<'de> for Dim<I>
-    where I: Deserialize<'de>,
+where
+    I: Deserialize<'de>,
 {
     fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
-        where D: Deserializer<'de>
+    where
+        D: Deserializer<'de>,
     {
         I::deserialize(deserializer).map(Dim::new)
     }
 }
 
-/// **Requires crate feature `"serde-1"`**
-impl Serialize for IxDyn
-{
+/// **Requires crate feature `"serde"`**
+impl Serialize for IxDyn {
     fn serialize<Se>(&self, serializer: Se) -> Result<Se::Ok, Se::Error>
-        where Se: Serializer
+    where
+        Se: Serializer,
     {
         self.ix().serialize(serializer)
     }
 }
 
-/// **Requires crate feature `"serde-1"`**
-impl<'de> Deserialize<'de> for IxDyn
-{
+/// **Requires crate feature `"serde"`**
+impl<'de> Deserialize<'de> for IxDyn {
     fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
-        where D: Deserializer<'de>
+    where
+        D: Deserializer<'de>,
     {
         let v = Vec::<Ix>::deserialize(deserializer)?;
         Ok(v.into_dimension())
     }
 }
 
-/// **Requires crate feature `"serde-1"`**
+/// **Requires crate feature `"serde"`**
 impl<A, D, S> Serialize for ArrayBase<S, D>
-    where A: Serialize,
-          D: Dimension + Serialize,
-          S: Data<Elem = A>
-
+where
+    A: Serialize,
+    D: Dimension + Serialize,
+    S: Data<Elem = A>,
 {
     fn serialize<Se>(&self, serializer: Se) -> Result<Se::Ok, Se::Error>
-        where Se: Serializer
+    where
+        Se: Serializer,
     {
-        let mut state = try!(serializer.serialize_struct("Array", 3));
-        try!(state.serialize_field("v", &ARRAY_FORMAT_VERSION));
-        try!(state.serialize_field("dim", &self.raw_dim()));
-        try!(state.serialize_field("data", &Sequence(self.iter())));
+        let mut state = serializer.serialize_struct("Array", 3)?;
+        state.serialize_field("v", &ARRAY_FORMAT_VERSION)?;
+        state.serialize_field("dim", &self.raw_dim())?;
+        state.serialize_field("data", &Sequence(self.iter()))?;
         state.end()
     }
 }
 
 // private iterator wrapper
-struct Sequence<'a, A: 'a, D>(Iter<'a, A, D>);
+struct Sequence<'a, A, D>(Iter<'a, A, D>);
 
 impl<'a, A, D> Serialize for Sequence<'a, A, D>
-    where A: Serialize,
-          D: Dimension + Serialize
+where
+    A: Serialize,
+    D: Dimension + Serialize,
 {
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
-        where S: Serializer
+    where
+        S: Serializer,
     {
         let iter = &self.0;
-        let mut seq = try!(serializer.serialize_seq(Some(iter.len())));
+        let mut seq = serializer.serialize_seq(Some(iter.len()))?;
         for elt in iter.clone() {
-            try!(seq.serialize_element(elt));
+            seq.serialize_element(elt)?;
         }
         seq.end()
     }
@@ -124,20 +132,25 @@ enum ArrayField {
 
 impl<S, Di> ArrayVisitor<S, Di> {
     pub fn new() -> Self {
-        ArrayVisitor { _marker_a: PhantomData, _marker_b: PhantomData, }
+        ArrayVisitor {
+            _marker_a: PhantomData,
+            _marker_b: PhantomData,
+        }
     }
 }
 
-static ARRAY_FIELDS: &'static [&'static str] = &["v", "dim", "data"];
+static ARRAY_FIELDS: &[&str] = &["v", "dim", "data"];
 
-/// **Requires crate feature `"serde-1"`**
+/// **Requires crate feature `"serde"`**
 impl<'de, A, Di, S> Deserialize<'de> for ArrayBase<S, Di>
-    where A: Deserialize<'de>,
-          Di: Deserialize<'de> + Dimension,
-          S: DataOwned<Elem = A>
+where
+    A: Deserialize<'de>,
+    Di: Deserialize<'de> + Dimension,
+    S: DataOwned<Elem = A>,
 {
     fn deserialize<D>(deserializer: D) -> Result<ArrayBase<S, Di>, D::Error>
-        where D: Deserializer<'de>
+    where
+        D: Deserializer<'de>,
     {
         deserializer.deserialize_struct("Array", ARRAY_FIELDS, ArrayVisitor::new())
     }
@@ -145,19 +158,21 @@ impl<'de, A, Di, S> Deserialize<'de> for ArrayBase<S, Di>
 
 impl<'de> Deserialize<'de> for ArrayField {
     fn deserialize<D>(deserializer: D) -> Result<ArrayField, D::Error>
-        where D: Deserializer<'de>
+    where
+        D: Deserializer<'de>,
     {
         struct ArrayFieldVisitor;
 
         impl<'de> Visitor<'de> for ArrayFieldVisitor {
             type Value = ArrayField;
 
-            fn expecting(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
+            fn expecting(&self, formatter: &mut fmt::Formatter<'_>) -> fmt::Result {
                 formatter.write_str(r#""v", "dim", or "data""#)
             }
 
             fn visit_str<E>(self, value: &str) -> Result<ArrayField, E>
-                where E: de::Error
+            where
+                E: de::Error,
             {
                 match value {
                     "v" => Ok(ArrayField::Version),
@@ -168,13 +183,17 @@ impl<'de> Deserialize<'de> for ArrayField {
             }
 
             fn visit_bytes<E>(self, value: &[u8]) -> Result<ArrayField, E>
-                where E: de::Error
+            where
+                E: de::Error,
             {
                 match value {
                     b"v" => Ok(ArrayField::Version),
                     b"dim" => Ok(ArrayField::Dim),
                     b"data" => Ok(ArrayField::Data),
-                    other => Err(de::Error::unknown_field(&format!("{:?}",other), ARRAY_FIELDS)),
+                    other => Err(de::Error::unknown_field(
+                        &format!("{:?}", other),
+                        ARRAY_FIELDS,
+                    )),
                 }
             }
         }
@@ -183,37 +202,39 @@ impl<'de> Deserialize<'de> for ArrayField {
     }
 }
 
-impl<'de, A, Di, S> Visitor<'de> for ArrayVisitor<S,Di>
-    where A: Deserialize<'de>,
-          Di: Deserialize<'de> + Dimension,
-          S: DataOwned<Elem = A>
+impl<'de, A, Di, S> Visitor<'de> for ArrayVisitor<S, Di>
+where
+    A: Deserialize<'de>,
+    Di: Deserialize<'de> + Dimension,
+    S: DataOwned<Elem = A>,
 {
     type Value = ArrayBase<S, Di>;
 
-    fn expecting(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
+    fn expecting(&self, formatter: &mut fmt::Formatter<'_>) -> fmt::Result {
         formatter.write_str("ndarray representation")
     }
 
     fn visit_seq<V>(self, mut visitor: V) -> Result<ArrayBase<S, Di>, V::Error>
-        where V: SeqAccess<'de>,
+    where
+        V: SeqAccess<'de>,
     {
-        let v: u8 = match try!(visitor.next_element()) {
+        let v: u8 = match visitor.next_element()? {
             Some(value) => value,
             None => {
                 return Err(de::Error::invalid_length(0, &self));
             }
         };
 
-        try!(verify_version(v));
+        verify_version(v)?;
 
-        let dim: Di = match try!(visitor.next_element()) {
+        let dim: Di = match visitor.next_element()? {
             Some(value) => value,
             None => {
                 return Err(de::Error::invalid_length(1, &self));
             }
         };
 
-        let data: Vec<A> = match try!(visitor.next_element()) {
+        let data: Vec<A> = match visitor.next_element()? {
             Some(value) => value,
             None => {
                 return Err(de::Error::invalid_length(2, &self));
@@ -228,41 +249,42 @@ impl<'de, A, Di, S> Visitor<'de> for ArrayVisitor<S,Di>
     }
 
     fn visit_map<V>(self, mut visitor: V) -> Result<ArrayBase<S, Di>, V::Error>
-        where V: MapAccess<'de>,
+    where
+        V: MapAccess<'de>,
     {
         let mut v: Option<u8> = None;
         let mut data: Option<Vec<A>> = None;
         let mut dim: Option<Di> = None;
 
-        while let Some(key) = try!(visitor.next_key()) {
+        while let Some(key) = visitor.next_key()? {
             match key {
                 ArrayField::Version => {
-                    let val = try!(visitor.next_value());
-                    try!(verify_version(val));
+                    let val = visitor.next_value()?;
+                    verify_version(val)?;
                     v = Some(val);
-                },
+                }
                 ArrayField::Data => {
-                    data = Some(try!(visitor.next_value()));
-                },
+                    data = Some(visitor.next_value()?);
+                }
                 ArrayField::Dim => {
-                    dim = Some(try!(visitor.next_value()));
-                },
+                    dim = Some(visitor.next_value()?);
+                }
             }
         }
 
         let _v = match v {
             Some(v) => v,
-            None => try!(Err(de::Error::missing_field("v"))),
+            None => Err(de::Error::missing_field("v"))?,
         };
 
         let data = match data {
             Some(data) => data,
-            None => try!(Err(de::Error::missing_field("data"))),
+            None => Err(de::Error::missing_field("data"))?,
         };
 
         let dim = match dim {
             Some(dim) => dim,
-            None => try!(Err(de::Error::missing_field("dim"))),
+            None => Err(de::Error::missing_field("dim"))?,
         };
 
         if let Ok(array) = ArrayBase::from_shape_vec(dim, data) {
