@@ -6,8 +6,6 @@
 // option. This file may not be copied, modified, or distributed
 // except according to those terms.
 
-use std::cmp::Ordering;
-
 /// An axis index.
 ///
 /// An axis one of an array’s “dimensions”; an *n*-dimensional array has *n* axes.
@@ -15,28 +13,13 @@ use std::cmp::Ordering;
 ///
 /// All array axis arguments use this type to make the code easier to write
 /// correctly and easier to understand.
-#[derive(Eq, Ord, Hash, Debug)]
+#[derive(Clone, Copy, Debug, Eq, Hash, Ord, PartialEq, PartialOrd)]
 pub struct Axis(pub usize);
 
 impl Axis {
     /// Return the index of the axis.
     #[inline(always)]
-    pub fn index(&self) -> usize { self.0 }
-}
-
-copy_and_clone!{Axis}
-
-macro_rules! derive_cmp {
-    ($traitname:ident for $typename:ident, $method:ident -> $ret:ty) => {
-        impl $traitname for $typename {
-            #[inline(always)]
-            fn $method(&self, rhs: &Self) -> $ret {
-                (self.0).$method(&rhs.0)
-            }
-        }
+    pub fn index(self) -> usize {
+        self.0
     }
 }
-
-derive_cmp!{PartialEq for Axis, eq -> bool}
-derive_cmp!{PartialOrd for Axis, partial_cmp -> Option<Ordering>}
-
