@@ -1,11 +1,6 @@
-
-use std::ops::{
-    Index,
-    IndexMut,
-    Deref,
-    DerefMut,
-};
-use imp_prelude::*;
+use crate::imp_prelude::*;
+use std::hash::{Hash, Hasher};
+use std::ops::{Deref, DerefMut, Index, IndexMut};
 
 const CAP: usize = 4;
 
@@ -22,9 +17,7 @@ impl<T> Deref for IxDynRepr<T> {
         match *self {
             IxDynRepr::Inline(len, ref ar) => {
                 debug_assert!(len as (usize) <= ar.len());
-                unsafe {
-                    ar.get_unchecked(..len as usize)
-                }
+                unsafe { ar.get_unchecked(..len as usize) }
             }
             IxDynRepr::Alloc(ref ar) => &*ar,
         }
@@ -36,9 +29,7 @@ impl<T> DerefMut for IxDynRepr<T> {
         match *self {
             IxDynRepr::Inline(len, ref mut ar) => {
                 debug_assert!(len as (usize) <= ar.len());
-                unsafe {
-                    ar.get_unchecked_mut(..len as usize)
-                }
+                unsafe { ar.get_unchecked_mut(..len as usize) }
             }
             IxDynRepr::Alloc(ref mut ar) => &mut *ar,
         }
@@ -52,16 +43,13 @@ impl Default for IxDynRepr<Ix> {
     }
 }
 
-
-use ::libnum::Zero;
+use num_traits::Zero;
 
 impl<T: Copy + Zero> IxDynRepr<T> {
     pub fn copy_from(x: &[T]) -> Self {
         if x.len() <= CAP {
             let mut arr = [T::zero(); CAP];
-            for i in 0..x.len() {
-                arr[i] = x[i];
-            }
+            arr[..x.len()].copy_from_slice(&x[..]);
             IxDynRepr::Inline(x.len() as _, arr)
         } else {
             Self::from(x)
@@ -93,57 +81,58 @@ impl<T: Copy> IxDynRepr<T> {
 impl<T: Copy> Clone for IxDynRepr<T> {
     fn clone(&self) -> Self {
         match *self {
-            IxDynRepr::Inline(len, arr) => {
-                IxDynRepr::Inline(len, arr)
-            }
-            _ => Self::from(&self[..])
+            IxDynRepr::Inline(len, arr) => IxDynRepr::Inline(len, arr),
+            _ => Self::from(&self[..]),
         }
     }
 }
 
-impl<T: Eq> Eq for IxDynRepr<T> { }
+impl<T: Eq> Eq for IxDynRepr<T> {}
 
 impl<T: PartialEq> PartialEq for IxDynRepr<T> {
     fn eq(&self, rhs: &Self) -> bool {
         match (self, rhs) {
             (&IxDynRepr::Inline(slen, ref sarr), &IxDynRepr::Inline(rlen, ref rarr)) => {
-                slen == rlen &&
-                    (0..CAP as usize).filter(|&i| i < slen as usize)
+                slen == rlen
+                    && (0..CAP as usize)
+                        .filter(|&i| i < slen as usize)
                         .all(|i| sarr[i] == rarr[i])
             }
-            _ => self[..] == rhs[..]
+            _ => self[..] == rhs[..],
         }
     }
 }
 
+impl<T: Hash> Hash for IxDynRepr<T> {
+    fn hash<H: Hasher>(&self, state: &mut H) {
+        Hash::hash(&self[..], state)
+    }
+}
+
 /// Dynamic dimension or index type.
 ///
 /// Use `IxDyn` directly. This type implements a dynamic number of
 /// dimensions or indices. Short dimensions are stored inline and don't need
 /// any dynamic memory allocation.
-#[derive(Debug, Clone, PartialEq, Eq, Default)]
+#[derive(Debug, Clone, PartialEq, Eq, Hash, Default)]
 pub struct IxDynImpl(IxDynRepr<Ix>);
-unsafe impl Send for IxDynImpl {}
-unsafe impl Sync for IxDynImpl {}
 
 impl IxDynImpl {
     pub(crate) fn insert(&self, i: usize) -> Self {
         let len = self.len();
         debug_assert!(i <= len);
-        IxDynImpl(
-            if len < CAP {
-                let mut out = [1; CAP];
-                out[0..i].copy_from_slice(&self[0..i]);
-                out[i+1..len+1].copy_from_slice(&self[i..len]);
-                IxDynRepr::Inline((len + 1) as u32, out)
-            } else {
-                let mut out = Vec::with_capacity(len + 1);
-                out.extend_from_slice(&self[0..i]);
-                out.push(1);
-                out.extend_from_slice(&self[i..len]);
-                IxDynRepr::from_vec(out)
-            }
-        )
+        IxDynImpl(if len < CAP {
+            let mut out = [1; CAP];
+            out[0..i].copy_from_slice(&self[0..i]);
+            out[i + 1..=len].copy_from_slice(&self[i..len]);
+            IxDynRepr::Inline((len + 1) as u32, out)
+        } else {
+            let mut out = Vec::with_capacity(len + 1);
+            out.extend_from_slice(&self[0..i]);
+            out.push(1);
+            out.extend_from_slice(&self[i..len]);
+            IxDynRepr::from_vec(out)
+        })
     }
 
     fn remove(&self, i: usize) -> Self {
@@ -182,7 +171,8 @@ impl From<Vec<Ix>> for IxDynImpl {
 }
 
 impl<J> Index<J> for IxDynImpl
-    where [Ix]: Index<J>,
+where
+    [Ix]: Index<J>,
 {
     type Output = <[Ix] as Index<J>>::Output;
     fn index(&self, index: J) -> &Self::Output {
@@ -191,7 +181,8 @@ impl<J> Index<J> for IxDynImpl
 }
 
 impl<J> IndexMut<J> for IxDynImpl
-    where [Ix]: IndexMut<J>,
+where
+    [Ix]: IndexMut<J>,
 {
     fn index_mut(&mut self, index: J) -> &mut Self::Output {
         &mut self.0[index]
@@ -218,7 +209,7 @@ impl<'a> IntoIterator for &'a IxDynImpl {
     type IntoIter = <&'a [Ix] as IntoIterator>::IntoIter;
     #[inline]
     fn into_iter(self) -> Self::IntoIter {
-        self[..].into_iter()
+        self[..].iter()
     }
 }
 
@@ -233,7 +224,7 @@ impl IxDyn {
     /// Create a new dimension value with `n` axes, all zeros
     #[inline]
     pub fn zeros(n: usize) -> IxDyn {
-        const ZEROS: &'static [usize] = &[0; 4];
+        const ZEROS: &[usize] = &[0; 4];
         if n <= ZEROS.len() {
             Dim(&ZEROS[..n])
         } else {
