@@ -6,19 +6,20 @@
 // option. This file may not be copied, modified, or distributed
 // except according to those terms.
 
+use std::mem::{forget, size_of};
 use std::slice;
-use std::mem::{size_of, forget};
 
-use dimension;
-use imp_prelude::*;
+use crate::dimension;
+use crate::imp_prelude::*;
 
 /// Create an [**`Array`**](type.Array.html) with one, two or
 /// three dimensions.
 ///
 /// ```
-/// #[macro_use(array)]
 /// extern crate ndarray;
 ///
+/// use ndarray::array;
+///
 /// fn main() {
 ///     let a1 = array![1, 2, 3, 4];
 ///
@@ -47,19 +48,18 @@ macro_rules! array {
         $crate::Array2::from(vec![$([$($x,)*],)*])
     }};
     ($($x:expr),* $(,)*) => {{
-        $crate::Array::from_vec(vec![$($x,)*])
+        $crate::Array::from(vec![$($x,)*])
     }};
 }
 
 /// Create a zero-dimensional array with the element `x`.
-pub fn arr0<A>(x: A) -> Array0<A>
-{
+pub fn arr0<A>(x: A) -> Array0<A> {
     unsafe { ArrayBase::from_shape_vec_unchecked((), vec![x]) }
 }
 
 /// Create a one-dimensional array with elements from `xs`.
 pub fn arr1<A: Clone>(xs: &[A]) -> Array1<A> {
-    ArrayBase::from_vec(xs.to_vec())
+    ArrayBase::from(xs.to_vec())
 }
 
 /// Create a one-dimensional array with elements from `xs`.
@@ -68,7 +68,7 @@ pub fn rcarr1<A: Clone>(xs: &[A]) -> ArcArray<A, Ix1> {
 }
 
 /// Create a zero-dimensional array view borrowing `x`.
-pub fn aview0<A>(x: &A) -> ArrayView0<A> {
+pub fn aview0<A>(x: &A) -> ArrayView0<'_, A> {
     unsafe { ArrayView::from_shape_ptr(Ix0(), x) }
 }
 
@@ -84,7 +84,7 @@ pub fn aview0<A>(x: &A) -> ArrayView0<A> {
 ///
 /// assert_eq!(a2d.sum(), 1024.0);
 /// ```
-pub fn aview1<A>(xs: &[A]) -> ArrayView1<A> {
+pub fn aview1<A>(xs: &[A]) -> ArrayView1<'_, A> {
     ArrayView::from(xs)
 }
 
@@ -92,7 +92,7 @@ pub fn aview1<A>(xs: &[A]) -> ArrayView1<A> {
 ///
 /// **Panics** if the product of non-zero axis lengths overflows `isize`. (This
 /// can only occur when `V` is zero-sized.)
-pub fn aview2<A, V: FixedInitializer<Elem=A>>(xs: &[V]) -> ArrayView2<A> {
+pub fn aview2<A, V: FixedInitializer<Elem = A>>(xs: &[V]) -> ArrayView2<'_, A> {
     let cols = V::len();
     let rows = xs.len();
     let dim = Ix2(rows, cols);
@@ -115,10 +115,9 @@ pub fn aview2<A, V: FixedInitializer<Elem=A>>(xs: &[V]) -> ArrayView2<A> {
 /// Create a one-dimensional read-write array view with elements borrowing `xs`.
 ///
 /// ```
-/// #[macro_use(s)]
 /// extern crate ndarray;
 ///
-/// use ndarray::aview_mut1;
+/// use ndarray::{aview_mut1, s};
 ///
 /// // Create an array view over some data, then slice it and modify it.
 /// fn main() {
@@ -130,7 +129,7 @@ pub fn aview2<A, V: FixedInitializer<Elem=A>>(xs: &[V]) -> ArrayView2<A> {
 ///     assert_eq!(&data[..10], [5, 0, 0, 5, 0, 0, 5, 0, 0, 5]);
 /// }
 /// ```
-pub fn aview_mut1<A>(xs: &mut [A]) -> ArrayViewMut1<A> {
+pub fn aview_mut1<A>(xs: &mut [A]) -> ArrayViewMut1<'_, A> {
     ArrayViewMut::from(xs)
 }
 
@@ -159,7 +158,7 @@ pub fn aview_mut1<A>(xs: &mut [A]) -> ArrayViewMut1<A> {
 ///     assert_eq!(&data[..3], [[1., -1.], [1., -1.], [1., -1.]]);
 /// }
 /// ```
-pub fn aview_mut2<A, V: FixedInitializer<Elem=A>>(xs: &mut [V]) -> ArrayViewMut2<A> {
+pub fn aview_mut2<A, V: FixedInitializer<Elem = A>>(xs: &mut [V]) -> ArrayViewMut2<'_, A> {
     let cols = V::len();
     let rows = xs.len();
     let dim = Ix2(rows, cols);
@@ -221,19 +220,15 @@ impl_arr_init!(0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16,);
 /// );
 /// ```
 pub fn arr2<A: Clone, V: FixedInitializer<Elem = A>>(xs: &[V]) -> Array2<A>
-    where V: Clone,
+where
+    V: Clone,
 {
     Array2::from(xs.to_vec())
 }
 
-impl<A> From<Vec<A>> for Array1<A> {
-    fn from(xs: Vec<A>) -> Self {
-        Array1::from_vec(xs)
-    }
-}
-
 impl<A, V> From<Vec<V>> for Array2<A>
-    where V: FixedInitializer<Elem = A>
+where
+    V: FixedInitializer<Elem = A>,
 {
     /// Converts the `Vec` of arrays to an owned 2-D array.
     ///
@@ -262,8 +257,9 @@ impl<A, V> From<Vec<V>> for Array2<A>
 }
 
 impl<A, V, U> From<Vec<V>> for Array3<A>
-    where V: FixedInitializer<Elem=U>,
-          U: FixedInitializer<Elem=A>
+where
+    V: FixedInitializer<Elem = U>,
+    U: FixedInitializer<Elem = A>,
 {
     /// Converts the `Vec` of arrays to an owned 3-D array.
     ///
@@ -314,18 +310,23 @@ pub fn rcarr2<A: Clone, V: Clone + FixedInitializer<Elem = A>>(xs: &[V]) -> ArcA
 ///     a.shape() == [3, 2, 2]
 /// );
 /// ```
-pub fn arr3<A: Clone, V: FixedInitializer<Elem=U>, U: FixedInitializer<Elem=A>>(xs: &[V])
-    -> Array3<A>
-    where V: Clone,
-          U: Clone,
+pub fn arr3<A: Clone, V: FixedInitializer<Elem = U>, U: FixedInitializer<Elem = A>>(
+    xs: &[V],
+) -> Array3<A>
+where
+    V: Clone,
+    U: Clone,
 {
     Array3::from(xs.to_vec())
 }
 
 /// Create a three-dimensional array with elements from `xs`.
-pub fn rcarr3<A: Clone, V: FixedInitializer<Elem=U>, U: FixedInitializer<Elem=A>>(xs: &[V])
-    -> ArcArray<A, Ix3>
-    where V: Clone, U: Clone,
+pub fn rcarr3<A: Clone, V: FixedInitializer<Elem = U>, U: FixedInitializer<Elem = A>>(
+    xs: &[V],
+) -> ArcArray<A, Ix3>
+where
+    V: Clone,
+    U: Clone,
 {
     arr3(xs).into_shared()
 }
