@@ -6,23 +6,24 @@
 // option. This file may not be copied, modified, or distributed
 // except according to those terms.
 
-
 //! Methods for one-dimensional arrays.
-use imp_prelude::*;
+use crate::imp_prelude::*;
 
 /// # Methods For 1-D Arrays
 impl<A, S> ArrayBase<S, Ix1>
-    where S: Data<Elem=A>,
+where
+    S: RawData<Elem = A>,
 {
     /// Return an vector with the elements of the one-dimensional array.
     pub fn to_vec(&self) -> Vec<A>
-        where A: Clone,
+    where
+        A: Clone,
+        S: Data,
     {
         if let Some(slc) = self.as_slice() {
             slc.to_vec()
         } else {
-            ::iterators::to_vec(self.iter().map(|x| x.clone()))
+            crate::iterators::to_vec(self.iter().cloned())
         }
     }
 }
-
