@@ -10,18 +10,19 @@
 //!
 //!
 
-use libnum::{Zero, One, Float};
-use std::isize;
-use std::mem;
+#![allow(clippy::match_wild_err_arm)]
 
-use imp_prelude::*;
-use StrideShape;
-use dimension;
-use linspace;
-use error::{self, ShapeError};
-use indices;
-use indexes;
-use iterators::{to_vec, to_vec_mapped};
+use num_traits::{Float, One, Zero};
+
+use crate::dimension;
+use crate::error::{self, ShapeError};
+use crate::extension::nonnull::nonnull_from_vec_data;
+use crate::imp_prelude::*;
+use crate::indexes;
+use crate::indices;
+use crate::iterators::{to_vec, to_vec_mapped};
+use crate::StrideShape;
+use crate::{geomspace, linspace, logspace};
 
 /// # Constructor Methods for Owned Arrays
 ///
@@ -30,7 +31,8 @@ use iterators::{to_vec, to_vec_mapped};
 ///
 /// ## Constructor methods for one-dimensional arrays.
 impl<S, A> ArrayBase<S, Ix1>
-    where S: DataOwned<Elem=A>,
+where
+    S: DataOwned<Elem = A>,
 {
     /// Create a one-dimensional array from a vector (no copying needed).
     ///
@@ -39,38 +41,23 @@ impl<S, A> ArrayBase<S, Ix1>
     /// ```rust
     /// use ndarray::Array;
     ///
-    /// let array = Array::from_vec(vec![1., 2., 3., 4.]);
+    /// let array = Array::from(vec![1., 2., 3., 4.]);
     /// ```
+    #[deprecated(note = "use standard `from`", since = "0.13.0")]
     pub fn from_vec(v: Vec<A>) -> Self {
-        if mem::size_of::<A>() == 0 {
-            assert!(
-                v.len() <= isize::MAX as usize,
-                "Length must fit in `isize`.",
-            );
-        }
-        unsafe { Self::from_shape_vec_unchecked(v.len() as Ix, v) }
+        Self::from(v)
     }
 
-    /// Create a one-dimensional array from an iterable.
+    /// Create a one-dimensional array with `n` evenly spaced elements from
+    /// `start` to `end` (inclusive). `A` must be a floating point type.
     ///
-    /// **Panics** if the length is greater than `isize::MAX`.
+    /// Note that if `start > end`, the first element will still be `start`,
+    /// and the following elements will be decreasing. This is different from
+    /// the behavior of `std::ops::RangeInclusive`, which interprets `start >
+    /// end` to mean that the range is empty.
     ///
-    /// ```rust
-    /// use ndarray::{Array, arr1};
-    ///
-    /// let array = Array::from_iter((0..5).map(|x| x * x));
-    /// assert!(array == arr1(&[0, 1, 4, 9, 16]))
-    /// ```
-    pub fn from_iter<I>(iterable: I) -> Self
-        where I: IntoIterator<Item=A>
-    {
-        Self::from_vec(iterable.into_iter().collect())
-    }
-
-    /// Create a one-dimensional array from the inclusive interval
-    /// `[start, end]` with `n` elements. `A` must be a floating point type.
-    ///
-    /// **Panics** if `n` is greater than `isize::MAX`.
+    /// **Panics** if `n` is greater than `isize::MAX` or if converting `n - 1`
+    /// to type `A` fails.
     ///
     /// ```rust
     /// use ndarray::{Array, arr1};
@@ -79,14 +66,14 @@ impl<S, A> ArrayBase<S, Ix1>
     /// assert!(array == arr1(&[0.0, 0.25, 0.5, 0.75, 1.0]))
     /// ```
     pub fn linspace(start: A, end: A, n: usize) -> Self
-        where A: Float,
+    where
+        A: Float,
     {
-        Self::from_vec(to_vec(linspace::linspace(start, end, n)))
+        Self::from(to_vec(linspace::linspace(start, end, n)))
     }
 
-    /// Create a one-dimensional array from the half-open interval
-    /// `[start, end)` with elements spaced by `step`. `A` must be a floating
-    /// point type.
+    /// Create a one-dimensional array with elements from `start` to `end`
+    /// (exclusive), incrementing by `step`. `A` must be a floating point type.
     ///
     /// **Panics** if the length is greater than `isize::MAX`.
     ///
@@ -97,22 +84,87 @@ impl<S, A> ArrayBase<S, Ix1>
     /// assert!(array == arr1(&[0., 1., 2., 3., 4.]))
     /// ```
     pub fn range(start: A, end: A, step: A) -> Self
-        where A: Float,
+    where
+        A: Float,
     {
-        Self::from_vec(to_vec(linspace::range(start, end, step)))
+        Self::from(to_vec(linspace::range(start, end, step)))
+    }
+
+    /// Create a one-dimensional array with `n` logarithmically spaced
+    /// elements, with the starting value being `base.powf(start)` and the
+    /// final one being `base.powf(end)`. `A` must be a floating point type.
+    ///
+    /// If `base` is negative, all values will be negative.
+    ///
+    /// **Panics** if `n` is greater than `isize::MAX` or if converting `n - 1`
+    /// to type `A` fails.
+    ///
+    /// ```rust
+    /// use approx::assert_abs_diff_eq;
+    /// use ndarray::{Array, arr1};
+    ///
+    /// # #[cfg(feature = "approx")] {
+    /// let array = Array::logspace(10.0, 0.0, 3.0, 4);
+    /// assert_abs_diff_eq!(array, arr1(&[1e0, 1e1, 1e2, 1e3]));
+    ///
+    /// let array = Array::logspace(-10.0, 3.0, 0.0, 4);
+    /// assert_abs_diff_eq!(array, arr1(&[-1e3, -1e2, -1e1, -1e0]));
+    /// # }
+    /// ```
+    pub fn logspace(base: A, start: A, end: A, n: usize) -> Self
+    where
+        A: Float,
+    {
+        Self::from(to_vec(logspace::logspace(base, start, end, n)))
+    }
+
+    /// Create a one-dimensional array with `n` geometrically spaced elements
+    /// from `start` to `end` (inclusive). `A` must be a floating point type.
+    ///
+    /// Returns `None` if `start` and `end` have different signs or if either
+    /// one is zero. Conceptually, this means that in order to obtain a `Some`
+    /// result, `end / start` must be positive.
+    ///
+    /// **Panics** if `n` is greater than `isize::MAX` or if converting `n - 1`
+    /// to type `A` fails.
+    ///
+    /// ```rust
+    /// use approx::assert_abs_diff_eq;
+    /// use ndarray::{Array, arr1};
+    ///
+    /// # fn example() -> Option<()> {
+    /// # #[cfg(feature = "approx")] {
+    /// let array = Array::geomspace(1e0, 1e3, 4)?;
+    /// assert_abs_diff_eq!(array, arr1(&[1e0, 1e1, 1e2, 1e3]), epsilon = 1e-12);
+    ///
+    /// let array = Array::geomspace(-1e3, -1e0, 4)?;
+    /// assert_abs_diff_eq!(array, arr1(&[-1e3, -1e2, -1e1, -1e0]), epsilon = 1e-12);
+    /// # }
+    /// # Some(())
+    /// # }
+    /// #
+    /// # fn main() { example().unwrap() }
+    /// ```
+    pub fn geomspace(start: A, end: A, n: usize) -> Option<Self>
+    where
+        A: Float,
+    {
+        Some(Self::from(to_vec(geomspace::geomspace(start, end, n)?)))
     }
 }
 
 /// ## Constructor methods for two-dimensional arrays.
 impl<S, A> ArrayBase<S, Ix2>
-    where S: DataOwned<Elem=A>,
+where
+    S: DataOwned<Elem = A>,
 {
     /// Create an identity matrix of size `n` (square 2D array).
     ///
     /// **Panics** if `n * n` would overflow `isize`.
     pub fn eye(n: Ix) -> Self
-        where S: DataMut,
-              A: Clone + Zero + One,
+    where
+        S: DataMut,
+        A: Clone + Zero + One,
     {
         let mut eye = Self::zeros((n, n));
         for a_ii in eye.diag_mut() {
@@ -120,16 +172,42 @@ impl<S, A> ArrayBase<S, Ix2>
         }
         eye
     }
+
+    /// Create a 2D matrix from its diagonal
+    ///
+    /// **Panics** if `diag.len() * diag.len()` would overflow `isize`.
+    ///
+    /// ```rust
+    /// use ndarray::{Array2, arr1, arr2};
+    ///
+    /// let diag = arr1(&[1, 2]);
+    /// let array = Array2::from_diag(&diag);
+    /// assert_eq!(array, arr2(&[[1, 0], [0, 2]]));
+    /// ```
+    pub fn from_diag<S2>(diag: &ArrayBase<S2, Ix1>) -> Self
+    where
+        A: Clone + Zero,
+        S: DataMut,
+        S2: Data<Elem = A>,
+    {
+        let n = diag.len();
+        let mut arr = Self::zeros((n, n));
+        arr.diag_mut().assign(&diag);
+        arr
+    }
 }
 
 #[cfg(not(debug_assertions))]
+#[allow(clippy::match_wild_err_arm)]
 macro_rules! size_of_shape_checked_unwrap {
     ($dim:expr) => {
         match dimension::size_of_shape_checked($dim) {
             Ok(sz) => sz,
-            Err(_) => panic!("ndarray: Shape too large, product of non-zero axis lengths overflows isize"),
+            Err(_) => {
+                panic!("ndarray: Shape too large, product of non-zero axis lengths overflows isize")
+            }
         }
-    }
+    };
 }
 
 #[cfg(debug_assertions)]
@@ -143,7 +221,7 @@ macro_rules! size_of_shape_checked_unwrap {
                 $dim
             ),
         }
-    }
+    };
 }
 
 /// ## Constructor methods for n-dimensional arrays.
@@ -163,8 +241,9 @@ macro_rules! size_of_shape_checked_unwrap {
 /// `Into<StrideShape>` argument *optionally* support custom strides, for
 /// example a shape given like `(10, 2, 2).strides((1, 10, 20))` is valid.
 impl<S, A, D> ArrayBase<S, D>
-    where S: DataOwned<Elem=A>,
-          D: Dimension,
+where
+    S: DataOwned<Elem = A>,
+    D: Dimension,
 {
     /// Create an array with copies of `elem`, shape `shape`.
     ///
@@ -187,8 +266,9 @@ impl<S, A, D> ArrayBase<S, D>
     /// assert!(b.strides() == &[1, 2, 4]);
     /// ```
     pub fn from_elem<Sh>(shape: Sh, elem: A) -> Self
-        where A: Clone,
-              Sh: ShapeBuilder<Dim=D>,
+    where
+        A: Clone,
+        Sh: ShapeBuilder<Dim = D>,
     {
         let shape = shape.into_shape();
         let size = size_of_shape_checked_unwrap!(&shape.dim);
@@ -200,8 +280,9 @@ impl<S, A, D> ArrayBase<S, D>
     ///
     /// **Panics** if the product of non-zero axis lengths overflows `isize`.
     pub fn zeros<Sh>(shape: Sh) -> Self
-        where A: Clone + Zero,
-              Sh: ShapeBuilder<Dim=D>,
+    where
+        A: Clone + Zero,
+        Sh: ShapeBuilder<Dim = D>,
     {
         Self::from_elem(shape, A::zero())
     }
@@ -210,8 +291,9 @@ impl<S, A, D> ArrayBase<S, D>
     ///
     /// **Panics** if the product of non-zero axis lengths overflows `isize`.
     pub fn ones<Sh>(shape: Sh) -> Self
-        where A: Clone + One,
-              Sh: ShapeBuilder<Dim=D>,
+    where
+        A: Clone + One,
+        Sh: ShapeBuilder<Dim = D>,
     {
         Self::from_elem(shape, A::one())
     }
@@ -220,8 +302,9 @@ impl<S, A, D> ArrayBase<S, D>
     ///
     /// **Panics** if the product of non-zero axis lengths overflows `isize`.
     pub fn default<Sh>(shape: Sh) -> Self
-        where A: Default,
-              Sh: ShapeBuilder<Dim=D>,
+    where
+        A: Default,
+        Sh: ShapeBuilder<Dim = D>,
     {
         let shape = shape.into_shape();
         let size = size_of_shape_checked_unwrap!(&shape.dim);
@@ -232,12 +315,13 @@ impl<S, A, D> ArrayBase<S, D>
     /// Create an array with values created by the function `f`.
     ///
     /// `f` is called with the index of the element to create; the elements are
-    /// visited in arbitirary order.
+    /// visited in arbitrary order.
     ///
     /// **Panics** if the product of non-zero axis lengths overflows `isize`.
     pub fn from_shape_fn<Sh, F>(shape: Sh, f: F) -> Self
-        where Sh: ShapeBuilder<Dim=D>,
-              F: FnMut(D::Pattern) -> A,
+    where
+        Sh: ShapeBuilder<Dim = D>,
+        F: FnMut(D::Pattern) -> A,
     {
         let shape = shape.into_shape();
         let _ = size_of_shape_checked_unwrap!(&shape.dim);
@@ -246,7 +330,7 @@ impl<S, A, D> ArrayBase<S, D>
             unsafe { Self::from_shape_vec_unchecked(shape, v) }
         } else {
             let dim = shape.dim.clone();
-            let v = to_vec_mapped(indexes::indices_iter_f(dim).into_iter(), f);
+            let v = to_vec_mapped(indexes::indices_iter_f(dim), f);
             unsafe { Self::from_shape_vec_unchecked(shape, v) }
         }
     }
@@ -285,14 +369,14 @@ impl<S, A, D> ArrayBase<S, D>
     /// );
     /// ```
     pub fn from_shape_vec<Sh>(shape: Sh, v: Vec<A>) -> Result<Self, ShapeError>
-        where Sh: Into<StrideShape<D>>,
+    where
+        Sh: Into<StrideShape<D>>,
     {
         // eliminate the type parameter Sh as soon as possible
         Self::from_shape_vec_impl(shape.into(), v)
     }
 
-    fn from_shape_vec_impl(shape: StrideShape<D>, v: Vec<A>) -> Result<Self, ShapeError>
-    {
+    fn from_shape_vec_impl(shape: StrideShape<D>, v: Vec<A>) -> Result<Self, ShapeError> {
         let dim = shape.dim;
         let strides = shape.strides;
         if shape.custom {
@@ -328,22 +412,21 @@ impl<S, A, D> ArrayBase<S, D>
     /// 5. The strides must not allow any element to be referenced by two different
     ///    indices.
     pub unsafe fn from_shape_vec_unchecked<Sh>(shape: Sh, v: Vec<A>) -> Self
-        where Sh: Into<StrideShape<D>>,
+    where
+        Sh: Into<StrideShape<D>>,
     {
         let shape = shape.into();
         Self::from_vec_dim_stride_unchecked(shape.dim, shape.strides, v)
     }
 
-    unsafe fn from_vec_dim_stride_unchecked(dim: D, strides: D, mut v: Vec<A>)
-        -> Self
-    {
+    unsafe fn from_vec_dim_stride_unchecked(dim: D, strides: D, mut v: Vec<A>) -> Self {
         // debug check for issues that indicates wrong use of this constructor
         debug_assert!(dimension::can_index_slice(&v, &dim, &strides).is_ok());
         ArrayBase {
-            ptr: v.as_mut_ptr(),
+            ptr: nonnull_from_vec_data(&mut v),
             data: DataOwned::new(v),
-            strides: strides,
-            dim: dim
+            strides,
+            dim,
         }
     }
 
@@ -370,10 +453,9 @@ impl<S, A, D> ArrayBase<S, D>
     /// ### Examples
     ///
     /// ```
-    /// #[macro_use(s)]
     /// extern crate ndarray;
     ///
-    /// use ndarray::Array2;
+    /// use ndarray::{s, Array2};
     ///
     /// // Example Task: Let's create a column shifted copy of a in b
     ///
@@ -394,8 +476,9 @@ impl<S, A, D> ArrayBase<S, D>
     /// # }
     /// ```
     pub unsafe fn uninitialized<Sh>(shape: Sh) -> Self
-        where A: Copy,
-              Sh: ShapeBuilder<Dim=D>,
+    where
+        A: Copy,
+        Sh: ShapeBuilder<Dim = D>,
     {
         let shape = shape.into_shape();
         let size = size_of_shape_checked_unwrap!(&shape.dim);
@@ -403,5 +486,4 @@ impl<S, A, D> ArrayBase<S, D>
         v.set_len(size);
         Self::from_shape_vec_unchecked(shape, v)
     }
-
 }
