@@ -1,15 +1,14 @@
-
-use imp_prelude::*;
 use super::ElementsBase;
-use IntoDimension;
-use Layout;
-use NdProducer;
+use crate::imp_prelude::*;
+use crate::IntoDimension;
+use crate::Layout;
+use crate::NdProducer;
 
 /// Window producer and iterable
 ///
 /// See [`.windows()`](../struct.ArrayBase.html#method.windows) for more
 /// information.
-pub struct Windows<'a, A: 'a, D> {
+pub struct Windows<'a, A, D> {
     base: ArrayView<'a, A, D>,
     window: D,
     strides: D,
@@ -21,10 +20,16 @@ impl<'a, A, D: Dimension> Windows<'a, A, D> {
         E: IntoDimension<Dim = D>,
     {
         let window = window_size.into_dimension();
-        ndassert!(a.ndim() == window.ndim(),
-            concat!("Window dimension {} does not match array dimension {} ",
-            "(with array of shape {:?})"),
-            window.ndim(), a.ndim(), a.shape());
+        ndassert!(
+            a.ndim() == window.ndim(),
+            concat!(
+                "Window dimension {} does not match array dimension {} ",
+                "(with array of shape {:?})"
+            ),
+            window.ndim(),
+            a.ndim(),
+            a.shape()
+        );
         let mut size = a.dim;
         for (sz, &ws) in size.slice_mut().iter_mut().zip(window.slice()) {
             assert_ne!(ws, 0, "window-size must not be zero!");
@@ -36,8 +41,8 @@ impl<'a, A, D: Dimension> Windows<'a, A, D> {
 
         unsafe {
             Windows {
-                base: ArrayView::from_shape_ptr(size.clone().strides(a.strides), a.ptr),
-                window: window,
+                base: ArrayView::from_shape_ptr(size.clone().strides(a.strides), a.ptr.as_ptr()),
+                window,
                 strides: window_strides,
             }
         }
@@ -64,8 +69,9 @@ impl_ndproducer! {
 }
 
 impl<'a, A, D> IntoIterator for Windows<'a, A, D>
-    where D: Dimension,
-          A: 'a,
+where
+    D: Dimension,
+    A: 'a,
 {
     type Item = <Self::IntoIter as Iterator>::Item;
     type IntoIter = WindowsIter<'a, A, D>;
@@ -82,13 +88,13 @@ impl<'a, A, D> IntoIterator for Windows<'a, A, D>
 ///
 /// See [`.windows()`](../struct.ArrayBase.html#method.windows) for more
 /// information.
-pub struct WindowsIter<'a, A: 'a, D> {
+pub struct WindowsIter<'a, A, D> {
     iter: ElementsBase<'a, A, D>,
     window: D,
     strides: D,
 }
 
-impl_iterator!{
+impl_iterator! {
     ['a, A, D: Dimension]
     [Clone => 'a, A, D: Clone]
     WindowsIter {
