@@ -5,8 +5,14 @@
 // <LICENSE-MIT or http://opensource.org/licenses/MIT>, at your
 // option. This file may not be copied, modified, or distributed
 // except according to those terms.
-#![crate_name="ndarray"]
-#![doc(html_root_url = "https://docs.rs/ndarray/0.12/")]
+#![crate_name = "ndarray"]
+#![doc(html_root_url = "https://docs.rs/ndarray/0.13/")]
+#![allow(
+    clippy::many_single_char_names,
+    clippy::deref_addrof,
+    clippy::unreadable_literal,
+    clippy::many_single_char_names
+)]
 
 //! The `ndarray` crate provides an *n*-dimensional container for general elements
 //! and for numerics.
@@ -32,8 +38,8 @@
 //! - Higher order operations and arithmetic are performant
 //! - Array views can be used to slice and mutate any `[T]` data using
 //!   `ArrayView::from` and `ArrayViewMut::from`.
-//! - `Zip` for lock step function application across two or more arrays or other
-//!   item producers (`NdProducer` trait).
+//! - [`Zip`](struct.Zip.html) for lock step function application across two or more arrays or other
+//!   item producers ([`NdProducer`](trait.NdProducer.html) trait).
 //!
 //! ## Crate Status
 //!
@@ -55,22 +61,22 @@
 //!     needs matching memory layout to be efficient (with some exceptions).
 //!   + Efficient floating point matrix multiplication even for very large
 //!     matrices; can optionally use BLAS to improve it further.
-//!   + See also the [`ndarray-parallel`] crate for integration with rayon.
-//! - **Requires Rust 1.27**
-//!
-//! [`ndarray-parallel`]: https://docs.rs/ndarray-parallel
+//! - **Requires Rust 1.37 or later**
 //!
 //! ## Crate Feature Flags
 //!
 //! The following crate feature flags are available. They are configured in your
 //! `Cargo.toml`.
 //!
-//! - `rustc-serialize`
+//! - `serde`
 //!   - Optional, compatible with Rust stable
-//!   - Enables serialization support for rustc-serialize 0.3
-//! - `serde-1`
+//!   - Enables serialization support for serde 1.x
+//! - `rayon`
 //!   - Optional, compatible with Rust stable
-//!   - Enables serialization support for serde 1.0
+//!   - Enables parallel iterators, parallelized methods and [`par_azip!`].
+//! - `approx`
+//!   - Optional, compatible with Rust stable
+//!   - Enables implementations of traits from the [`approx`] crate.
 //! - `blas`
 //!   - Optional and experimental, compatible with Rust stable
 //!   - Enable transparent BLAS support for matrix multiplication.
@@ -86,26 +92,24 @@
 //!
 //! * If you have experience with NumPy, you may also be interested in
 //!   [`ndarray_for_numpy_users`](doc/ndarray_for_numpy_users/index.html).
+//!
+//! ## The ndarray ecosystem
+//!
+//! `ndarray` provides a lot of functionality, but it's not a one-stop solution.
+//!
+//! `ndarray` includes matrix multiplication and other binary/unary operations out of the box.
+//! More advanced linear algebra routines (e.g. SVD decomposition or eigenvalue computation)
+//! can be found in [`ndarray-linalg`](https://crates.io/crates/ndarray-linalg).
+//!
+//! The same holds for statistics: `ndarray` provides some basic functionalities (e.g. `mean`)
+//! but more advanced routines can be found in [`ndarray-stats`](https://crates.io/crates/ndarray-stats).
+//!
+//! If you are looking to generate random arrays instead, check out [`ndarray-rand`](https://crates.io/crates/ndarray-rand).
 
-#[cfg(feature = "serde-1")]
-extern crate serde;
-#[cfg(feature = "rustc-serialize")]
-extern crate rustc_serialize as serialize;
-
-#[cfg(feature="blas")]
-extern crate cblas_sys;
-#[cfg(feature="blas")]
+#[cfg(feature = "blas")]
 extern crate blas_src;
-
-extern crate matrixmultiply;
-
-#[macro_use(izip)] extern crate itertools;
-extern crate num_traits as libnum;
-extern crate num_complex;
-
-#[cfg(test)]
-#[macro_use(quickcheck)]
-extern crate quickcheck;
+#[cfg(feature = "blas")]
+extern crate cblas_sys;
 
 #[cfg(feature = "docs")]
 pub mod doc;
@@ -113,94 +117,81 @@ pub mod doc;
 use std::marker::PhantomData;
 use std::sync::Arc;
 
-pub use dimension::{
-    Dimension,
-    IntoDimension,
-    RemoveAxis,
-    Axis,
-    AxisDescription,
-};
-pub use dimension::dim::*;
+pub use crate::dimension::dim::*;
+pub use crate::dimension::{Axis, AxisDescription, Dimension, IntoDimension, RemoveAxis};
 
-pub use dimension::NdIndex;
-pub use dimension::IxDynImpl;
-pub use indexes::{indices, indices_of};
-pub use error::{ShapeError, ErrorKind};
-pub use slice::{Slice, SliceInfo, SliceNextDim, SliceOrIndex};
+pub use crate::dimension::IxDynImpl;
+pub use crate::dimension::NdIndex;
+pub use crate::error::{ErrorKind, ShapeError};
+pub use crate::indexes::{indices, indices_of};
+pub use crate::slice::{Slice, SliceInfo, SliceNextDim, SliceOrIndex};
 
-use iterators::Baseiter;
-use iterators::{ElementsBase, ElementsBaseMut, Iter, IterMut, Lanes, LanesMut};
+use crate::iterators::Baseiter;
+use crate::iterators::{ElementsBase, ElementsBaseMut, Iter, IterMut, Lanes, LanesMut};
 
-pub use arraytraits::AsArray;
-pub use linalg_traits::{LinalgScalar, NdFloat};
-pub use stacking::stack;
+pub use crate::arraytraits::AsArray;
+pub use crate::linalg_traits::{LinalgScalar, NdFloat};
+pub use crate::stacking::stack;
 
-pub use shape_builder::{ ShapeBuilder};
-pub use impl_views::IndexLonger;
+pub use crate::impl_views::IndexLonger;
+pub use crate::shape_builder::ShapeBuilder;
 
-#[macro_use] mod macro_utils;
-#[macro_use] mod private;
+#[macro_use]
+mod macro_utils;
+#[macro_use]
+mod private;
 mod aliases;
-mod arraytraits;
-#[cfg(feature = "serde-1")]
+#[cfg(feature = "approx")]
+mod array_approx;
+#[cfg(feature = "serde")]
 mod array_serde;
-#[cfg(feature = "rustc-serialize")]
-mod array_serialize;
 mod arrayformat;
+mod arraytraits;
 mod data_traits;
 
-pub use aliases::*;
+pub use crate::aliases::*;
 
-pub use data_traits::{
-    Data,
-    DataMut,
-    DataOwned,
-    DataShared,
-    DataClone,
+#[allow(deprecated)]
+pub use crate::data_traits::{
+    Data, DataClone, DataMut, DataOwned, DataShared, RawData, RawDataClone, RawDataMut,
 };
 
-mod dimension;
-
 mod free_functions;
-pub use free_functions::*;
-pub use iterators::iter;
+pub use crate::free_functions::*;
+pub use crate::iterators::iter;
 
-mod slice;
-mod layout;
+mod error;
+mod extension;
+mod geomspace;
 mod indexes;
 mod iterators;
+mod layout;
 mod linalg_traits;
 mod linspace;
+mod logspace;
 mod numeric_util;
-mod error;
 mod shape_builder;
+#[macro_use]
+mod slice;
 mod stacking;
 #[macro_use]
 mod zip;
 
-pub use zip::{
-    Zip,
-    NdProducer,
-    IntoNdProducer,
-    FoldWhile,
-};
+mod dimension;
 
-pub use layout::Layout;
+pub use crate::zip::{FoldWhile, IntoNdProducer, NdProducer, Zip};
+
+pub use crate::layout::Layout;
 
 /// Implementation's prelude. Common types used everywhere.
 mod imp_prelude {
-    pub use prelude::*;
-    pub use ArcArray;
-    pub use {
-        RemoveAxis,
-        Data,
-        DataMut,
-        DataOwned,
-        DataShared,
-        ViewRepr,
-        Ix, Ixs,
+    pub use crate::dimension::DimensionExt;
+    pub use crate::prelude::*;
+    pub use crate::ArcArray;
+    pub use crate::{
+        CowRepr, Data, DataMut, DataOwned, DataShared, Ix, Ixs, RawData, RawDataMut, RawViewRepr,
+        RemoveAxis, ViewRepr,
     };
-    pub use dimension::DimensionExt;
 }
 
 pub mod prelude;
@@ -223,18 +214,21 @@ pub type Ixs = isize;
 /// The `ArrayBase<S, D>` is parameterized by `S` for the data container and
 /// `D` for the dimensionality.
 ///
-/// Type aliases [`Array`], [`ArcArray`], [`ArrayView`], and [`ArrayViewMut`] refer
-/// to `ArrayBase` with different types for the data container.
+/// Type aliases [`Array`], [`ArcArray`], [`CowArray`], [`ArrayView`], and
+/// [`ArrayViewMut`] refer to `ArrayBase` with different types for the data
+/// container.
 ///
 /// [`Array`]: type.Array.html
 /// [`ArcArray`]: type.ArcArray.html
 /// [`ArrayView`]: type.ArrayView.html
 /// [`ArrayViewMut`]: type.ArrayViewMut.html
+/// [`CowArray`]: type.CowArray.html
 ///
 /// ## Contents
 ///
 /// + [Array](#array)
 /// + [ArcArray](#arcarray)
+/// + [CowArray](#cowarray)
 /// + [Array Views](#array-views)
 /// + [Indexing and Dimension](#indexing-and-dimension)
 /// + [Loops, Producers and Iterators](#loops-producers-and-iterators)
@@ -278,6 +272,16 @@ pub type Ixs = isize;
 /// [`view_mut()`](#method.view_mut) or [`get_mut()`](#method.get_mut),
 /// will break sharing and require a clone of the data (if it is not uniquely held).
 ///
+/// ## `CowArray`
+///
+/// [`CowArray`](type.CowArray.html) is analogous to
+/// [`std::borrow::Cow`](https://doc.rust-lang.org/std/borrow/enum.Cow.html).
+/// It can represent either an immutable view or a uniquely owned array. If a
+/// `CowArray` instance is the immutable view variant, then calling a method
+/// for mutating elements in the array will cause it to be converted into the
+/// owned variant (by cloning all the elements) before the modification is
+/// performed.
+///
 /// ## Array Views
 ///
 /// [`ArrayView`] and [`ArrayViewMut`] are read-only and read-write array views
@@ -409,12 +413,12 @@ pub type Ixs = isize;
 ///
 /// // 2. Use Zip to pair each row in 2D `a` with elements in 1D `b`
 /// use ndarray::Zip;
-/// let mut b = Array::zeros(a.rows());
+/// let mut b = Array::zeros(a.nrows());
 ///
 /// Zip::from(a.genrows())
 ///     .and(&mut b)
 ///     .apply(|a_row, b_elt| {
-///         *b_elt = a_row[a.cols() - 1] - a_row[0];
+///         *b_elt = a_row[a.ncols() - 1] - a_row[0];
 ///     });
 /// ```
 ///
@@ -470,11 +474,9 @@ pub type Ixs = isize;
 /// [`.slice_collapse()`]: #method.slice_collapse
 ///
 /// ```
-/// // import the s![] macro
-/// #[macro_use(s)]
 /// extern crate ndarray;
 ///
-/// use ndarray::{arr2, arr3};
+/// use ndarray::{arr2, arr3, s};
 ///
 /// fn main() {
 ///
@@ -554,9 +556,9 @@ pub type Ixs = isize;
 /// [`.outer_iter_mut()`]: #method.outer_iter_mut
 ///
 /// ```
-/// #[macro_use(s)] extern crate ndarray;
+/// extern crate ndarray;
 ///
-/// use ndarray::{arr3, aview1, aview2, Axis};
+/// use ndarray::{arr3, aview1, aview2, s, Axis};
 ///
 /// # fn main() {
 ///
@@ -616,6 +618,25 @@ pub type Ixs = isize;
 /// - `B @ &A` which consumes `B`, updates it with the result, and returns it
 /// - `C @= &A` which performs an arithmetic operation in place
 ///
+/// Note that the element type needs to implement the operator trait and the
+/// `Clone` trait.
+///
+/// ```
+/// use ndarray::{array, ArrayView1};
+///
+/// let owned1 = array![1, 2];
+/// let owned2 = array![3, 4];
+/// let view1 = ArrayView1::from(&[5, 6]);
+/// let view2 = ArrayView1::from(&[7, 8]);
+/// let mut mutable = array![9, 10];
+///
+/// let sum1 = &view1 + &view2;   // Allocates a new array. Note the explicit `&`.
+/// // let sum2 = view1 + &view2; // This doesn't work because `view1` is not an owned array.
+/// let sum3 = owned1 + view1;    // Consumes `owned1`, updates it, and returns it.
+/// let sum4 = owned2 + &view2;   // Consumes `owned2`, updates it, and returns it.
+/// mutable += &view2;            // Updates `mutable` in-place.
+/// ```
+///
 /// ### Binary Operators with Array and Scalar
 ///
 /// The trait [`ScalarOperand`](trait.ScalarOperand.html) marks types that can be used in arithmetic
@@ -676,7 +697,7 @@ pub type Ixs = isize;
 /// <table>
 /// <tr>
 /// <th rowspan="2">Output</th>
-/// <th colspan="4">Input</th>
+/// <th colspan="5">Input</th>
 /// </tr>
 ///
 /// <tr>
@@ -692,6 +713,11 @@ pub type Ixs = isize;
 /// </td>
 /// <td>
 ///
+/// `CowArray<'a, A, D>`
+///
+/// </td>
+/// <td>
+///
 /// `ArrayView<'a, A, D>`
 ///
 /// </td>
@@ -722,6 +748,11 @@ pub type Ixs = isize;
 /// </td>
 /// <td>
 ///
+/// [`a.into_owned()`][.into_owned()]
+///
+/// </td>
+/// <td>
+///
 /// [`a.to_owned()`][.to_owned()]
 ///
 /// </td>
@@ -752,6 +783,11 @@ pub type Ixs = isize;
 /// </td>
 /// <td>
 ///
+/// [`a.into_owned().into_shared()`][.into_shared()]
+///
+/// </td>
+/// <td>
+///
 /// [`a.to_owned().into_shared()`][.into_shared()]
 ///
 /// </td>
@@ -762,6 +798,41 @@ pub type Ixs = isize;
 /// </td>
 /// </tr>
 ///
+/// <!--Conversions to `CowArray<'a, A, D>`-->
+///
+/// <tr>
+/// <td>
+///
+/// `CowArray<'a, A, D>`
+///
+/// </td>
+/// <td>
+///
+/// [`CowArray::from(a)`](type.CowArray.html#impl-From<ArrayBase<OwnedRepr<A>%2C%20D>>)
+///
+/// </td>
+/// <td>
+///
+/// [`CowArray::from(a.into_owned())`](type.CowArray.html#impl-From<ArrayBase<OwnedRepr<A>%2C%20D>>)
+///
+/// </td>
+/// <td>
+///
+/// no-op
+///
+/// </td>
+/// <td>
+///
+/// [`CowArray::from(a)`](type.CowArray.html#impl-From<ArrayBase<ViewRepr<%26%27a%20A>%2C%20D>>)
+///
+/// </td>
+/// <td>
+///
+/// [`CowArray::from(a.view())`](type.CowArray.html#impl-From<ArrayBase<ViewRepr<%26%27a%20A>%2C%20D>>)
+///
+/// </td>
+/// </tr>
+///
 /// <!--Conversions to `ArrayView<'b, A, D>`-->
 ///
 /// <tr>
@@ -782,6 +853,11 @@ pub type Ixs = isize;
 /// </td>
 /// <td>
 ///
+/// [`a.view()`][.view()]
+///
+/// </td>
+/// <td>
+///
 /// [`a.view()`][.view()] or [`a.reborrow()`][ArrayView::reborrow()]
 ///
 /// </td>
@@ -812,6 +888,11 @@ pub type Ixs = isize;
 /// </td>
 /// <td>
 ///
+/// [`a.view_mut()`][.view_mut()]
+///
+/// </td>
+/// <td>
+///
 /// illegal
 ///
 /// </td>
@@ -830,7 +911,7 @@ pub type Ixs = isize;
 /// equivalent with dim `D2` (e.g. converting from dynamic dim to const dim)
 ///
 /// </td>
-/// <td colspan="4">
+/// <td colspan="5">
 ///
 /// [`a.into_dimensionality::<D2>()`][.into_dimensionality()]
 ///
@@ -845,7 +926,7 @@ pub type Ixs = isize;
 /// equivalent with dim `IxDyn`
 ///
 /// </td>
-/// <td colspan="4">
+/// <td colspan="5">
 ///
 /// [`a.into_dyn()`][.into_dyn()]
 ///
@@ -860,7 +941,7 @@ pub type Ixs = isize;
 /// `Array<B, D>` (new element type)
 ///
 /// </td>
-/// <td colspan="4">
+/// <td colspan="5">
 ///
 /// [`a.map(|x| x.do_your_conversion())`][.map()]
 ///
@@ -886,8 +967,8 @@ pub type Ixs = isize;
 /// `Array<A, D>` | `Vec<A>` | [`.into_raw_vec()`](type.Array.html#method.into_raw_vec)<sup>[1](#into_raw_vec)</sup>
 /// `&ArrayBase<S, D>` | `&[A]` | [`.as_slice()`](#method.as_slice)<sup>[2](#req_contig_std)</sup>, [`.as_slice_memory_order()`](#method.as_slice_memory_order)<sup>[3](#req_contig)</sup>
 /// `&mut ArrayBase<S: DataMut, D>` | `&mut [A]` | [`.as_slice_mut()`](#method.as_slice_mut)<sup>[2](#req_contig_std)</sup>, [`.as_slice_memory_order_mut()`](#method.as_slice_memory_order_mut)<sup>[3](#req_contig)</sup>
-/// `ArrayView<A, D>` | `&[A]` | [`.into_slice()`](type.ArrayView.html#method.into_slice)
-/// `ArrayViewMut<A, D>` | `&mut [A]` | [`.into_slice()`](type.ArrayViewMut.html#method.into_slice)
+/// `ArrayView<A, D>` | `&[A]` | [`.to_slice()`](type.ArrayView.html#method.to_slice)<sup>[2](#req_contig_std)</sup>
+/// `ArrayViewMut<A, D>` | `&mut [A]` | [`.into_slice()`](type.ArrayViewMut.html#method.into_slice)<sup>[2](#req_contig_std)</sup>
 /// `Array0<A>` | `A` | [`.into_scalar()`](type.Array.html#method.into_scalar)
 ///
 /// <sup><a name="into_raw_vec">1</a></sup>Returns the data in memory order.
@@ -898,7 +979,8 @@ pub type Ixs = isize;
 /// <sup><a name="req_contig">3</a></sup>Works only if the array is contiguous.
 ///
 /// The table above does not include all the constructors; it only shows
-/// conversions to/from `Vec`s/slices. See below for more constructors.
+/// conversions to/from `Vec`s/slices. See
+/// [below](#constructor-methods-for-owned-arrays) for more constructors.
 ///
 /// [ArrayView::reborrow()]: type.ArrayView.html#method.reborrow
 /// [ArrayViewMut::reborrow()]: type.ArrayViewMut.html#method.reborrow
@@ -911,6 +993,101 @@ pub type Ixs = isize;
 /// [.view()]: #method.view
 /// [.view_mut()]: #method.view_mut
 ///
+/// ### Conversions from Nested `Vec`s/`Array`s
+///
+/// It's generally a good idea to avoid nested `Vec`/`Array` types, such as
+/// `Vec<Vec<A>>` or `Vec<Array2<A>>` because:
+///
+/// * they require extra heap allocations compared to a single `Array`,
+///
+/// * they can scatter data all over memory (because of multiple allocations),
+///
+/// * they cause unnecessary indirection (traversing multiple pointers to reach
+///   the data),
+///
+/// * they don't enforce consistent shape within the nested
+///   `Vec`s/`ArrayBase`s, and
+///
+/// * they are generally more difficult to work with.
+///
+/// The most common case where users might consider using nested
+/// `Vec`s/`Array`s is when creating an array by appending rows/subviews in a
+/// loop, where the rows/subviews are computed within the loop. However, there
+/// are better ways than using nested `Vec`s/`Array`s.
+///
+/// If you know ahead-of-time the shape of the final array, the cleanest
+/// solution is to allocate the final array before the loop, and then assign
+/// the data to it within the loop, like this:
+///
+/// ```rust
+/// use ndarray::{array, Array2, Axis};
+///
+/// let mut arr = Array2::zeros((2, 3));
+/// for (i, mut row) in arr.axis_iter_mut(Axis(0)).enumerate() {
+///     // Perform calculations and assign to `row`; this is a trivial example:
+///     row.fill(i);
+/// }
+/// assert_eq!(arr, array![[0, 0, 0], [1, 1, 1]]);
+/// ```
+///
+/// If you don't know ahead-of-time the shape of the final array, then the
+/// cleanest solution is generally to append the data to a flat `Vec`, and then
+/// convert it to an `Array` at the end with
+/// [`::from_shape_vec()`](#method.from_shape_vec). You just have to be careful
+/// that the layout of the data (the order of the elements in the flat `Vec`)
+/// is correct.
+///
+/// ```rust
+/// use ndarray::{array, Array2};
+///
+/// # fn main() -> Result<(), Box<std::error::Error>> {
+/// let ncols = 3;
+/// let mut data = Vec::new();
+/// let mut nrows = 0;
+/// for i in 0..2 {
+///     // Compute `row` and append it to `data`; this is a trivial example:
+///     let row = vec![i; ncols];
+///     data.extend_from_slice(&row);
+///     nrows += 1;
+/// }
+/// let arr = Array2::from_shape_vec((nrows, ncols), data)?;
+/// assert_eq!(arr, array![[0, 0, 0], [1, 1, 1]]);
+/// # Ok(())
+/// # }
+/// ```
+///
+/// If neither of these options works for you, and you really need to convert
+/// nested `Vec`/`Array` instances to an `Array`, the cleanest solution is
+/// generally to use
+/// [`Iterator::flatten()`](https://doc.rust-lang.org/std/iter/trait.Iterator.html#method.flatten)
+/// to get a flat `Vec`, and then convert the `Vec` to an `Array` with
+/// [`::from_shape_vec()`](#method.from_shape_vec), like this:
+///
+/// ```rust
+/// use ndarray::{array, Array2, Array3};
+///
+/// # fn main() -> Result<(), Box<std::error::Error>> {
+/// let nested: Vec<Array2<i32>> = vec![
+///     array![[1, 2, 3], [4, 5, 6]],
+///     array![[7, 8, 9], [10, 11, 12]],
+/// ];
+/// let inner_shape = nested[0].dim();
+/// let shape = (nested.len(), inner_shape.0, inner_shape.1);
+/// let flat: Vec<i32> = nested.iter().flatten().cloned().collect();
+/// let arr = Array3::from_shape_vec(shape, flat)?;
+/// assert_eq!(arr, array![
+///     [[1, 2, 3], [4, 5, 6]],
+///     [[7, 8, 9], [10, 11, 12]],
+/// ]);
+/// # Ok(())
+/// # }
+/// ```
+///
+/// Note that this implementation assumes that the nested `Vec`s are all the
+/// same shape and that the `Vec` is non-empty. Depending on your application,
+/// it may be a good idea to add checks for these assumptions and possibly
+/// choose a different way to handle the empty case.
+///
 // # For implementors
 //
 // All methods must uphold the following constraints:
@@ -1024,14 +1201,15 @@ pub type Ixs = isize;
 //
 // [`.offset()`]: https://doc.rust-lang.org/stable/std/primitive.pointer.html#method.offset-1
 pub struct ArrayBase<S, D>
-    where S: Data
+where
+    S: RawData,
 {
     /// Data buffer / ownership information. (If owned, contains the data
     /// buffer; if borrowed, contains the lifetime and mutability.)
     data: S,
     /// A non-null and aligned pointer into the buffer held by `data`; may
     /// point anywhere in its range.
-    ptr: *mut S::Elem,
+    ptr: std::ptr::NonNull<S::Elem>,
     /// The lengths of the axes.
     dim: D,
     /// The element count stride per axis. To be parsed as `isize`.
@@ -1043,7 +1221,7 @@ pub struct ArrayBase<S, D>
 /// It can act as both an owner as the data as well as a shared reference (view like).
 ///
 /// **Note: this type alias is obsolete.** See the equivalent [`ArcArray`] instead.
-// Use soon: #[deprecated(note="RcArray is replaced by ArcArray")]
+#[deprecated(note = "`RcArray` has been renamed to `ArcArray`")]
 pub type RcArray<A, D> = ArrayBase<OwnedRcRepr<A>, D>;
 
 /// An array where the data has shared ownership and is copy on write.
@@ -1093,6 +1271,27 @@ pub type ArcArray<A, D> = ArrayBase<OwnedArcRepr<A>, D>;
 /// and so on.
 pub type Array<A, D> = ArrayBase<OwnedRepr<A>, D>;
 
+/// An array with copy-on-write behavior.
+///
+/// An `CowArray` represents either a uniquely owned array or a view of an
+/// array. The `'a` corresponds to the lifetime of the view variant.
+///
+/// This type is analogous to
+/// [`std::borrow::Cow`](https://doc.rust-lang.org/std/borrow/enum.Cow.html).
+/// If a `CowArray` instance is the immutable view variant, then calling a
+/// method for mutating elements in the array will cause it to be converted
+/// into the owned variant (by cloning all the elements) before the
+/// modification is performed.
+///
+/// Array views have all the methods of an array (see [`ArrayBase`][ab]).
+///
+/// See also [`ArcArray`](type.ArcArray.html), which also provides
+/// copy-on-write behavior but has a reference-counted pointer to the data
+/// instead of either a view or a uniquely owned copy.
+///
+/// [ab]: struct.ArrayBase.html
+pub type CowArray<'a, A, D> = ArrayBase<CowRepr<'a, A>, D>;
+
 /// A read-only array view.
 ///
 /// An array view represents an array or a part of it, created from
@@ -1123,6 +1322,57 @@ pub type ArrayView<'a, A, D> = ArrayBase<ViewRepr<&'a A>, D>;
 /// [ab]: struct.ArrayBase.html
 pub type ArrayViewMut<'a, A, D> = ArrayBase<ViewRepr<&'a mut A>, D>;
 
+/// A read-only array view without a lifetime.
+///
+/// This is similar to [`ArrayView`] but does not carry any lifetime or
+/// ownership information, and its data cannot be read without an unsafe
+/// conversion into an [`ArrayView`]. The relationship between `RawArrayView`
+/// and [`ArrayView`] is somewhat analogous to the relationship between `*const
+/// T` and `&T`, but `RawArrayView` has additional requirements that `*const T`
+/// does not, such as alignment and non-nullness.
+///
+/// [`ArrayView`]: type.ArrayView.html
+///
+/// The `RawArrayView<A, D>` is parameterized by `A` for the element type and
+/// `D` for the dimensionality.
+///
+/// Raw array views have all the methods of an array (see
+/// [`ArrayBase`](struct.ArrayBase.html)).
+///
+/// See also [`RawArrayViewMut`](type.RawArrayViewMut.html).
+///
+/// # Warning
+///
+/// You can't use this type wih an arbitrary raw pointer; see
+/// [`from_shape_ptr`](#method.from_shape_ptr) for details.
+pub type RawArrayView<A, D> = ArrayBase<RawViewRepr<*const A>, D>;
+
+/// A mutable array view without a lifetime.
+///
+/// This is similar to [`ArrayViewMut`] but does not carry any lifetime or
+/// ownership information, and its data cannot be read/written without an
+/// unsafe conversion into an [`ArrayViewMut`]. The relationship between
+/// `RawArrayViewMut` and [`ArrayViewMut`] is somewhat analogous to the
+/// relationship between `*mut T` and `&mut T`, but `RawArrayViewMut` has
+/// additional requirements that `*mut T` does not, such as alignment and
+/// non-nullness.
+///
+/// [`ArrayViewMut`]: type.ArrayViewMut.html
+///
+/// The `RawArrayViewMut<A, D>` is parameterized by `A` for the element type
+/// and `D` for the dimensionality.
+///
+/// Raw array views have all the methods of an array (see
+/// [`ArrayBase`](struct.ArrayBase.html)).
+///
+/// See also [`RawArrayView`](type.RawArrayView.html).
+///
+/// # Warning
+///
+/// You can't use this type wih an arbitrary raw pointer; see
+/// [`from_shape_ptr`](#method.from_shape_ptr) for details.
+pub type RawArrayViewMut<A, D> = ArrayBase<RawViewRepr<*mut A>, D>;
+
 /// Array's representation.
 ///
 /// *Don’t use this type directly—use the type alias
@@ -1134,7 +1384,7 @@ pub struct OwnedRepr<A>(Vec<A>);
 ///
 /// *Don’t use this type directly—use the type alias
 /// [`RcArray`](type.RcArray.html) for the array type!*
-#[deprecated(note="RcArray is replaced by ArcArray")]
+#[deprecated(note = "RcArray is replaced by ArcArray")]
 pub use self::OwnedArcRepr as OwnedRcRepr;
 
 /// ArcArray's representation.
@@ -1150,6 +1400,23 @@ impl<A> Clone for OwnedArcRepr<A> {
     }
 }
 
+/// Array pointer’s representation.
+///
+/// *Don’t use this type directly—use the type aliases
+/// [`RawArrayView`](type.RawArrayView.html) /
+/// [`RawArrayViewMut`](type.RawArrayViewMut.html) for the array type!*
+#[derive(Copy, Clone)]
+// This is just a marker type, to carry the mutability and element type.
+pub struct RawViewRepr<A> {
+    ptr: PhantomData<A>,
+}
+
+impl<A> RawViewRepr<A> {
+    #[inline(always)]
+    fn new() -> Self {
+        RawViewRepr { ptr: PhantomData }
+    }
+}
 
 /// Array view’s representation.
 ///
@@ -1169,6 +1436,35 @@ impl<A> ViewRepr<A> {
     }
 }
 
+/// CowArray's representation.
+///
+/// *Don't use this type directly—use the type alias
+/// [`CowArray`](type.CowArray.html) for the array type!*
+pub enum CowRepr<'a, A> {
+    /// Borrowed data.
+    View(ViewRepr<&'a A>),
+    /// Owned data.
+    Owned(OwnedRepr<A>),
+}
+
+impl<'a, A> CowRepr<'a, A> {
+    /// Returns `true` iff the data is the `View` variant.
+    pub fn is_view(&self) -> bool {
+        match self {
+            CowRepr::View(_) => true,
+            CowRepr::Owned(_) => false,
+        }
+    }
+
+    /// Returns `true` iff the data is the `Owned` variant.
+    pub fn is_owned(&self) -> bool {
+        match self {
+            CowRepr::View(_) => false,
+            CowRepr::Owned(_) => true,
+        }
+    }
+}
+
 mod impl_clone;
 
 mod impl_constructors;
@@ -1178,20 +1474,27 @@ mod impl_owned_array;
 
 /// Private Methods
 impl<A, S, D> ArrayBase<S, D>
-    where S: Data<Elem=A>, D: Dimension
+where
+    S: Data<Elem = A>,
+    D: Dimension,
 {
     #[inline]
-    fn broadcast_unwrap<E>(&self, dim: E) -> ArrayView<A, E>
-        where E: Dimension,
+    fn broadcast_unwrap<E>(&self, dim: E) -> ArrayView<'_, A, E>
+    where
+        E: Dimension,
     {
         #[cold]
         #[inline(never)]
         fn broadcast_panic<D, E>(from: &D, to: &E) -> !
-            where D: Dimension,
-                  E: Dimension,
+        where
+            D: Dimension,
+            E: Dimension,
         {
-            panic!("ndarray: could not broadcast array from shape: {:?} to: {:?}",
-                   from.slice(), to.slice())
+            panic!(
+                "ndarray: could not broadcast array from shape: {:?} to: {:?}",
+                from.slice(),
+                to.slice()
+            )
         }
 
         match self.broadcast(dim.clone()) {
@@ -1203,17 +1506,16 @@ impl<A, S, D> ArrayBase<S, D>
     // Broadcast to dimension `E`, without checking that the dimensions match
     // (Checked in debug assertions).
     #[inline]
-    fn broadcast_assume<E>(&self, dim: E) -> ArrayView<A, E>
-        where E: Dimension,
+    fn broadcast_assume<E>(&self, dim: E) -> ArrayView<'_, A, E>
+    where
+        E: Dimension,
     {
         let dim = dim.into_dimension();
         debug_assert_eq!(self.shape(), dim.slice());
         let ptr = self.ptr;
         let mut strides = dim.clone();
         strides.slice_mut().copy_from_slice(self.strides.slice());
-        unsafe {
-            ArrayView::new_(ptr, dim, strides)
-        }
+        unsafe { ArrayView::new_(ptr.as_ptr(), dim, strides) }
     }
 
     fn raw_strides(&self) -> D {
@@ -1223,24 +1525,21 @@ impl<A, S, D> ArrayBase<S, D>
     /// Apply closure `f` to each element in the array, in whatever
     /// order is the fastest to visit.
     fn unordered_foreach_mut<F>(&mut self, mut f: F)
-        where S: DataMut,
-              F: FnMut(&mut A)
+    where
+        S: DataMut,
+        F: FnMut(&mut A),
     {
         if let Some(slc) = self.as_slice_memory_order_mut() {
-            // FIXME: Use for loop when slice iterator is perf is restored
-            for i in 0..slc.len() {
-                f(&mut slc[i]);
+            slc.iter_mut().for_each(f);
+        } else {
+            for row in self.inner_rows_mut() {
+                row.into_iter_().fold((), |(), elt| f(elt));
             }
-            return;
-        }
-        for row in self.inner_rows_mut() {
-            row.into_iter_().fold((), |(), elt| f(elt));
         }
     }
 
     /// Remove array axis `axis` and return the result.
-    fn try_remove_axis(self, axis: Axis) -> ArrayBase<S, D::Smaller>
-    {
+    fn try_remove_axis(self, axis: Axis) -> ArrayBase<S, D::Smaller> {
         let d = self.dim.try_remove_axis(axis);
         let s = self.strides.try_remove_axis(axis);
         ArrayBase {
@@ -1252,21 +1551,24 @@ impl<A, S, D> ArrayBase<S, D>
     }
 
     /// n-d generalization of rows, just like inner iter
-    fn inner_rows(&self) -> iterators::Lanes<A, D::Smaller>
-    {
+    fn inner_rows(&self) -> iterators::Lanes<'_, A, D::Smaller> {
         let n = self.ndim();
         Lanes::new(self.view(), Axis(n.saturating_sub(1)))
     }
 
     /// n-d generalization of rows, just like inner iter
-    fn inner_rows_mut(&mut self) -> iterators::LanesMut<A, D::Smaller>
-        where S: DataMut
+    fn inner_rows_mut(&mut self) -> iterators::LanesMut<'_, A, D::Smaller>
+    where
+        S: DataMut,
     {
         let n = self.ndim();
         LanesMut::new(self.view_mut(), Axis(n.saturating_sub(1)))
     }
 }
 
+// parallel methods
+#[cfg(feature = "rayon")]
+pub mod parallel;
 
 mod impl_1d;
 mod impl_2d;
@@ -1277,11 +1579,17 @@ mod numeric;
 pub mod linalg;
 
 mod impl_ops;
-pub use impl_ops::ScalarOperand;
+pub use crate::impl_ops::ScalarOperand;
 
 // Array view methods
 mod impl_views;
 
+// Array raw view methods
+mod impl_raw_views;
+
+// Copy-on-write array methods
+mod impl_cow;
+
 /// A contiguous array shape of n dimensions.
 ///
 /// Either c- or f- memory ordered (*c* a.k.a *row major* is the default).
@@ -1298,3 +1606,8 @@ pub struct StrideShape<D> {
     strides: D,
     custom: bool,
 }
+
+/// Returns `true` if the pointer is aligned.
+pub(crate) fn is_aligned<T>(ptr: *const T) -> bool {
+    (ptr as usize) % ::std::mem::align_of::<T>() == 0
+}
