@@ -6,41 +6,38 @@
 // option. This file may not be copied, modified, or distributed
 // except according to those terms.
 
-use imp_prelude::*;
-use numeric_util;
+use crate::imp_prelude::*;
+use crate::numeric_util;
 
-use {
-    LinalgScalar,
-    Zip,
-};
+use crate::{LinalgScalar, Zip};
 
 use std::any::TypeId;
 
-#[cfg(feature="blas")]
+#[cfg(feature = "blas")]
 use std::cmp;
-#[cfg(feature="blas")]
+#[cfg(feature = "blas")]
 use std::mem::swap;
-#[cfg(feature="blas")]
+#[cfg(feature = "blas")]
 use std::os::raw::c_int;
 
-#[cfg(feature="blas")]
+#[cfg(feature = "blas")]
 use cblas_sys as blas_sys;
-#[cfg(feature="blas")]
-use cblas_sys::{CblasNoTrans, CblasTrans, CblasRowMajor, CBLAS_LAYOUT};
+#[cfg(feature = "blas")]
+use cblas_sys::{CblasNoTrans, CblasRowMajor, CblasTrans, CBLAS_LAYOUT};
 
 /// len of vector before we use blas
-#[cfg(feature="blas")]
+#[cfg(feature = "blas")]
 const DOT_BLAS_CUTOFF: usize = 32;
 /// side of matrix before we use blas
-#[cfg(feature="blas")]
+#[cfg(feature = "blas")]
 const GEMM_BLAS_CUTOFF: usize = 7;
-#[cfg(feature="blas")]
+#[cfg(feature = "blas")]
 #[allow(non_camel_case_types)]
 type blas_index = c_int; // blas index type
 
-
 impl<A, S> ArrayBase<S, Ix1>
-    where S: Data<Elem=A>,
+where
+    S: Data<Elem = A>,
 {
     /// Perform dot product or matrix multiplication of arrays `self` and `rhs`.
     ///
@@ -60,14 +57,16 @@ impl<A, S> ArrayBase<S, Ix1>
     /// *Note:* If enabled, uses blas `dot` for elements of `f32, f64` when memory
     /// layout allows.
     pub fn dot<Rhs>(&self, rhs: &Rhs) -> <Self as Dot<Rhs>>::Output
-        where Self: Dot<Rhs>
+    where
+        Self: Dot<Rhs>,
     {
         Dot::dot(self, rhs)
     }
 
     fn dot_generic<S2>(&self, rhs: &ArrayBase<S2, Ix1>) -> A
-        where S2: Data<Elem=A>,
-              A: LinalgScalar,
+    where
+        S2: Data<Elem = A>,
+        A: LinalgScalar,
     {
         debug_assert_eq!(self.len(), rhs.len());
         assert!(self.len() == rhs.len());
@@ -79,24 +78,26 @@ impl<A, S> ArrayBase<S, Ix1>
         let mut sum = A::zero();
         for i in 0..self.len() {
             unsafe {
-                sum = sum.clone() + self.uget(i).clone() * rhs.uget(i).clone();
+                sum = sum + *self.uget(i) * *rhs.uget(i);
             }
         }
         sum
     }
 
-    #[cfg(not(feature="blas"))]
+    #[cfg(not(feature = "blas"))]
     fn dot_impl<S2>(&self, rhs: &ArrayBase<S2, Ix1>) -> A
-        where S2: Data<Elem=A>,
-              A: LinalgScalar,
+    where
+        S2: Data<Elem = A>,
+        A: LinalgScalar,
     {
         self.dot_generic(rhs)
     }
 
-    #[cfg(feature="blas")]
+    #[cfg(feature = "blas")]
     fn dot_impl<S2>(&self, rhs: &ArrayBase<S2, Ix1>) -> A
-        where S2: Data<Elem=A>,
-              A: LinalgScalar,
+    where
+        S2: Data<Elem = A>,
+        A: LinalgScalar,
     {
         // Use only if the vector is large enough to be worth it
         if self.len() >= DOT_BLAS_CUTOFF {
@@ -104,28 +105,27 @@ impl<A, S> ArrayBase<S, Ix1>
             assert!(self.len() == rhs.len());
             macro_rules! dot {
                 ($ty:ty, $func:ident) => {{
-            if blas_compat_1d::<$ty, _>(self) && blas_compat_1d::<$ty, _>(rhs) {
-                unsafe {
-                    let (lhs_ptr, n, incx) = blas_1d_params(self.ptr,
-                                                            self.len(),
-                                                            self.strides()[0]);
-                    let (rhs_ptr, _, incy) = blas_1d_params(rhs.ptr,
-                                                            rhs.len(),
-                                                            rhs.strides()[0]);
-                    let ret = blas_sys::$func(
-                        n,
-                        lhs_ptr as *const $ty,
-                        incx,
-                        rhs_ptr as *const $ty,
-                        incy);
-                    return cast_as::<$ty, A>(&ret);
-                }
-            }
-                }}
+                    if blas_compat_1d::<$ty, _>(self) && blas_compat_1d::<$ty, _>(rhs) {
+                        unsafe {
+                            let (lhs_ptr, n, incx) =
+                                blas_1d_params(self.ptr.as_ptr(), self.len(), self.strides()[0]);
+                            let (rhs_ptr, _, incy) =
+                                blas_1d_params(rhs.ptr.as_ptr(), rhs.len(), rhs.strides()[0]);
+                            let ret = blas_sys::$func(
+                                n,
+                                lhs_ptr as *const $ty,
+                                incx,
+                                rhs_ptr as *const $ty,
+                                incy,
+                            );
+                            return cast_as::<$ty, A>(&ret);
+                        }
+                    }
+                }};
             }
 
-            dot!{f32, cblas_sdot};
-            dot!{f64, cblas_ddot};
+            dot! {f32, cblas_sdot};
+            dot! {f64, cblas_ddot};
         }
         self.dot_generic(rhs)
     }
@@ -136,10 +136,12 @@ impl<A, S> ArrayBase<S, Ix1>
 /// BLAS wants a pointer to the element with lowest address,
 /// which agrees with our pointer for non-negative strides, but
 /// is at the opposite end for negative strides.
-#[cfg(feature="blas")]
-unsafe fn blas_1d_params<A>(ptr: *const A, len: usize, stride: isize)
-    -> (*const A, blas_index, blas_index)
-{
+#[cfg(feature = "blas")]
+unsafe fn blas_1d_params<A>(
+    ptr: *const A,
+    len: usize,
+    stride: isize,
+) -> (*const A, blas_index, blas_index) {
     // [x x x x]
     //        ^--ptr
     //        stride = -1
@@ -165,9 +167,10 @@ pub trait Dot<Rhs> {
 }
 
 impl<A, S, S2> Dot<ArrayBase<S2, Ix1>> for ArrayBase<S, Ix1>
-    where S: Data<Elem=A>,
-          S2: Data<Elem=A>,
-          A: LinalgScalar,
+where
+    S: Data<Elem = A>,
+    S2: Data<Elem = A>,
+    A: LinalgScalar,
 {
     type Output = A;
 
@@ -179,16 +182,16 @@ impl<A, S, S2> Dot<ArrayBase<S2, Ix1>> for ArrayBase<S, Ix1>
     /// **Panics** if the arrays are not of the same length.<br>
     /// *Note:* If enabled, uses blas `dot` for elements of `f32, f64` when memory
     /// layout allows.
-    fn dot(&self, rhs: &ArrayBase<S2, Ix1>) -> A
-    {
+    fn dot(&self, rhs: &ArrayBase<S2, Ix1>) -> A {
         self.dot_impl(rhs)
     }
 }
 
 impl<A, S, S2> Dot<ArrayBase<S2, Ix2>> for ArrayBase<S, Ix1>
-    where S: Data<Elem=A>,
-          S2: Data<Elem=A>,
-          A: LinalgScalar,
+where
+    S: Data<Elem = A>,
+    S2: Data<Elem = A>,
+    A: LinalgScalar,
 {
     type Output = Array<A, Ix1>;
 
@@ -201,14 +204,14 @@ impl<A, S, S2> Dot<ArrayBase<S2, Ix2>> for ArrayBase<S, Ix1>
     /// Return a result array with shape *N*.
     ///
     /// **Panics** if shapes are incompatible.
-    fn dot(&self, rhs: &ArrayBase<S2, Ix2>) -> Array<A, Ix1>
-    {
+    fn dot(&self, rhs: &ArrayBase<S2, Ix2>) -> Array<A, Ix1> {
         rhs.t().dot(self)
     }
 }
 
 impl<A, S> ArrayBase<S, Ix2>
-    where S: Data<Elem=A>,
+where
+    S: Data<Elem = A>,
 {
     /// Perform matrix multiplication of rectangular arrays `self` and `rhs`.
     ///
@@ -240,20 +243,21 @@ impl<A, S> ArrayBase<S, Ix2>
     /// );
     /// ```
     pub fn dot<Rhs>(&self, rhs: &Rhs) -> <Self as Dot<Rhs>>::Output
-        where Self: Dot<Rhs>
+    where
+        Self: Dot<Rhs>,
     {
         Dot::dot(self, rhs)
     }
 }
 
 impl<A, S, S2> Dot<ArrayBase<S2, Ix2>> for ArrayBase<S, Ix2>
-    where S: Data<Elem=A>,
-          S2: Data<Elem=A>,
-          A: LinalgScalar,
+where
+    S: Data<Elem = A>,
+    S2: Data<Elem = A>,
+    A: LinalgScalar,
 {
     type Output = Array2<A>;
-    fn dot(&self, b: &ArrayBase<S2, Ix2>) -> Array2<A>
-    {
+    fn dot(&self, b: &ArrayBase<S2, Ix2>) -> Array2<A> {
         let a = self.view();
         let b = b.view();
         let ((m, k), (k2, n)) = (a.dim(), b.dim());
@@ -281,11 +285,13 @@ impl<A, S, S2> Dot<ArrayBase<S2, Ix2>> for ArrayBase<S, Ix2>
 #[inline(never)]
 fn dot_shape_error(m: usize, k: usize, k2: usize, n: usize) -> ! {
     match m.checked_mul(n) {
-        Some(len) if len <= ::std::isize::MAX as usize => {},
+        Some(len) if len <= ::std::isize::MAX as usize => {}
         _ => panic!("ndarray: shape {} × {} overflows isize", m, n),
     }
-    panic!("ndarray: inputs {} × {} and {} × {} are not compatible for matrix multiplication",
-           m, k, k2, n);
+    panic!(
+        "ndarray: inputs {} × {} and {} × {} are not compatible for matrix multiplication",
+        m, k, k2, n
+    );
 }
 
 #[cold]
@@ -305,13 +311,13 @@ fn general_dot_shape_error(m: usize, k: usize, k2: usize, n: usize, c1: usize, c
 ///
 /// **Panics** if shapes are incompatible.
 impl<A, S, S2> Dot<ArrayBase<S2, Ix1>> for ArrayBase<S, Ix2>
-    where S: Data<Elem=A>,
-          S2: Data<Elem=A>,
-          A: LinalgScalar,
+where
+    S: Data<Elem = A>,
+    S2: Data<Elem = A>,
+    A: LinalgScalar,
 {
     type Output = Array<A, Ix1>;
-    fn dot(&self, rhs: &ArrayBase<S2, Ix1>) -> Array<A, Ix1>
-    {
+    fn dot(&self, rhs: &ArrayBase<S2, Ix1>) -> Array<A, Ix1> {
         let ((m, a), n) = (self.dim(), rhs.dim());
         if a != n {
             dot_shape_error(m, a, n, 1);
@@ -327,8 +333,9 @@ impl<A, S, S2> Dot<ArrayBase<S2, Ix1>> for ArrayBase<S, Ix2>
 }
 
 impl<A, S, D> ArrayBase<S, D>
-    where S: Data<Elem=A>,
-          D: Dimension,
+where
+    S: Data<Elem = A>,
+    D: Dimension,
 {
     /// Perform the operation `self += alpha * rhs` efficiently, where
     /// `alpha` is a scalar and `rhs` is another array. This operation is
@@ -338,10 +345,11 @@ impl<A, S, D> ArrayBase<S, D>
     ///
     /// **Panics** if broadcasting isn’t possible.
     pub fn scaled_add<S2, E>(&mut self, alpha: A, rhs: &ArrayBase<S2, E>)
-        where S: DataMut,
-              S2: Data<Elem=A>,
-              A: LinalgScalar,
-              E: Dimension,
+    where
+        S: DataMut,
+        S2: Data<Elem = A>,
+        A: LinalgScalar,
+        E: Dimension,
     {
         self.zip_mut_with(rhs, move |y, &x| *y = *y + (alpha * x));
     }
@@ -349,22 +357,23 @@ impl<A, S, D> ArrayBase<S, D>
 
 // mat_mul_impl uses ArrayView arguments to send all array kinds into
 // the same instantiated implementation.
-#[cfg(not(feature="blas"))]
+#[cfg(not(feature = "blas"))]
 use self::mat_mul_general as mat_mul_impl;
 
-#[cfg(feature="blas")]
-fn mat_mul_impl<A>(alpha: A,
-                   lhs: &ArrayView2<A>,
-                   rhs: &ArrayView2<A>,
-                   beta: A,
-                   c: &mut ArrayViewMut2<A>)
-    where A: LinalgScalar,
+#[cfg(feature = "blas")]
+fn mat_mul_impl<A>(
+    alpha: A,
+    lhs: &ArrayView2<'_, A>,
+    rhs: &ArrayView2<'_, A>,
+    beta: A,
+    c: &mut ArrayViewMut2<'_, A>,
+) where
+    A: LinalgScalar,
 {
     // size cutoff for using BLAS
     let cut = GEMM_BLAS_CUTOFF;
     let ((mut m, a), (_, mut n)) = (lhs.dim(), rhs.dim());
-    if !(m > cut || n > cut || a > cut) ||
-        !(same_type::<A, f32>() || same_type::<A, f64>()) {
+    if !(m > cut || n > cut || a > cut) || !(same_type::<A, f32>() || same_type::<A, f64>()) {
         return mat_mul_general(alpha, lhs, rhs, beta, c);
     }
     {
@@ -420,25 +429,25 @@ fn mat_mul_impl<A>(alpha: A,
                     // Where Op is notrans/trans/conjtrans
                     unsafe {
                         blas_sys::$gemm(
-                        CblasRowMajor,
-                        lhs_trans,
-                        rhs_trans,
-                        m as blas_index, // m, rows of Op(a)
-                        n as blas_index, // n, cols of Op(b)
-                        k as blas_index, // k, cols of Op(a)
-                        cast_as(&alpha),        // alpha
-                        lhs_.ptr as *const _,   // a
-                        lhs_stride, // lda
-                        rhs_.ptr as *const _,   // b
-                        rhs_stride, // ldb
-                        cast_as(&beta),         // beta
-                        c_.ptr as *mut _,       // c
-                        c_stride, // ldc
-                    );
+                            CblasRowMajor,
+                            lhs_trans,
+                            rhs_trans,
+                            m as blas_index,               // m, rows of Op(a)
+                            n as blas_index,               // n, cols of Op(b)
+                            k as blas_index,               // k, cols of Op(a)
+                            cast_as(&alpha),               // alpha
+                            lhs_.ptr.as_ptr() as *const _, // a
+                            lhs_stride,                    // lda
+                            rhs_.ptr.as_ptr() as *const _, // b
+                            rhs_stride,                    // ldb
+                            cast_as(&beta),                // beta
+                            c_.ptr.as_ptr() as *mut _,     // c
+                            c_stride,                      // ldc
+                        );
                     }
-                return;
+                    return;
                 }
-            }
+            };
         }
         gemm!(f32, cblas_sgemm);
         gemm!(f64, cblas_dgemm);
@@ -447,12 +456,14 @@ fn mat_mul_impl<A>(alpha: A,
 }
 
 /// C ← α A B + β C
-fn mat_mul_general<A>(alpha: A,
-                      lhs: &ArrayView2<A>,
-                      rhs: &ArrayView2<A>,
-                      beta: A,
-                      c: &mut ArrayViewMut2<A>)
-    where A: LinalgScalar,
+fn mat_mul_general<A>(
+    alpha: A,
+    lhs: &ArrayView2<'_, A>,
+    rhs: &ArrayView2<'_, A>,
+    beta: A,
+    c: &mut ArrayViewMut2<'_, A>,
+) where
+    A: LinalgScalar,
 {
     let ((m, k), (_, n)) = (lhs.dim(), rhs.dim());
 
@@ -464,7 +475,9 @@ fn mat_mul_general<A>(alpha: A,
     if same_type::<A, f32>() {
         unsafe {
             ::matrixmultiply::sgemm(
-                m, k, n,
+                m,
+                k,
+                n,
                 cast_as(&alpha),
                 ap as *const _,
                 lhs.strides()[0],
@@ -474,13 +487,16 @@ fn mat_mul_general<A>(alpha: A,
                 rhs.strides()[1],
                 cast_as(&beta),
                 cp as *mut _,
-                rsc, csc
+                rsc,
+                csc,
             );
         }
     } else if same_type::<A, f64>() {
         unsafe {
             ::matrixmultiply::dgemm(
-                m, k, n,
+                m,
+                k,
+                n,
                 cast_as(&alpha),
                 ap as *const _,
                 lhs.strides()[0],
@@ -490,12 +506,13 @@ fn mat_mul_general<A>(alpha: A,
                 rhs.strides()[1],
                 cast_as(&beta),
                 cp as *mut _,
-                rsc, csc
+                rsc,
+                csc,
             );
         }
     } else {
         // It's a no-op if `c` has zero length.
-        if c.len() == 0 {
+        if c.is_empty() {
             return;
         }
 
@@ -509,8 +526,11 @@ fn mat_mul_general<A>(alpha: A,
         loop {
             unsafe {
                 let elt = c.uget_mut((i, j));
-                *elt = *elt * beta + alpha * (0..k).fold(A::zero(),
-                    move |s, x| s + *lhs.uget((i, x)) * *rhs.uget((x, j)));
+                *elt = *elt * beta
+                    + alpha
+                        * (0..k).fold(A::zero(), move |s, x| {
+                            s + *lhs.uget((i, x)) * *rhs.uget((x, j))
+                        });
             }
             j += 1;
             if j == n {
@@ -535,15 +555,17 @@ fn mat_mul_general<A>(alpha: A,
 /// *Note:* If enabled, uses blas `gemm` for elements of `f32, f64` when memory
 /// layout allows.  The default matrixmultiply backend is otherwise used for
 /// `f32, f64` for all memory layouts.
-pub fn general_mat_mul<A, S1, S2, S3>(alpha: A,
-                                      a: &ArrayBase<S1, Ix2>,
-                                      b: &ArrayBase<S2, Ix2>,
-                                      beta: A,
-                                      c: &mut ArrayBase<S3, Ix2>)
-    where S1: Data<Elem=A>,
-          S2: Data<Elem=A>,
-          S3: DataMut<Elem=A>,
-          A: LinalgScalar,
+pub fn general_mat_mul<A, S1, S2, S3>(
+    alpha: A,
+    a: &ArrayBase<S1, Ix2>,
+    b: &ArrayBase<S2, Ix2>,
+    beta: A,
+    c: &mut ArrayBase<S3, Ix2>,
+) where
+    S1: Data<Elem = A>,
+    S2: Data<Elem = A>,
+    S3: DataMut<Elem = A>,
+    A: LinalgScalar,
 {
     let ((m, k), (k2, n)) = (a.dim(), b.dim());
     let (m2, n2) = c.dim();
@@ -564,15 +586,18 @@ pub fn general_mat_mul<A, S1, S2, S3>(alpha: A,
 /// ***Panics*** if array shapes are not compatible<br>
 /// *Note:* If enabled, uses blas `gemv` for elements of `f32, f64` when memory
 /// layout allows.
-pub fn general_mat_vec_mul<A, S1, S2, S3>(alpha: A,
-                                          a: &ArrayBase<S1, Ix2>,
-                                          x: &ArrayBase<S2, Ix1>,
-                                          beta: A,
-                                          y: &mut ArrayBase<S3, Ix1>)
-    where S1: Data<Elem=A>,
-          S2: Data<Elem=A>,
-          S3: DataMut<Elem=A>,
-          A: LinalgScalar,
+#[allow(clippy::collapsible_if)]
+pub fn general_mat_vec_mul<A, S1, S2, S3>(
+    alpha: A,
+    a: &ArrayBase<S1, Ix2>,
+    x: &ArrayBase<S2, Ix1>,
+    beta: A,
+    y: &mut ArrayBase<S3, Ix1>,
+) where
+    S1: Data<Elem = A>,
+    S2: Data<Elem = A>,
+    S3: DataMut<Elem = A>,
+    A: LinalgScalar,
 {
     let ((m, k), k2) = (a.dim(), x.dim());
     let m2 = y.dim();
@@ -583,13 +608,19 @@ pub fn general_mat_vec_mul<A, S1, S2, S3>(alpha: A,
         macro_rules! gemv {
             ($ty:ty, $gemv:ident) => {
                 if let Some(layout) = blas_layout::<$ty, _>(&a) {
-                    if blas_compat_1d::<$ty, _>(&x)
-                        && blas_compat_1d::<$ty, _>(&y)
-                    {
+                    if blas_compat_1d::<$ty, _>(&x) && blas_compat_1d::<$ty, _>(&y) {
+                        // Determine stride between rows or columns. Note that the stride is
+                        // adjusted to at least `k` or `m` to handle the case of a matrix with a
+                        // trivial (length 1) dimension, since the stride for the trivial dimension
+                        // may be arbitrary.
                         let a_trans = CblasNoTrans;
                         let a_stride = match layout {
-                            CBLAS_LAYOUT::CblasRowMajor => a.strides()[0] as blas_index,
-                            CBLAS_LAYOUT::CblasColMajor => a.strides()[1] as blas_index,
+                            CBLAS_LAYOUT::CblasRowMajor => {
+                                a.strides()[0].max(k as isize) as blas_index
+                            }
+                            CBLAS_LAYOUT::CblasColMajor => {
+                                a.strides()[1].max(m as isize) as blas_index
+                            }
                         };
 
                         let x_stride = x.strides()[0] as blas_index;
@@ -599,22 +630,22 @@ pub fn general_mat_vec_mul<A, S1, S2, S3>(alpha: A,
                             blas_sys::$gemv(
                                 layout,
                                 a_trans,
-                                m as blas_index, // m, rows of Op(a)
-                                k as blas_index, // n, cols of Op(a)
-                                cast_as(&alpha),     // alpha
-                                a.ptr as *const _,   // a
-                                a_stride, // lda
-                                x.ptr as *const _,   // x
+                                m as blas_index,            // m, rows of Op(a)
+                                k as blas_index,            // n, cols of Op(a)
+                                cast_as(&alpha),            // alpha
+                                a.ptr.as_ptr() as *const _, // a
+                                a_stride,                   // lda
+                                x.ptr.as_ptr() as *const _, // x
                                 x_stride,
-                                cast_as(&beta),      // beta
-                                y.ptr as *mut _,     // x
+                                cast_as(&beta),           // beta
+                                y.ptr.as_ptr() as *mut _, // x
                                 y_stride,
                             );
                         }
                         return;
                     }
                 }
-            }
+            };
         }
         #[cfg(feature = "blas")]
         gemv!(f32, cblas_sgemv);
@@ -624,17 +655,13 @@ pub fn general_mat_vec_mul<A, S1, S2, S3>(alpha: A,
         /* general */
 
         if beta.is_zero() {
-            Zip::from(a.outer_iter())
-                .and(y)
-                .apply(|row, elt| {
-                    *elt = row.dot(x) * alpha;
-                });
+            Zip::from(a.outer_iter()).and(y).apply(|row, elt| {
+                *elt = row.dot(x) * alpha;
+            });
         } else {
-            Zip::from(a.outer_iter())
-                .and(y)
-                .apply(|row, elt| {
-                    *elt = *elt * beta + row.dot(x) * alpha;
-                });
+            Zip::from(a.outer_iter()).and(y).apply(|row, elt| {
+                *elt = *elt * beta + row.dot(x) * alpha;
+            });
         }
     }
 }
@@ -650,16 +677,15 @@ fn same_type<A: 'static, B: 'static>() -> bool {
 // **Panics** if `A` and `B` are not the same type
 fn cast_as<A: 'static + Copy, B: 'static + Copy>(a: &A) -> B {
     assert!(same_type::<A, B>());
-    unsafe {
-        ::std::ptr::read(a as *const _ as *const B)
-    }
+    unsafe { ::std::ptr::read(a as *const _ as *const B) }
 }
 
-#[cfg(feature="blas")]
+#[cfg(feature = "blas")]
 fn blas_compat_1d<A, S>(a: &ArrayBase<S, Ix1>) -> bool
-    where S: Data,
-          A: 'static,
-          S::Elem: 'static,
+where
+    S: Data,
+    A: 'static,
+    S::Elem: 'static,
 {
     if !same_type::<A, S::Elem>() {
         return false;
@@ -668,24 +694,24 @@ fn blas_compat_1d<A, S>(a: &ArrayBase<S, Ix1>) -> bool
         return false;
     }
     let stride = a.strides()[0];
-    if stride > blas_index::max_value() as isize ||
-        stride < blas_index::min_value() as isize {
+    if stride > blas_index::max_value() as isize || stride < blas_index::min_value() as isize {
         return false;
     }
     true
 }
 
-#[cfg(feature="blas")]
+#[cfg(feature = "blas")]
 enum MemoryOrder {
     C,
     F,
 }
 
-#[cfg(feature="blas")]
+#[cfg(feature = "blas")]
 fn blas_row_major_2d<A, S>(a: &ArrayBase<S, Ix2>) -> bool
-    where S: Data,
-          A: 'static,
-          S::Elem: 'static,
+where
+    S: Data,
+    A: 'static,
+    S::Elem: 'static,
 {
     if !same_type::<A, S::Elem>() {
         return false;
@@ -693,11 +719,12 @@ fn blas_row_major_2d<A, S>(a: &ArrayBase<S, Ix2>) -> bool
     is_blas_2d(&a.dim, &a.strides, MemoryOrder::C)
 }
 
-#[cfg(feature="blas")]
+#[cfg(feature = "blas")]
 fn blas_column_major_2d<A, S>(a: &ArrayBase<S, Ix2>) -> bool
-    where S: Data,
-          A: 'static,
-          S::Elem: 'static,
+where
+    S: Data,
+    A: 'static,
+    S::Elem: 'static,
 {
     if !same_type::<A, S::Elem>() {
         return false;
@@ -705,7 +732,7 @@ fn blas_column_major_2d<A, S>(a: &ArrayBase<S, Ix2>) -> bool
     is_blas_2d(&a.dim, &a.strides, MemoryOrder::F)
 }
 
-#[cfg(feature="blas")]
+#[cfg(feature = "blas")]
 fn is_blas_2d(dim: &Ix2, stride: &Ix2, order: MemoryOrder) -> bool {
     let (m, n) = dim.into_pattern();
     let s0 = stride[0] as isize;
@@ -720,28 +747,27 @@ fn is_blas_2d(dim: &Ix2, stride: &Ix2, order: MemoryOrder) -> bool {
     if s0 < 1 || s1 < 1 {
         return false;
     }
-    if (s0 > blas_index::max_value() as isize || s0 < blas_index::min_value() as isize) ||
-        (s1 > blas_index::max_value() as isize || s1 < blas_index::min_value() as isize)
+    if (s0 > blas_index::max_value() as isize || s0 < blas_index::min_value() as isize)
+        || (s1 > blas_index::max_value() as isize || s1 < blas_index::min_value() as isize)
     {
         return false;
     }
-    if m > blas_index::max_value() as usize ||
-        n > blas_index::max_value() as usize
-    {
+    if m > blas_index::max_value() as usize || n > blas_index::max_value() as usize {
         return false;
     }
     true
 }
 
-#[cfg(feature="blas")]
+#[cfg(feature = "blas")]
 fn blas_layout<A, S>(a: &ArrayBase<S, Ix2>) -> Option<CBLAS_LAYOUT>
-    where S: Data,
-          A: 'static,
-          S::Elem: 'static,
+where
+    S: Data,
+    A: 'static,
+    S::Elem: 'static,
 {
     if blas_row_major_2d::<A, _>(a) {
         Some(CBLAS_LAYOUT::CblasRowMajor)
-    }  else if blas_column_major_2d::<A, _>(a) {
+    } else if blas_column_major_2d::<A, _>(a) {
         Some(CBLAS_LAYOUT::CblasColMajor)
     } else {
         None
@@ -749,7 +775,7 @@ fn blas_layout<A, S>(a: &ArrayBase<S, Ix2>) -> Option<CBLAS_LAYOUT>
 }
 
 #[cfg(test)]
-#[cfg(feature="blas")]
+#[cfg(feature = "blas")]
 mod blas_tests {
     use super::*;
 
