@@ -6,19 +6,20 @@
 // option. This file may not be copied, modified, or distributed
 // except according to those terms.
 
-use std::ops::{Add, Div, Mul};
-use libnum::{self, One, Zero, Float};
 use itertools::free::enumerate;
+use num_traits::{self, Float, FromPrimitive, Zero};
+use std::ops::{Add, Div, Mul};
 
-use imp_prelude::*;
-use numeric_util;
+use crate::imp_prelude::*;
+use crate::numeric_util;
 
-use {FoldWhile, Zip};
+use crate::{FoldWhile, Zip};
 
 /// # Numerical Methods for Arrays
 impl<A, S, D> ArrayBase<S, D>
-    where S: Data<Elem=A>,
-          D: Dimension,
+where
+    S: Data<Elem = A>,
+    D: Dimension,
 {
     /// Return the sum of all elements in the array.
     ///
@@ -30,7 +31,8 @@ impl<A, S, D> ArrayBase<S, D>
     /// assert_eq!(a.sum(), 10.);
     /// ```
     pub fn sum(&self) -> A
-        where A: Clone + Add<Output=A> + libnum::Zero,
+    where
+        A: Clone + Add<Output = A> + num_traits::Zero,
     {
         if let Some(slc) = self.as_slice_memory_order() {
             return numeric_util::unrolled_fold(slc, A::zero, A::add);
@@ -46,13 +48,41 @@ impl<A, S, D> ArrayBase<S, D>
         sum
     }
 
+    /// Returns the [arithmetic mean] x̅ of all elements in the array:
+    ///
+    /// ```text
+    ///     1   n
+    /// x̅ = ―   ∑ xᵢ
+    ///     n  i=1
+    /// ```
+    ///
+    /// If the array is empty, `None` is returned.
+    ///
+    /// **Panics** if `A::from_usize()` fails to convert the number of elements in the array.
+    ///
+    /// [arithmetic mean]: https://en.wikipedia.org/wiki/Arithmetic_mean
+    pub fn mean(&self) -> Option<A>
+    where
+        A: Clone + FromPrimitive + Add<Output = A> + Div<Output = A> + Zero,
+    {
+        let n_elements = self.len();
+        if n_elements == 0 {
+            None
+        } else {
+            let n_elements = A::from_usize(n_elements)
+                .expect("Converting number of elements to `A` must not fail.");
+            Some(self.sum() / n_elements)
+        }
+    }
+
     /// Return the sum of all elements in the array.
     ///
     /// *This method has been renamed to `.sum()` and will be deprecated in the
     /// next version.*
     // #[deprecated(note="renamed to `sum`", since="0.13")]
     pub fn scalar_sum(&self) -> A
-        where A: Clone + Add<Output=A> + libnum::Zero,
+    where
+        A: Clone + Add<Output = A> + num_traits::Zero,
     {
         self.sum()
     }
@@ -67,7 +97,8 @@ impl<A, S, D> ArrayBase<S, D>
     /// assert_eq!(a.product(), 24.);
     /// ```
     pub fn product(&self) -> A
-        where A: Clone + Mul<Output=A> + libnum::One,
+    where
+        A: Clone + Mul<Output = A> + num_traits::One,
     {
         if let Some(slc) = self.as_slice_memory_order() {
             return numeric_util::unrolled_fold(slc, A::one, A::mul);
@@ -88,20 +119,21 @@ impl<A, S, D> ArrayBase<S, D>
     /// ```
     /// use ndarray::{aview0, aview1, arr2, Axis};
     ///
-    /// let a = arr2(&[[1., 2.],
-    ///                [3., 4.]]);
+    /// let a = arr2(&[[1., 2., 3.],
+    ///                [4., 5., 6.]]);
     /// assert!(
-    ///     a.sum_axis(Axis(0)) == aview1(&[4., 6.]) &&
-    ///     a.sum_axis(Axis(1)) == aview1(&[3., 7.]) &&
+    ///     a.sum_axis(Axis(0)) == aview1(&[5., 7., 9.]) &&
+    ///     a.sum_axis(Axis(1)) == aview1(&[6., 15.]) &&
     ///
-    ///     a.sum_axis(Axis(0)).sum_axis(Axis(0)) == aview0(&10.)
+    ///     a.sum_axis(Axis(0)).sum_axis(Axis(0)) == aview0(&21.)
     /// );
     /// ```
     ///
     /// **Panics** if `axis` is out of bounds.
     pub fn sum_axis(&self, axis: Axis) -> Array<A, D::Smaller>
-        where A: Clone + Zero + Add<Output=A>,
-              D: RemoveAxis,
+    where
+        A: Clone + Zero + Add<Output = A>,
+        D: RemoveAxis,
     {
         let n = self.len_of(axis);
         let mut res = Array::zeros(self.raw_dim().remove_axis(axis));
@@ -123,30 +155,37 @@ impl<A, S, D> ArrayBase<S, D>
 
     /// Return mean along `axis`.
     ///
-    /// **Panics** if `axis` is out of bounds or if the length of the axis is
-    /// zero and division by zero panics for type `A`.
+    /// Return `None` if the length of the axis is zero.
+    ///
+    /// **Panics** if `axis` is out of bounds or if `A::from_usize()`
+    /// fails for the axis length.
     ///
     /// ```
-    /// use ndarray::{aview1, arr2, Axis};
+    /// use ndarray::{aview0, aview1, arr2, Axis};
     ///
-    /// let a = arr2(&[[1., 2.],
-    ///                [3., 4.]]);
+    /// let a = arr2(&[[1., 2., 3.],
+    ///                [4., 5., 6.]]);
     /// assert!(
-    ///     a.mean_axis(Axis(0)) == aview1(&[2.0, 3.0]) &&
-    ///     a.mean_axis(Axis(1)) == aview1(&[1.5, 3.5])
+    ///     a.mean_axis(Axis(0)).unwrap() == aview1(&[2.5, 3.5, 4.5]) &&
+    ///     a.mean_axis(Axis(1)).unwrap() == aview1(&[2., 5.]) &&
+    ///
+    ///     a.mean_axis(Axis(0)).unwrap().mean_axis(Axis(0)).unwrap() == aview0(&3.5)
     /// );
     /// ```
-    pub fn mean_axis(&self, axis: Axis) -> Array<A, D::Smaller>
-        where A: Clone + Zero + One + Add<Output=A> + Div<Output=A>,
-              D: RemoveAxis,
+    pub fn mean_axis(&self, axis: Axis) -> Option<Array<A, D::Smaller>>
+    where
+        A: Clone + Zero + FromPrimitive + Add<Output = A> + Div<Output = A>,
+        D: RemoveAxis,
     {
-        let n = self.len_of(axis);
-        let sum = self.sum_axis(axis);
-        let mut cnt = A::zero();
-        for _ in 0..n {
-            cnt = cnt + A::one();
+        let axis_length = self.len_of(axis);
+        if axis_length == 0 {
+            None
+        } else {
+            let axis_length =
+                A::from_usize(axis_length).expect("Converting axis length to `A` must not fail.");
+            let sum = self.sum_axis(axis);
+            Some(sum / aview0(&axis_length))
         }
-        sum / &aview0(&cnt)
     }
 
     /// Return variance along `axis`.
@@ -174,8 +213,11 @@ impl<A, S, D> ArrayBase<S, D>
     ///     n  i=1
     /// ```
     ///
-    /// **Panics** if `ddof` is greater than or equal to the length of the
-    /// axis, if `axis` is out of bounds, or if the length of the axis is zero.
+    /// and `n` is the length of the axis.
+    ///
+    /// **Panics** if `ddof` is less than zero or greater than `n`, if `axis`
+    /// is out of bounds, or if `A::from_usize()` fails for any any of the
+    /// numbers in the range `0..=n`.
     ///
     /// # Example
     ///
@@ -190,27 +232,28 @@ impl<A, S, D> ArrayBase<S, D>
     /// ```
     pub fn var_axis(&self, axis: Axis, ddof: A) -> Array<A, D::Smaller>
     where
-        A: Float,
+        A: Float + FromPrimitive,
         D: RemoveAxis,
     {
-        let mut count = A::zero();
+        let zero = A::from_usize(0).expect("Converting 0 to `A` must not fail.");
+        let n = A::from_usize(self.len_of(axis)).expect("Converting length to `A` must not fail.");
+        assert!(
+            !(ddof < zero || ddof > n),
+            "`ddof` must not be less than zero or greater than the length of \
+             the axis",
+        );
+        let dof = n - ddof;
         let mut mean = Array::<A, _>::zeros(self.dim.remove_axis(axis));
         let mut sum_sq = Array::<A, _>::zeros(self.dim.remove_axis(axis));
-        for subview in self.axis_iter(axis) {
-            count = count + A::one();
-            azip!(mut mean, mut sum_sq, x (subview) in {
+        for (i, subview) in self.axis_iter(axis).enumerate() {
+            let count = A::from_usize(i + 1).expect("Converting index to `A` must not fail.");
+            azip!((mean in &mut mean, sum_sq in &mut sum_sq, &x in &subview) {
                 let delta = x - *mean;
                 *mean = *mean + delta / count;
                 *sum_sq = (x - *mean).mul_add(delta, *sum_sq);
             });
         }
-        if ddof >= count {
-            panic!("`ddof` needs to be strictly smaller than the length \
-                    of the axis you are computing the variance for!")
-        } else {
-            let dof = count - ddof;
-            sum_sq.mapv_into(|s| s / dof)
-        }
+        sum_sq.mapv_into(|s| s / dof)
     }
 
     /// Return standard deviation along `axis`.
@@ -225,9 +268,9 @@ impl<A, S, D> ArrayBase<S, D>
     /// The standard deviation is defined as:
     ///
     /// ```text
-    ///                    1       n
-    /// stddev = sqrt ( ――――――――   ∑ (xᵢ - x̅)² )
-    ///                 n - ddof  i=1
+    ///               ⎛    1       n          ⎞
+    /// stddev = sqrt ⎜ ――――――――   ∑ (xᵢ - x̅)²⎟
+    ///               ⎝ n - ddof  i=1         ⎠
     /// ```
     ///
     /// where
@@ -238,8 +281,11 @@ impl<A, S, D> ArrayBase<S, D>
     ///     n  i=1
     /// ```
     ///
-    /// **Panics** if `ddof` is greater than or equal to the length of the
-    /// axis, if `axis` is out of bounds, or if the length of the axis is zero.
+    /// and `n` is the length of the axis.
+    ///
+    /// **Panics** if `ddof` is less than zero or greater than `n`, if `axis`
+    /// is out of bounds, or if `A::from_usize()` fails for any any of the
+    /// numbers in the range `0..=n`.
     ///
     /// # Example
     ///
@@ -254,7 +300,7 @@ impl<A, S, D> ArrayBase<S, D>
     /// ```
     pub fn std_axis(&self, axis: Axis, ddof: A) -> Array<A, D::Smaller>
     where
-        A: Float,
+        A: Float + FromPrimitive,
         D: RemoveAxis,
     {
         self.var_axis(axis, ddof).mapv_into(|x| x.sqrt())
@@ -266,10 +312,15 @@ impl<A, S, D> ArrayBase<S, D>
     /// If their shapes disagree, `rhs` is broadcast to the shape of `self`.
     ///
     /// **Panics** if broadcasting to the same shape isn’t possible.
+    #[deprecated(
+        note = "Use `abs_diff_eq` - it requires the `approx` crate feature",
+        since = "0.13.0"
+    )]
     pub fn all_close<S2, E>(&self, rhs: &ArrayBase<S2, E>, tol: A) -> bool
-        where A: Float,
-              S2: Data<Elem=A>,
-              E: Dimension,
+    where
+        A: Float,
+        S2: Data<Elem = A>,
+        E: Dimension,
     {
         !Zip::from(self)
             .and(rhs.broadcast_unwrap(self.raw_dim()))
@@ -279,7 +330,7 @@ impl<A, S, D> ArrayBase<S, D>
                 } else {
                     FoldWhile::Done(())
                 }
-            }).is_done()
+            })
+            .is_done()
     }
 }
-
