@@ -3,7 +3,6 @@
 //! can feel free to extend those traits without worrying about it
 //! being a breaking change for other implementations.
 
-
 /// If this type is pub but not publicly reachable, third parties
 /// can't name it and can't implement traits using it.
 pub struct PrivateMarker;
@@ -12,15 +11,15 @@ macro_rules! private_decl {
     () => {
         /// This trait is private to implement; this method exists to make it
         /// impossible to implement outside the crate.
-        fn __private__(&self) -> ::private::PrivateMarker;
+        fn __private__(&self) -> crate::private::PrivateMarker;
     }
 }
 
 macro_rules! private_impl {
     () => {
         #[doc(hidden)]
-        fn __private__(&self) -> ::private::PrivateMarker {
-            ::private::PrivateMarker
+        fn __private__(&self) -> crate::private::PrivateMarker {
+            crate::private::PrivateMarker
         }
     }
 }
