@@ -5,11 +5,11 @@
 // <LICENSE-MIT or http://opensource.org/licenses/MIT>, at your
 // option. This file may not be copied, modified, or distributed
 // except according to those terms.
-use error::{ShapeError, ErrorKind};
-use std::ops::{Deref, Range, RangeFrom, RangeFull, RangeInclusive, RangeTo, RangeToInclusive};
+use crate::error::{ErrorKind, ShapeError};
+use crate::Dimension;
 use std::fmt;
 use std::marker::PhantomData;
-use super::Dimension;
+use std::ops::{Deref, Range, RangeFrom, RangeFull, RangeInclusive, RangeTo, RangeToInclusive};
 
 /// A slice (range with step size).
 ///
@@ -48,11 +48,7 @@ impl Slice {
     /// (This method checks with a debug assertion that `step` is not zero.)
     pub fn new(start: isize, end: Option<isize>, step: isize) -> Slice {
         debug_assert_ne!(step, 0, "Slice::new: step must be nonzero");
-        Slice {
-            start,
-            end,
-            step,
-        }
+        Slice { start, end, step }
     }
 
     /// Create a new `Slice` with the given step size (multiplied with the
@@ -63,7 +59,10 @@ impl Slice {
     #[inline]
     pub fn step_by(self, step: isize) -> Self {
         debug_assert_ne!(step, 0, "Slice::step_by: step must be nonzero");
-        Slice { step: self.step * step, ..self }
+        Slice {
+            step: self.step * step,
+            ..self
+        }
     }
 }
 
@@ -105,13 +104,13 @@ pub enum SliceOrIndex {
     Index(isize),
 }
 
-copy_and_clone!{SliceOrIndex}
+copy_and_clone! {SliceOrIndex}
 
 impl SliceOrIndex {
     /// Returns `true` if `self` is a `Slice` value.
     pub fn is_slice(&self) -> bool {
         match self {
-            &SliceOrIndex::Slice { .. } => true,
+            SliceOrIndex::Slice { .. } => true,
             _ => false,
         }
     }
@@ -119,7 +118,7 @@ impl SliceOrIndex {
     /// Returns `true` if `self` is an `Index` value.
     pub fn is_index(&self) -> bool {
         match self {
-            &SliceOrIndex::Index(_) => true,
+            SliceOrIndex::Index(_) => true,
             _ => false,
         }
     }
@@ -148,7 +147,7 @@ impl SliceOrIndex {
 }
 
 impl fmt::Display for SliceOrIndex {
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         match *self {
             SliceOrIndex::Index(index) => write!(f, "{}", index)?,
             SliceOrIndex::Slice { start, end, step } => {
@@ -316,10 +315,7 @@ where
     /// consistent with `indices`.
     #[doc(hidden)]
     pub unsafe fn new_unchecked(indices: T, out_dim: PhantomData<D>) -> SliceInfo<T, D> {
-        SliceInfo {
-            out_dim: out_dim,
-            indices: indices,
-        }
+        SliceInfo { out_dim, indices }
     }
 }
 
@@ -339,7 +335,7 @@ where
         }
         Ok(SliceInfo {
             out_dim: PhantomData,
-            indices: indices,
+            indices,
         })
     }
 }
@@ -414,10 +410,9 @@ where
     }
 }
 
-
 #[doc(hidden)]
 pub trait SliceNextDim<D1, D2> {
-    fn next_dim(&self, PhantomData<D1>) -> PhantomData<D2>;
+    fn next_dim(&self, _: PhantomData<D1>) -> PhantomData<D2>;
 }
 
 macro_rules! impl_slicenextdim_equal {
@@ -427,7 +422,7 @@ macro_rules! impl_slicenextdim_equal {
                 PhantomData
             }
         }
-    }
+    };
 }
 impl_slicenextdim_equal!(isize);
 impl_slicenextdim_equal!(usize);
@@ -500,10 +495,9 @@ impl_slicenextdim_larger!((), Slice);
 /// # Example
 ///
 /// ```
-/// #[macro_use]
 /// extern crate ndarray;
 ///
-/// use ndarray::{Array2, ArrayView2};
+/// use ndarray::{s, Array2, ArrayView2};
 ///
 /// fn laplacian(v: &ArrayView2<f32>) -> Array2<f32> {
 ///     -4. * &v.slice(s![1..-1, 1..-1])
@@ -533,7 +527,6 @@ impl_slicenextdim_larger!((), Slice);
 /// For example,
 ///
 /// ```
-/// # #[macro_use]
 /// # extern crate ndarray;
 /// #
 /// # use ndarray::prelude::*;
@@ -557,7 +550,7 @@ macro_rules! s(
                 #[allow(unsafe_code)]
                 unsafe {
                     $crate::SliceInfo::new_unchecked(
-                        [$($stack)* s!(@convert r, $s)],
+                        [$($stack)* $crate::s!(@convert r, $s)],
                         out_dim,
                     )
                 }
@@ -572,7 +565,7 @@ macro_rules! s(
                 #[allow(unsafe_code)]
                 unsafe {
                     $crate::SliceInfo::new_unchecked(
-                        [$($stack)* s!(@convert r)],
+                        [$($stack)* $crate::s!(@convert r)],
                         out_dim,
                     )
                 }
@@ -581,19 +574,19 @@ macro_rules! s(
     };
     // convert a..b;c into @convert(a..b, c), final item, trailing comma
     (@parse $dim:expr, [$($stack:tt)*] $r:expr;$s:expr ,) => {
-        s![@parse $dim, [$($stack)*] $r;$s]
+        $crate::s![@parse $dim, [$($stack)*] $r;$s]
     };
     // convert a..b into @convert(a..b), final item, trailing comma
     (@parse $dim:expr, [$($stack:tt)*] $r:expr ,) => {
-        s![@parse $dim, [$($stack)*] $r]
+        $crate::s![@parse $dim, [$($stack)*] $r]
     };
     // convert a..b;c into @convert(a..b, c)
     (@parse $dim:expr, [$($stack:tt)*] $r:expr;$s:expr, $($t:tt)*) => {
         match $r {
             r => {
-                s![@parse
+                $crate::s![@parse
                    $crate::SliceNextDim::next_dim(&r, $dim),
-                   [$($stack)* s!(@convert r, $s),]
+                   [$($stack)* $crate::s!(@convert r, $s),]
                    $($t)*
                 ]
             }
@@ -603,14 +596,25 @@ macro_rules! s(
     (@parse $dim:expr, [$($stack:tt)*] $r:expr, $($t:tt)*) => {
         match $r {
             r => {
-                s![@parse
+                $crate::s![@parse
                    $crate::SliceNextDim::next_dim(&r, $dim),
-                   [$($stack)* s!(@convert r),]
+                   [$($stack)* $crate::s!(@convert r),]
                    $($t)*
                 ]
             }
         }
     };
+    // empty call, i.e. `s![]`
+    (@parse ::std::marker::PhantomData::<$crate::Ix0>, []) => {
+        {
+            #[allow(unsafe_code)]
+            unsafe {
+                $crate::SliceInfo::new_unchecked([], ::std::marker::PhantomData::<$crate::Ix0>)
+            }
+        }
+    };
+    // Catch-all clause for syntax errors
+    (@parse $($t:tt)*) => { compile_error!("Invalid syntax in s![] call.") };
     // convert range/index into SliceOrIndex
     (@convert $r:expr) => {
         <$crate::SliceOrIndex as ::std::convert::From<_>>::from($r)
@@ -622,6 +626,6 @@ macro_rules! s(
     ($($t:tt)*) => {
         // The extra `*&` is a workaround for this compiler bug:
         // https://github.com/rust-lang/rust/issues/23014
-        &*&s![@parse ::std::marker::PhantomData::<$crate::Ix0>, [] $($t)*]
+        &*&$crate::s![@parse ::std::marker::PhantomData::<$crate::Ix0>, [] $($t)*]
     };
 );
