@@ -6,8 +6,8 @@
 // option. This file may not be copied, modified, or distributed
 // except according to those terms.
 
-use imp_prelude::*;
-use error::{ShapeError, ErrorKind, from_kind};
+use crate::error::{from_kind, ErrorKind, ShapeError};
+use crate::imp_prelude::*;
 
 /// Stack arrays along the given axis.
 ///
@@ -29,12 +29,15 @@ use error::{ShapeError, ErrorKind, from_kind};
 ///                  [3., 3.]]))
 /// );
 /// ```
-pub fn stack<'a, A, D>(axis: Axis, arrays: &[ArrayView<'a, A, D>])
-    -> Result<Array<A, D>, ShapeError>
-    where A: Copy,
-          D: RemoveAxis
+pub fn stack<'a, A, D>(
+    axis: Axis,
+    arrays: &[ArrayView<'a, A, D>],
+) -> Result<Array<A, D>, ShapeError>
+where
+    A: Copy,
+    D: RemoveAxis,
 {
-    if arrays.len() == 0 {
+    if arrays.is_empty() {
         return Err(from_kind(ErrorKind::Unsupported));
     }
     let mut res_dim = arrays[0].raw_dim();
@@ -42,7 +45,10 @@ pub fn stack<'a, A, D>(axis: Axis, arrays: &[ArrayView<'a, A, D>])
         return Err(from_kind(ErrorKind::OutOfBounds));
     }
     let common_dim = res_dim.remove_axis(axis);
-    if arrays.iter().any(|a| a.raw_dim().remove_axis(axis) != common_dim) {
+    if arrays
+        .iter()
+        .any(|a| a.raw_dim().remove_axis(axis) != common_dim)
+    {
         return Err(from_kind(ErrorKind::IncompatibleShape));
     }
 
@@ -56,7 +62,7 @@ pub fn stack<'a, A, D>(axis: Axis, arrays: &[ArrayView<'a, A, D>])
     unsafe {
         v.set_len(size);
     }
-    let mut res = try!(Array::from_shape_vec(res_dim, v));
+    let mut res = Array::from_shape_vec(res_dim, v)?;
 
     {
         let mut assign_view = res.view_mut();
@@ -80,10 +86,9 @@ pub fn stack<'a, A, D>(axis: Axis, arrays: &[ArrayView<'a, A, D>])
 /// ***Panics*** if the `stack` function would return an error.
 ///
 /// ```
-/// #[macro_use(stack)]
 /// extern crate ndarray;
 ///
-/// use ndarray::{arr2, Axis};
+/// use ndarray::{arr2, stack, Axis};
 ///
 /// # fn main() {
 ///
