@@ -9,14 +9,14 @@
 #[macro_use]
 mod zipmacro;
 
-use imp_prelude::*;
-use IntoDimension;
-use NdIndex;
-use Layout;
+use crate::imp_prelude::*;
+use crate::IntoDimension;
+use crate::Layout;
+use crate::NdIndex;
 
-use layout::{CORDER, FORDER};
-use layout::LayoutPriv;
-use indexes::{Indices, indices};
+use crate::indexes::{indices, Indices};
+use crate::layout::LayoutPriv;
+use crate::layout::{CORDER, FORDER};
 
 /// Return if the expression is a break value.
 macro_rules! fold_while {
@@ -25,29 +25,30 @@ macro_rules! fold_while {
             FoldWhile::Continue(x) => x,
             x => return x,
         }
-    }
+    };
 }
 
-
 /// Broadcast an array so that it acts like a larger size and/or shape array.
 ///
 /// See [broadcasting][1] for more information.
 ///
 /// [1]: struct.ArrayBase.html#broadcasting
 trait Broadcast<E>
-    where E: IntoDimension,
+where
+    E: IntoDimension,
 {
-    type Output: NdProducer<Dim=E::Dim>;
+    type Output: NdProducer<Dim = E::Dim>;
     /// Broadcast the array to the new dimensions `shape`.
     ///
     /// ***Panics*** if broadcasting isn’t possible.
     fn broadcast_unwrap(self, shape: E) -> Self::Output;
-    private_decl!{}
+    private_decl! {}
 }
 
 impl<S, D> ArrayBase<S, D>
-    where S: Data,
-          D: Dimension,
+where
+    S: RawData,
+    D: Dimension,
 {
     pub(crate) fn layout_impl(&self) -> Layout {
         Layout::new(if self.is_standard_layout() {
@@ -56,7 +57,7 @@ impl<S, D> ArrayBase<S, D>
             } else {
                 CORDER
             }
-        } else if self.ndim() > 1 && self.t().is_standard_layout() {
+        } else if self.ndim() > 1 && self.raw_view().reversed_axes().is_standard_layout() {
             FORDER
         } else {
             0
@@ -65,25 +66,25 @@ impl<S, D> ArrayBase<S, D>
 }
 
 impl<'a, A, D, E> Broadcast<E> for ArrayView<'a, A, D>
-    where E: IntoDimension,
-          D: Dimension,
+where
+    E: IntoDimension,
+    D: Dimension,
 {
     type Output = ArrayView<'a, A, E::Dim>;
     fn broadcast_unwrap(self, shape: E) -> Self::Output {
-        let res: ArrayView<A, E::Dim> = (&self).broadcast_unwrap(shape.into_dimension());
-        unsafe {
-            ArrayView::new_(res.ptr, res.dim, res.strides)
-        }
+        let res: ArrayView<'_, A, E::Dim> = (&self).broadcast_unwrap(shape.into_dimension());
+        unsafe { ArrayView::new_(res.ptr.as_ptr(), res.dim, res.strides) }
     }
-    private_impl!{}
+    private_impl! {}
 }
 
-pub trait Splittable : Sized {
-    fn split_at(self, Axis, Ix) -> (Self, Self);
+pub trait Splittable: Sized {
+    fn split_at(self, axis: Axis, index: Ix) -> (Self, Self);
 }
 
 impl<D> Splittable for D
-    where D: Dimension,
+where
+    D: Dimension,
 {
     fn split_at(self, axis: Axis, index: Ix) -> (Self, Self) {
         let mut d1 = self;
@@ -106,23 +107,29 @@ pub trait IntoNdProducer {
     type Item;
     /// Dimension type of the producer
     type Dim: Dimension;
-    type Output: NdProducer<Dim=Self::Dim, Item=Self::Item>;
+    type Output: NdProducer<Dim = Self::Dim, Item = Self::Item>;
     /// Convert the value into an `NdProducer`.
     fn into_producer(self) -> Self::Output;
 }
 
-impl<P> IntoNdProducer for P where P: NdProducer {
+impl<P> IntoNdProducer for P
+where
+    P: NdProducer,
+{
     type Item = P::Item;
     type Dim = P::Dim;
     type Output = Self;
-    fn into_producer(self) -> Self::Output { self }
+    fn into_producer(self) -> Self::Output {
+        self
+    }
 }
 
 /// A producer of an n-dimensional set of elements;
 /// for example an array view, mutable array view or an iterator
 /// that yields chunks.
 ///
-/// Producers are used as a arguments to `Zip` and `azip!()`.
+/// Producers are used as a arguments to [`Zip`](struct.Zip.html) and
+/// [`azip!()`](macro.azip.html).
 ///
 /// # Comparison to `IntoIterator`
 ///
@@ -150,7 +157,7 @@ pub trait NdProducer {
     // stride (= along a particular axis)
     #[doc(hidden)]
     /// Pointer or stand-in for pointer
-    type Ptr: Offset<Stride=Self::Stride>;
+    type Ptr: Offset<Stride = Self::Stride>;
     #[doc(hidden)]
     /// Pointer stride
     type Stride: Copy;
@@ -166,23 +173,32 @@ pub trait NdProducer {
     #[doc(hidden)]
     fn as_ptr(&self) -> Self::Ptr;
     #[doc(hidden)]
-    unsafe fn as_ref(&self, Self::Ptr) -> Self::Item;
+    unsafe fn as_ref(&self, ptr: Self::Ptr) -> Self::Item;
     #[doc(hidden)]
     unsafe fn uget_ptr(&self, i: &Self::Dim) -> Self::Ptr;
     #[doc(hidden)]
     fn stride_of(&self, axis: Axis) -> <Self::Ptr as Offset>::Stride;
     #[doc(hidden)]
-    #[inline(always)]
     fn contiguous_stride(&self) -> Self::Stride;
     #[doc(hidden)]
-    fn split_at(self, axis: Axis, index: usize) -> (Self, Self) where Self: Sized;
-    private_decl!{}
+    fn split_at(self, axis: Axis, index: usize) -> (Self, Self)
+    where
+        Self: Sized;
+    private_decl! {}
 }
 
-pub trait Offset : Copy {
+pub trait Offset: Copy {
     type Stride: Copy;
     unsafe fn stride_offset(self, s: Self::Stride, index: usize) -> Self;
-    private_decl!{}
+    private_decl! {}
+}
+
+impl<T> Offset for *const T {
+    type Stride = isize;
+    unsafe fn stride_offset(self, s: Self::Stride, index: usize) -> Self {
+        self.offset(s * (index as isize))
+    }
+    private_impl! {}
 }
 
 impl<T> Offset for *mut T {
@@ -190,16 +206,16 @@ impl<T> Offset for *mut T {
     unsafe fn stride_offset(self, s: Self::Stride, index: usize) -> Self {
         self.offset(s * (index as isize))
     }
-    private_impl!{}
+    private_impl! {}
 }
 
-trait ZippableTuple : Sized {
+trait ZippableTuple: Sized {
     type Item;
-    type Ptr: OffsetTuple<Args=Self::Stride> + Copy;
+    type Ptr: OffsetTuple<Args = Self::Stride> + Copy;
     type Dim: Dimension;
     type Stride: Copy;
     fn as_ptr(&self) -> Self::Ptr;
-    unsafe fn as_ref(&self, Self::Ptr) -> Self::Item;
+    unsafe fn as_ref(&self, ptr: Self::Ptr) -> Self::Item;
     unsafe fn uget_ptr(&self, i: &Self::Dim) -> Self::Ptr;
     fn stride_of(&self, index: usize) -> Self::Stride;
     fn contiguous_stride(&self) -> Self::Stride;
@@ -209,8 +225,9 @@ trait ZippableTuple : Sized {
 /// An array reference is an n-dimensional producer of element references
 /// (like ArrayView).
 impl<'a, A: 'a, S, D> IntoNdProducer for &'a ArrayBase<S, D>
-    where D: Dimension,
-          S: Data<Elem=A>,
+where
+    D: Dimension,
+    S: Data<Elem = A>,
 {
     type Item = &'a A;
     type Dim = D;
@@ -223,8 +240,9 @@ impl<'a, A: 'a, S, D> IntoNdProducer for &'a ArrayBase<S, D>
 /// A mutable array reference is an n-dimensional producer of mutable element
 /// references (like ArrayViewMut).
 impl<'a, A: 'a, S, D> IntoNdProducer for &'a mut ArrayBase<S, D>
-    where D: Dimension,
-          S: DataMut<Elem=A>,
+where
+    D: Dimension,
+    S: DataMut<Elem = A>,
 {
     type Item = &'a mut A;
     type Dim = D;
@@ -274,14 +292,13 @@ impl<'a, A: 'a> IntoNdProducer for &'a mut Vec<A> {
     }
 }
 
-impl<'a, A, D: Dimension> NdProducer for ArrayView<'a, A, D>
-{
+impl<'a, A, D: Dimension> NdProducer for ArrayView<'a, A, D> {
     type Item = &'a A;
     type Dim = D;
     type Ptr = *mut A;
     type Stride = isize;
 
-    private_impl!{}
+    private_impl! {}
     #[doc(hidden)]
     fn raw_dim(&self) -> Self::Dim {
         self.raw_dim()
@@ -309,7 +326,7 @@ impl<'a, A, D: Dimension> NdProducer for ArrayView<'a, A, D>
 
     #[doc(hidden)]
     unsafe fn uget_ptr(&self, i: &Self::Dim) -> *mut A {
-        self.ptr.offset(i.index_unchecked(&self.strides))
+        self.ptr.as_ptr().offset(i.index_unchecked(&self.strides))
     }
 
     #[doc(hidden)]
@@ -318,7 +335,9 @@ impl<'a, A, D: Dimension> NdProducer for ArrayView<'a, A, D>
     }
 
     #[inline(always)]
-    fn contiguous_stride(&self) -> Self::Stride { 1 }
+    fn contiguous_stride(&self) -> Self::Stride {
+        1
+    }
 
     #[doc(hidden)]
     fn split_at(self, axis: Axis, index: usize) -> (Self, Self) {
@@ -332,7 +351,7 @@ impl<'a, A, D: Dimension> NdProducer for ArrayViewMut<'a, A, D> {
     type Ptr = *mut A;
     type Stride = isize;
 
-    private_impl!{}
+    private_impl! {}
     #[doc(hidden)]
     fn raw_dim(&self) -> Self::Dim {
         self.raw_dim()
@@ -360,7 +379,7 @@ impl<'a, A, D: Dimension> NdProducer for ArrayViewMut<'a, A, D> {
 
     #[doc(hidden)]
     unsafe fn uget_ptr(&self, i: &Self::Dim) -> *mut A {
-        self.ptr.offset(i.index_unchecked(&self.strides))
+        self.ptr.as_ptr().offset(i.index_unchecked(&self.strides))
     }
 
     #[doc(hidden)]
@@ -369,7 +388,9 @@ impl<'a, A, D: Dimension> NdProducer for ArrayViewMut<'a, A, D> {
     }
 
     #[inline(always)]
-    fn contiguous_stride(&self) -> Self::Stride { 1 }
+    fn contiguous_stride(&self) -> Self::Stride {
+        1
+    }
 
     #[doc(hidden)]
     fn split_at(self, axis: Axis, index: usize) -> (Self, Self) {
@@ -377,6 +398,111 @@ impl<'a, A, D: Dimension> NdProducer for ArrayViewMut<'a, A, D> {
     }
 }
 
+impl<A, D: Dimension> NdProducer for RawArrayView<A, D> {
+    type Item = *const A;
+    type Dim = D;
+    type Ptr = *const A;
+    type Stride = isize;
+
+    private_impl! {}
+    #[doc(hidden)]
+    fn raw_dim(&self) -> Self::Dim {
+        self.raw_dim()
+    }
+
+    #[doc(hidden)]
+    fn equal_dim(&self, dim: &Self::Dim) -> bool {
+        self.dim.equal(dim)
+    }
+
+    #[doc(hidden)]
+    fn as_ptr(&self) -> *const A {
+        self.as_ptr()
+    }
+
+    #[doc(hidden)]
+    fn layout(&self) -> Layout {
+        self.layout_impl()
+    }
+
+    #[doc(hidden)]
+    unsafe fn as_ref(&self, ptr: *const A) -> *const A {
+        ptr
+    }
+
+    #[doc(hidden)]
+    unsafe fn uget_ptr(&self, i: &Self::Dim) -> *const A {
+        self.ptr.as_ptr().offset(i.index_unchecked(&self.strides))
+    }
+
+    #[doc(hidden)]
+    fn stride_of(&self, axis: Axis) -> isize {
+        self.stride_of(axis)
+    }
+
+    #[inline(always)]
+    fn contiguous_stride(&self) -> Self::Stride {
+        1
+    }
+
+    #[doc(hidden)]
+    fn split_at(self, axis: Axis, index: usize) -> (Self, Self) {
+        self.split_at(axis, index)
+    }
+}
+
+impl<A, D: Dimension> NdProducer for RawArrayViewMut<A, D> {
+    type Item = *mut A;
+    type Dim = D;
+    type Ptr = *mut A;
+    type Stride = isize;
+
+    private_impl! {}
+    #[doc(hidden)]
+    fn raw_dim(&self) -> Self::Dim {
+        self.raw_dim()
+    }
+
+    #[doc(hidden)]
+    fn equal_dim(&self, dim: &Self::Dim) -> bool {
+        self.dim.equal(dim)
+    }
+
+    #[doc(hidden)]
+    fn as_ptr(&self) -> *mut A {
+        self.as_ptr() as _
+    }
+
+    #[doc(hidden)]
+    fn layout(&self) -> Layout {
+        self.layout_impl()
+    }
+
+    #[doc(hidden)]
+    unsafe fn as_ref(&self, ptr: *mut A) -> *mut A {
+        ptr
+    }
+
+    #[doc(hidden)]
+    unsafe fn uget_ptr(&self, i: &Self::Dim) -> *mut A {
+        self.ptr.as_ptr().offset(i.index_unchecked(&self.strides))
+    }
+
+    #[doc(hidden)]
+    fn stride_of(&self, axis: Axis) -> isize {
+        self.stride_of(axis)
+    }
+
+    #[inline(always)]
+    fn contiguous_stride(&self) -> Self::Stride {
+        1
+    }
+
+    #[doc(hidden)]
+    fn split_at(self, axis: Axis, index: usize) -> (Self, Self) {
+        self.split_at(axis, index)
+    }
+}
 
 /// Lock step function application across several arrays or other producers.
 ///
@@ -437,7 +563,7 @@ impl<'a, A, D: Dimension> NdProducer for ArrayViewMut<'a, A, D> {
 ///
 /// use ndarray::{Array1, Axis};
 ///
-/// let mut totals = Array1::zeros(a.rows());
+/// let mut totals = Array1::zeros(a.nrows());
 ///
 /// Zip::from(&mut totals)
 ///     .and(a.genrows())
@@ -453,29 +579,32 @@ pub struct Zip<Parts, D> {
     layout: Layout,
 }
 
-impl<P, D> Zip<(P, ), D>
-    where D: Dimension,
-          P: NdProducer<Dim=D>
+impl<P, D> Zip<(P,), D>
+where
+    D: Dimension,
+    P: NdProducer<Dim = D>,
 {
     /// Create a new `Zip` from the input array or other producer `p`.
     ///
     /// The Zip will take the exact dimension of `p` and all inputs
     /// must have the same dimensions (or be broadcast to them).
     pub fn from<IP>(p: IP) -> Self
-        where IP: IntoNdProducer<Dim=D, Output=P, Item=P::Item>
+    where
+        IP: IntoNdProducer<Dim = D, Output = P, Item = P::Item>,
     {
         let array = p.into_producer();
         let dim = array.raw_dim();
         Zip {
             dimension: dim,
             layout: array.layout(),
-            parts: (array, ),
+            parts: (array,),
         }
     }
 }
 impl<P, D> Zip<(Indices<D>, P), D>
-    where D: Dimension + Copy,
-          P: NdProducer<Dim=D>,
+where
+    D: Dimension + Copy,
+    P: NdProducer<Dim = D>,
 {
     /// Create a new `Zip` with an index producer and the producer `p`.
     ///
@@ -484,7 +613,8 @@ impl<P, D> Zip<(Indices<D>, P), D>
     ///
     /// *Note:* Indexed zip has overhead.
     pub fn indexed<IP>(p: IP) -> Self
-        where IP: IntoNdProducer<Dim=D, Output=P, Item=P::Item>
+    where
+        IP: IntoNdProducer<Dim = D, Output = P, Item = P::Item>,
     {
         let array = p.into_producer();
         let dim = array.raw_dim();
@@ -493,15 +623,19 @@ impl<P, D> Zip<(Indices<D>, P), D>
 }
 
 impl<Parts, D> Zip<Parts, D>
-    where D: Dimension,
+where
+    D: Dimension,
 {
-
     fn check<P>(&self, part: &P)
-        where P: NdProducer<Dim=D>
+    where
+        P: NdProducer<Dim = D>,
     {
-        ndassert!(part.equal_dim(&self.dimension),
+        ndassert!(
+            part.equal_dim(&self.dimension),
             "Zip: Producer dimension mismatch, expected: {:?}, got: {:?}",
-            self.dimension, part.raw_dim());
+            self.dimension,
+            part.raw_dim()
+        );
     }
 
     /// Return a the number of element tuples in the Zip
@@ -521,23 +655,32 @@ impl<Parts, D> Zip<Parts, D>
     /// others.
     fn max_stride_axis(&self) -> Axis {
         let i = match self.layout.flag() {
-            FORDER => self.dimension.slice().iter()
-                          .rposition(|&len| len > 1).unwrap_or(self.dimension.ndim() - 1),
+            FORDER => self
+                .dimension
+                .slice()
+                .iter()
+                .rposition(|&len| len > 1)
+                .unwrap_or(self.dimension.ndim() - 1),
             /* corder or default */
-            _ => self.dimension.slice().iter()
-                          .position(|&len| len > 1).unwrap_or(0),
+            _ => self
+                .dimension
+                .slice()
+                .iter()
+                .position(|&len| len > 1)
+                .unwrap_or(0),
         };
         Axis(i)
     }
-
 }
 
 impl<P, D> Zip<P, D>
-    where D: Dimension,
+where
+    D: Dimension,
 {
     fn apply_core<F, Acc>(&mut self, acc: Acc, function: F) -> FoldWhile<Acc>
-        where F: FnMut(Acc, P::Item) -> FoldWhile<Acc>,
-              P: ZippableTuple<Dim=D>,
+    where
+        F: FnMut(Acc, P::Item) -> FoldWhile<Acc>,
+        P: ZippableTuple<Dim = D>,
     {
         if self.layout.is(CORDER | FORDER) {
             self.apply_core_contiguous(acc, function)
@@ -546,8 +689,9 @@ impl<P, D> Zip<P, D>
         }
     }
     fn apply_core_contiguous<F, Acc>(&mut self, mut acc: Acc, mut function: F) -> FoldWhile<Acc>
-        where F: FnMut(Acc, P::Item) -> FoldWhile<Acc>,
-              P: ZippableTuple<Dim=D>,
+    where
+        F: FnMut(Acc, P::Item) -> FoldWhile<Acc>,
+        P: ZippableTuple<Dim = D>,
     {
         debug_assert!(self.layout.is(CORDER | FORDER));
         let size = self.dimension.size();
@@ -563,8 +707,9 @@ impl<P, D> Zip<P, D>
     }
 
     fn apply_core_strided<F, Acc>(&mut self, mut acc: Acc, mut function: F) -> FoldWhile<Acc>
-        where F: FnMut(Acc, P::Item) -> FoldWhile<Acc>,
-              P: ZippableTuple<Dim=D>,
+    where
+        F: FnMut(Acc, P::Item) -> FoldWhile<Acc>,
+        P: ZippableTuple<Dim = D>,
     {
         let n = self.dimension.ndim();
         if n == 0 {
@@ -607,7 +752,6 @@ impl<T> Offset for *mut T {
 }
 */
 
-
 trait OffsetTuple {
     type Args;
     unsafe fn stride_offset(self, stride: Self::Args, index: usize) -> Self;
@@ -636,7 +780,7 @@ macro_rules! offset_impl {
     }
 }
 
-offset_impl!{
+offset_impl! {
     [A ][ a],
     [A B][ a b],
     [A B C][ a b c],
@@ -695,7 +839,7 @@ macro_rules! zipt_impl {
     }
 }
 
-zipt_impl!{
+zipt_impl! {
     [A ][ a],
     [A B][ a b],
     [A B C][ a b c],
@@ -708,7 +852,10 @@ macro_rules! map_impl {
     ($([$notlast:ident $($p:ident)*],)+) => {
         $(
         #[allow(non_snake_case)]
-        impl<D: Dimension, $($p: NdProducer<Dim=D>),*> Zip<($($p,)*), D> {
+        impl<D, $($p),*> Zip<($($p,)*), D>
+            where D: Dimension,
+                  $($p: NdProducer<Dim=D> ,)*
+        {
             /// Apply a function to all elements of the input arrays,
             /// visiting elements in lock step.
             pub fn apply<F>(mut self, mut function: F)
@@ -723,8 +870,44 @@ macro_rules! map_impl {
             /// Apply a fold function to all elements of the input arrays,
             /// visiting elements in lock step.
             ///
-            /// The fold continues while the return value is a
-            /// `FoldWhile::Continue`.
+            /// # Example
+            ///
+            /// The expression `tr(AᵀB)` can be more efficiently computed as
+            /// the equivalent expression `∑ᵢⱼ(A∘B)ᵢⱼ` (i.e. the sum of the
+            /// elements of the entry-wise product). It would be possible to
+            /// evaluate this expression by first computing the entry-wise
+            /// product, `A∘B`, and then computing the elementwise sum of that
+            /// product, but it's possible to do this in a single loop (and
+            /// avoid an extra heap allocation if `A` and `B` can't be
+            /// consumed) by using `Zip`:
+            ///
+            /// ```
+            /// use ndarray::{array, Zip};
+            ///
+            /// let a = array![[1, 5], [3, 7]];
+            /// let b = array![[2, 4], [8, 6]];
+            ///
+            /// // Without using `Zip`. This involves two loops and an extra
+            /// // heap allocation for the result of `&a * &b`.
+            /// let sum_prod_nonzip = (&a * &b).sum();
+            /// // Using `Zip`. This is a single loop without any heap allocations.
+            /// let sum_prod_zip = Zip::from(&a).and(&b).fold(0, |acc, a, b| acc + a * b);
+            ///
+            /// assert_eq!(sum_prod_nonzip, sum_prod_zip);
+            /// ```
+            pub fn fold<F, Acc>(mut self, acc: Acc, mut function: F) -> Acc
+            where
+                F: FnMut(Acc, $($p::Item),*) -> Acc,
+            {
+                self.apply_core(acc, move |acc, args| {
+                    let ($($p,)*) = args;
+                    FoldWhile::Continue(function(acc, $($p),*))
+                }).into_inner()
+            }
+
+            /// Apply a fold function to the input arrays while the return
+            /// value is `FoldWhile::Continue`, visiting elements in lock step.
+            ///
             pub fn fold_while<F, Acc>(mut self, acc: Acc, mut function: F)
                 -> FoldWhile<Acc>
                 where F: FnMut(Acc, $($p::Item),*) -> FoldWhile<Acc>
@@ -735,6 +918,32 @@ macro_rules! map_impl {
                 })
             }
 
+            /// Tests if every element of the iterator matches a predicate.
+            ///
+            /// Returns `true` if `predicate` evaluates to `true` for all elements.
+            /// Returns `true` if the input arrays are empty.
+            ///
+            /// Example:
+            ///
+            /// ```
+            /// use ndarray::{array, Zip};
+            /// let a = array![1, 2, 3];
+            /// let b = array![1, 4, 9];
+            /// assert!(Zip::from(&a).and(&b).all(|&a, &b| a * a == b));
+            /// ```
+            pub fn all<F>(mut self, mut predicate: F) -> bool
+                where F: FnMut($($p::Item),*) -> bool
+            {
+                self.apply_core(true, move |_, args| {
+                    let ($($p,)*) = args;
+                    if predicate($($p),*) {
+                        FoldWhile::Continue(true)
+                    } else {
+                        FoldWhile::Done(false)
+                    }
+                }).into_inner()
+            }
+
             expand_if!(@bool [$notlast]
 
             /// Include the producer `p` in the Zip.
@@ -802,7 +1011,7 @@ macro_rules! map_impl {
     }
 }
 
-map_impl!{
+map_impl! {
     [true P1],
     [true P1 P2],
     [true P1 P2 P3],
@@ -824,7 +1033,7 @@ impl<T> FoldWhile<T> {
     /// Return the inner value
     pub fn into_inner(self) -> T {
         match self {
-            FoldWhile::Continue(x) | FoldWhile::Done(x) => x
+            FoldWhile::Continue(x) | FoldWhile::Done(x) => x,
         }
     }
 
