@@ -1,14 +1,19 @@
-#[macro_use]
-extern crate defmac;
-extern crate ndarray;
+#![allow(
+    clippy::many_single_char_names,
+    clippy::deref_addrof,
+    clippy::unreadable_literal,
+    clippy::many_single_char_names
+)]
 
+use defmac::defmac;
 use ndarray::prelude::*;
 
 #[test]
 fn test_from_shape_fn() {
     let step = 3.1;
-    let h = Array::from_shape_fn((5, 5),
-        |(i, j)| f64::sin(i as f64 / step) * f64::cos(j as f64  / step));
+    let h = Array::from_shape_fn((5, 5), |(i, j)| {
+        f64::sin(i as f64 / step) * f64::cos(j as f64 / step)
+    });
     assert_eq!(h.shape(), &[5, 5]);
 }
 
@@ -21,20 +26,23 @@ fn test_dimension_zero() {
 }
 
 #[test]
-fn test_rc_into_owned() {
+#[cfg(feature = "approx")]
+fn test_arc_into_owned() {
+    use approx::assert_abs_diff_ne;
+
     let a = Array2::from_elem((5, 5), 1.).into_shared();
     let mut b = a.clone();
     b.fill(0.);
     let mut c = b.into_owned();
     c.fill(2.);
     // test that they are unshared
-    assert!(!a.all_close(&c, 0.01));
+    assert_abs_diff_ne!(a, c, epsilon = 0.01);
 }
 
 #[test]
 fn test_arcarray_thread_safe() {
-    fn is_send<T: Send>(_t: &T) { }
-    fn is_sync<T: Sync>(_t: &T) { }
+    fn is_send<T: Send>(_t: &T) {}
+    fn is_sync<T: Sync>(_t: &T) {}
     let a = Array2::from_elem((5, 5), 1.).into_shared();
 
     is_send(&a);
@@ -47,7 +55,9 @@ fn test_uninit() {
         let mut a = Array::<f32, _>::uninitialized((3, 4).f());
         assert_eq!(a.dim(), (3, 4));
         assert_eq!(a.strides(), &[1, 3]);
-        let b = Array::<f32, _>::linspace(0., 25., a.len()).into_shape(a.dim()).unwrap();
+        let b = Array::<f32, _>::linspace(0., 25., a.len())
+            .into_shape(a.dim())
+            .unwrap();
         a.assign(&b);
         assert_eq!(&a, &b);
         assert_eq!(a.t(), b.t());
