@@ -1,80 +1,103 @@
-#[macro_use]
-extern crate ndarray;
-extern crate itertools;
+#![allow(
+    clippy::many_single_char_names,
+    clippy::deref_addrof,
+    clippy::unreadable_literal,
+    clippy::many_single_char_names,
+    clippy::float_cmp
+)]
 
 use ndarray::prelude::*;
 use ndarray::Zip;
+use std::iter::FromIterator;
 
-use itertools::{assert_equal, cloned, enumerate};
+use itertools::{assert_equal, cloned};
 
 use std::mem::swap;
 
-
 #[test]
 fn test_azip1() {
     let mut a = Array::zeros(62);
     let mut x = 0;
-    azip!(mut a in { *a = x; x += 1; });
+    azip!((a in &mut a) { *a = x; x += 1; });
     assert_equal(cloned(&a), 0..a.len());
 }
 
 #[test]
 fn test_azip2() {
     let mut a = Array::zeros((5, 7));
-    let b = Array::from_shape_fn(a.dim(), |(i, j)| 1. / (i + 2*j) as f32);
-    azip!(mut a, b in { *a = b; });
+    let b = Array::from_shape_fn(a.dim(), |(i, j)| 1. / (i + 2 * j) as f32);
+    azip!((a in &mut a, &b in &b) *a = b);
     assert_eq!(a, b);
 }
 
 #[test]
 fn test_azip2_1() {
     let mut a = Array::zeros((5, 7));
-    let b = Array::from_shape_fn((5, 10), |(i, j)| 1. / (i + 2*j) as f32);
+    let b = Array::from_shape_fn((5, 10), |(i, j)| 1. / (i + 2 * j) as f32);
     let b = b.slice(s![..;-1, 3..]);
-    azip!(mut a, b in { *a = b; });
+    azip!((a in &mut a, &b in &b) *a = b);
     assert_eq!(a, b);
 }
 
 #[test]
 fn test_azip2_3() {
-    let mut b = Array::from_shape_fn((5, 10), |(i, j)| 1. / (i + 2*j) as f32);
+    let mut b = Array::from_shape_fn((5, 10), |(i, j)| 1. / (i + 2 * j) as f32);
     let mut c = Array::from_shape_fn((5, 10), |(i, j)| f32::exp((i + j) as f32));
     let a = b.clone();
-    azip!(mut b, mut c in { swap(b, c) });
+    azip!((b in &mut b, c in &mut c) swap(b, c));
     assert_eq!(a, c);
     assert!(a != b);
 }
 
 #[test]
+fn test_azip_syntax_trailing_comma() {
+    let mut b = Array::<i32, _>::zeros((5, 5));
+    let mut c = Array::<i32, _>::ones((5, 5));
+    let a = b.clone();
+    azip!((b in &mut b, c in &mut c, ) swap(b, c));
+    assert_eq!(a, c);
+    assert!(a != b);
+}
+
+#[test]
+#[cfg(feature = "approx")]
 fn test_azip2_sum() {
+    use approx::assert_abs_diff_eq;
+
     let c = Array::from_shape_fn((5, 10), |(i, j)| f32::exp((i + j) as f32));
     for i in 0..2 {
         let ax = Axis(i);
         let mut b = Array::zeros(c.len_of(ax));
-        azip!(mut b, ref c (c.axis_iter(ax)) in { *b = c.sum() });
-        assert!(b.all_close(&c.sum_axis(Axis(1 - i)), 1e-6));
+        azip!((b in &mut b, c in c.axis_iter(ax)) *b = c.sum());
+        assert_abs_diff_eq!(b, c.sum_axis(Axis(1 - i)), epsilon = 1e-6);
     }
 }
 
 #[test]
+#[cfg(feature = "approx")]
 fn test_azip3_slices() {
+    use approx::assert_abs_diff_eq;
+
     let mut a = [0.; 32];
     let mut b = [0.; 32];
     let mut c = [0.; 32];
-    for (i, elt) in enumerate(&mut b) {
+    for (i, elt) in b.iter_mut().enumerate() {
         *elt = i as f32;
     }
 
-    azip!(mut a (&mut a[..]), b (&b[..]), mut c (&mut c[..]) in {
+    azip!((a in &mut a[..], b in &b[..], c in &mut c[..]) {
         *a += b / 10.;
         *c = a.sin();
     });
     let res = Array::linspace(0., 3.1, 32).mapv_into(f32::sin);
-    assert!(res.all_close(&ArrayView::from(&c), 1e-4));
+    assert_abs_diff_eq!(res, ArrayView::from(&c), epsilon = 1e-4);
 }
 
 #[test]
+#[cfg(feature = "approx")]
 fn test_broadcast() {
+    use approx::assert_abs_diff_eq;
+
     let n = 16;
     let mut a = Array::<f32, _>::zeros((n, n));
     let mut b = Array::<f32, _>::from_elem((1, n), 1.);
@@ -91,7 +114,8 @@ fn test_broadcast() {
             .and_broadcast(&e);
         z.apply(|x, &y, &z, &w| *x = y + z + w);
     }
-    assert!(a.all_close(&(&b + &d + &e), 1e-4));
+    let sum = &b + &d + &e;
+    assert_abs_diff_eq!(a, sum.broadcast((n, n)).unwrap(), epsilon = 1e-4);
 }
 
 #[should_panic]
@@ -100,8 +124,8 @@ fn test_zip_dim_mismatch_1() {
     let mut a = Array::zeros((5, 7));
     let mut d = a.raw_dim();
     d[0] += 1;
-    let b = Array::from_shape_fn(d, |(i, j)| 1. / (i + 2*j) as f32);
-    azip!(mut a, b in { *a = b; });
+    let b = Array::from_shape_fn(d, |(i, j)| 1. / (i + 2 * j) as f32);
+    azip!((a in &mut a, &b in &b) *a = b);
 }
 
 // Test that Zip handles memory layout correctly for
@@ -122,7 +146,7 @@ fn test_contiguous_but_not_c_or_f() {
     let correct_012 = a[[0, 1, 2]] + b[[0, 1, 2]];
 
     let mut ans = Array::zeros(a.dim().f());
-    azip!(mut ans, a, b in { *ans = a + b });
+    azip!((ans in &mut ans, &a in &a, &b in &b) *ans = a + b);
     println!("{:?}", a);
     println!("{:?}", b);
     println!("{:?}", ans);
@@ -131,7 +155,6 @@ fn test_contiguous_but_not_c_or_f() {
     assert_eq!(ans, correct);
 }
 
-
 #[test]
 fn test_clone() {
     let a = Array::from_iter(0..27).into_shape((3, 3, 3)).unwrap();
@@ -157,11 +180,10 @@ fn test_indices_1() {
     }
 
     let mut count = 0;
-    Zip::indexed(&a1)
-        .apply(|i, elt| {
-            count += 1;
-            assert_eq!(*elt, i);
-        });
+    Zip::indexed(&a1).apply(|i, elt| {
+        count += 1;
+        assert_eq!(*elt, i);
+    });
     assert_eq!(count, a1.len());
 
     let mut count = 0;
@@ -188,7 +210,7 @@ fn test_indices_2() {
     }
 
     let mut count = 0;
-    azip!(index i, a1 in {
+    azip!((index i, &a1 in &a1) {
         count += 1;
         assert_eq!(a1, i);
     });
@@ -218,11 +240,10 @@ fn test_indices_3() {
     }
 
     let mut count = 0;
-    Zip::indexed(&a1)
-        .apply(|i, elt| {
-            count += 1;
-            assert_eq!(*elt, i);
-        });
+    Zip::indexed(&a1).apply(|i, elt| {
+        count += 1;
+        assert_eq!(*elt, i);
+    });
     assert_eq!(count, a1.len());
 
     let mut count = 0;
@@ -272,3 +293,22 @@ fn test_indices_split_1() {
         }
     }
 }
+
+#[test]
+fn test_zip_all() {
+    let a = Array::<f32, _>::zeros(62);
+    let b = Array::<f32, _>::ones(62);
+    let mut c = Array::<f32, _>::ones(62);
+    c[5] = 0.0;
+    assert_eq!(true, Zip::from(&a).and(&b).all(|&x, &y| x + y == 1.0));
+    assert_eq!(false, Zip::from(&a).and(&b).all(|&x, &y| x == y));
+    assert_eq!(false, Zip::from(&a).and(&c).all(|&x, &y| x + y == 1.0));
+}
+
+#[test]
+fn test_zip_all_empty_array() {
+    let a = Array::<f32, _>::zeros(0);
+    let b = Array::<f32, _>::ones(0);
+    assert_eq!(true, Zip::from(&a).and(&b).all(|&_x, &_y| true));
+    assert_eq!(true, Zip::from(&a).and(&b).all(|&_x, &_y| false));
+}
